-------------------------------------
--CENTRAL_DISTRIBUCIO
-------------------------------------
CREATE TABLE USR_ENE.CENTRAL_DISTRIBUCIO
(
  COD_C_DISTRIBUCIO  INTEGER                    NOT NULL,
  MAXIM_ENERGIA      NUMBER                     NOT NULL,
  ADREA             VARCHAR2(100 BYTE)         NOT NULL,
  LOCALITAT          VARCHAR2(100 BYTE)         NOT NULL,
  CP                 VARCHAR2(5 BYTE)               NULL,
  PAIS               VARCHAR2(50 BYTE)          NOT NULL,
  ESTAT              VARCHAR2(7 BYTE)           NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_C_DISTRIBUCIO ON USR_ENE.CENTRAL_DISTRIBUCIO
(COD_C_DISTRIBUCIO)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;



-------------------------------------
--CENTRAL_PRODUCCIO
-------------------------------------
CREATE TABLE USR_ENE.CENTRAL_PRODUCCIO
(
  COD_C_PRODUCCIO   INTEGER                     NOT NULL,
  ENERGIA_MAXIMA    NUMBER                          NULL,
  DATA_INSPECCIO    DATE                        NOT NULL,
  CAPACITAT_MAXIMA  NUMBER                      NOT NULL,
  ADREA            VARCHAR2(100 BYTE)          NOT NULL,
  LOCALITAT         VARCHAR2(100 BYTE)          NOT NULL,
  CP                VARCHAR2(5 BYTE)                NULL,
  PAIS              VARCHAR2(50 BYTE)           NOT NULL,
  ESTAT             VARCHAR2(7 BYTE)            NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_C_PRODUCCIO ON USR_ENE.CENTRAL_PRODUCCIO
(COD_C_PRODUCCIO)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;



-------------------------------------
--ALIMENTA_CENTRAL
-------------------------------------
CREATE TABLE USR_ENE.ALIMENTA_CENTRAL
(
  COD_C_DISTRIBUCIO  INTEGER                    NOT NULL,
  COD_LINIA          INTEGER                    NOT NULL,
  COD_C_PRODUCCIO    INTEGER                    NOT NULL,
  ESTAT              VARCHAR2(7 BYTE)           NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_ALIMENTA_CENTRAL ON USR_ENE.ALIMENTA_CENTRAL
(COD_LINIA, COD_C_DISTRIBUCIO)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;


-------------------------------------
--ENERGIA_ABSOLUTA
-------------------------------------
CREATE TABLE USR_ENE.ENERGIA_ABSOLUTA
(
  COD_ENE_ABSOLUTA  INTEGER                     NOT NULL,
  COD_C_PRODUCCIO   INTEGER                     NOT NULL,
  DATA_PROD         DATE                        NOT NULL,
  ENERGIA_PROD      NUMBER                      NOT NULL,
  VALID             VARCHAR2(2 BYTE)            NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_ENE_ABSOLUTA ON USR_ENE.ENERGIA_ABSOLUTA
(COD_ENE_ABSOLUTA)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;


-------------------------------------
--CLIENT
-------------------------------------
CREATE TABLE USR_ENE.CLIENT
(
  COD_CLIENT  INTEGER                           NOT NULL,
  NIF         VARCHAR2(12 BYTE)                 NOT NULL,
  NOM         VARCHAR2(15 BYTE)                 NOT NULL,
  COGNOMS     VARCHAR2(50 BYTE)                 NOT NULL,
  ADREA      VARCHAR2(100 BYTE)                NOT NULL,
  LOCALITAT   VARCHAR2(100 BYTE)                NOT NULL,
  CP          VARCHAR2(5 BYTE)                  NOT NULL,
  PAIS        VARCHAR2(50 BYTE)                 NOT NULL,
  TELEFON     VARCHAR2(20 BYTE)                 NOT NULL,
  ESTAT       VARCHAR2(7 BYTE)                  NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_CLIENT ON USR_ENE.CLIENT
(COD_CLIENT)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;


CREATE UNIQUE INDEX USR_ENE.UK_CLIENT ON USR_ENE.CLIENT
(NIF)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;


-------------------------------------
--LINIA
-------------------------------------
CREATE TABLE USR_ENE.LINIA
(
  COD_LINIA         INTEGER                     NOT NULL,
  CAPACITAT_MAXIMA  NUMBER                      NOT NULL,
  ESTAT             VARCHAR2(8 BYTE)            NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_LINIA ON USR_ENE.LINIA
(COD_LINIA)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;


-------------------------------------
--CARBO
-------------------------------------

CREATE TABLE USR_ENE.CARBO
(
  COD_C_PRODUCCIO  INTEGER                      NOT NULL,
  EMISSIO_CO2      NUMBER                       NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_C_PRODUCCIO_CARBO ON USR_ENE.CARBO
(COD_C_PRODUCCIO)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;

-------------------------------------
--CONTRACTE
-------------------------------------
CREATE TABLE USR_ENE.CONTRACTE
(
  COD_CONTRACTE  INTEGER                        NOT NULL,
  COD_COMPTADOR  INTEGER                        NOT NULL,
  COD_CLIENT     INTEGER                        NOT NULL,
  DATA_ALTA      DATE                           NOT NULL,
  DATA_FI        DATE                               NULL,
  POTENCIA       NUMBER                         NOT NULL,
  ESTAT          VARCHAR2(7 BYTE)               NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_CONTRACTE ON USR_ENE.CONTRACTE
(COD_CONTRACTE)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;

-------------------------------------
--LECTURA_CONSUM
-------------------------------------
CREATE TABLE USR_ENE.LECTURA_CONSUM
(
  COD_LECT_CONSUM  INTEGER                      NOT NULL,
  COD_COMPTADOR    INTEGER                      NOT NULL,
  DATA_LECTURA     DATE                         NOT NULL,
  CONSUM_MES       NUMBER                       NOT NULL,
  LECTURA          NUMBER                       NOT NULL,
  MODALITAT        VARCHAR2(10 BYTE)            NOT NULL,
  ESTAT            VARCHAR2(7 BYTE)             NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_LECT_CONSUM ON USR_ENE.LECTURA_CONSUM
(COD_LECT_CONSUM)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;

-------------------------------------
--MODEL_COMPTADOR
-------------------------------------
CREATE TABLE USR_ENE.MODEL_COMPTADOR
(
  COD_MODEL    INTEGER                          NOT NULL,
  DATA_FAB     DATE                             NOT NULL,
  REFERENCIA   VARCHAR2(20 BYTE)                NOT NULL,
  EMPRESA_FAB  VARCHAR2(100 BYTE)               NOT NULL,
  DESCRIPCIO   VARCHAR2(100 BYTE)               NOT NULL,
  ADREA       VARCHAR2(100 BYTE)               NOT NULL,
  LOCALITAT    VARCHAR2(100 BYTE)               NOT NULL,
  CP           VARCHAR2(5 BYTE)                 NOT NULL,
  PAIS         VARCHAR2(50 BYTE)                NOT NULL,
  TELEFON      VARCHAR2(20 BYTE)                NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_MODEL ON USR_ENE.MODEL_COMPTADOR
(COD_MODEL)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;


CREATE UNIQUE INDEX USR_ENE.UK_REFERENCIA ON USR_ENE.MODEL_COMPTADOR
(REFERENCIA)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;

-------------------------------------
--COMPTADOR
-------------------------------------
CREATE TABLE USR_ENE.COMPTADOR
(
  COD_COMPTADOR   INTEGER                       NOT NULL,
  COD_MODEL       INTEGER                       NOT NULL,
  REFERENCIA      VARCHAR2(20 BYTE)             NOT NULL,
  DATA_INSTAL     DATE                              NULL,
  CONSUM_ABSOLUT  NUMBER                            NULL,
  ADREA          VARCHAR2(100 BYTE)            NOT NULL,
  LOCALITAT       VARCHAR2(100 BYTE)            NOT NULL,
  CP              VARCHAR2(5 BYTE)              NOT NULL,
  PAIS            VARCHAR2(50 BYTE)             NOT NULL,
  ESTAT           VARCHAR2(10 BYTE)             NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_COMPTADOR ON USR_ENE.COMPTADOR
(COD_COMPTADOR)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;


CREATE UNIQUE INDEX USR_ENE.UK_REFERENCIA_COMPT ON USR_ENE.COMPTADOR
(REFERENCIA)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;

-------------------------------------
--CONNECTA_COMPTADOR
-------------------------------------
CREATE TABLE USR_ENE.CONNECTA_COMPTADOR
(
  COD_COMPTADOR      INTEGER                    NOT NULL,
  COD_C_DISTRIBUCIO  INTEGER                    NOT NULL,
  ESTAT              VARCHAR2(7 BYTE)           NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_CONNECTA_COMPTADOR ON USR_ENE.CONNECTA_COMPTADOR
(COD_COMPTADOR, COD_C_DISTRIBUCIO)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;


-------------------------------------
--EOLICA
-------------------------------------
CREATE TABLE USR_ENE.EOLICA
(
  COD_C_PRODUCCIO  INTEGER                      NOT NULL,
  NUM_MOLINS       NUMBER                       NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_C_PRODUCCIO_EOLICA ON USR_ENE.EOLICA
(COD_C_PRODUCCIO)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;



-------------------------------------
--LOG
-------------------------------------
CREATE TABLE USR_ENE.LOG
(
  COD_LOG      INTEGER                          NOT NULL,
  DATA         DATE                             NOT NULL,
  PROCEDIMENT  VARCHAR2(200 BYTE)               NOT NULL,
  PARAM_IN     VARCHAR2(2000 BYTE)              NOT NULL,
  PARAM_OUT    VARCHAR2(2000 BYTE)              NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_LOG ON USR_ENE.LOG
(COD_LOG)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;

-------------------------------------
--NUCLEAR
-------------------------------------
CREATE TABLE USR_ENE.NUCLEAR
(
  COD_C_PRODUCCIO  INTEGER                      NOT NULL,
  ENERGIA_MINIMA   NUMBER                       NOT NULL,
  REBUIG           NUMBER                       NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_C_PRODUCCIO_NUCLEAR ON USR_ENE.NUCLEAR
(COD_C_PRODUCCIO)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;

-------------------------------------
--SOLAR
-------------------------------------
CREATE TABLE USR_ENE.SOLAR
(
  COD_C_PRODUCCIO  INTEGER                      NOT NULL,
  NUM_PANELLS      NUMBER                       NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_C_PRODUCCIO_SOLAR ON USR_ENE.SOLAR
(COD_C_PRODUCCIO)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;


-------------------------------------
--TERMICA
-------------------------------------
CREATE TABLE USR_ENE.TERMICA
(
  COD_C_PRODUCCIO  INTEGER                      NOT NULL,
  EMISSIOCO2       NUMBER                       NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_C_PRODUCCIO_TERMICA ON USR_ENE.TERMICA
(COD_C_PRODUCCIO)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;


-------------------------------------
--EST_CONSUM_COMPTADOR
-------------------------------------
CREATE TABLE USR_ENE.EST_CONSUM_COMPTADOR
(
  COD_C_PRODUCCIO  INTEGER                      NOT NULL,
  CONSUM           NUMBER                       NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_C_PRODUCCIO_ESTCOMCOMPT ON USR_ENE.EST_CONSUM_COMPTADOR
(COD_C_PRODUCCIO)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;


-------------------------------------
--EST_CONSUM_MIG
-------------------------------------
CREATE TABLE USR_ENE.EST_CONSUM_MIG
(
  NIF            VARCHAR2(12 BYTE)              NOT NULL,
  COD_COMPTADOR  INTEGER                        NOT NULL,
  CONSUM_MIG     NUMBER                             NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_CLIENT_ESTCONSMIG ON USR_ENE.EST_CONSUM_MIG
(NIF)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;


-------------------------------------
--EST_LINIA_CARREGADA
-------------------------------------
CREATE TABLE USR_ENE.EST_LINIA_CARREGADA
(
  COD_LINIA  INTEGER                            NOT NULL,
  CARREGA    NUMBER                             NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_LINIA_ESTLINCARREGADA ON USR_ENE.EST_LINIA_CARREGADA
(COD_LINIA)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;

-------------------------------------
--EST_NOMBRE_CENTRAL_PRODUCCIO
-------------------------------------
CREATE TABLE USR_ENE.EST_NOMBRE_CENTRAL_PRODUCCIO
(
  ANYO                   NUMBER                 NOT NULL,
  NUM_CENTRAL_PRODUCCIO  NUMBER                 NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_ANYO_ESTNOMCENTPROD ON USR_ENE.EST_NOMBRE_CENTRAL_PRODUCCIO
(ANYO)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;

-------------------------------------
--EST_PERCENTATGE_LINIES
-------------------------------------
CREATE TABLE USR_ENE.EST_PERCENTATGE_LINIES
(
  ANYO                NUMBER                    NULL,
  PERCENTATGE_LINIES  NUMBER                    NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_ANYO_ESTPERCENTLINIES ON USR_ENE.EST_PERCENTATGE_LINIES
(ANYO)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;

-------------------------------------
--EST_TOP_TEN
-------------------------------------
CREATE TABLE USR_ENE.EST_TOP_TEN
(
  COD_COMPTADOR  INTEGER                            NULL,
  CONSUM         NUMBER                             NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_COMPTADOR_ESTTOPTEN ON USR_ENE.EST_TOP_TEN
(COD_COMPTADOR)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;


-------------------------------------
--EST_VALOR_MITJA
-------------------------------------
CREATE TABLE USR_ENE.EST_VALOR_MITJA
(
  COD_LINIA    INTEGER                          NOT NULL,
  ANYO         NUMBER                           NOT NULL,
  VALOR_MITJA  NUMBER                           NOT NULL
)
TABLESPACE ENERGIA_DAT
LOGGING 
NOCOMPRESS 
NOCACHE
NOPARALLEL
MONITORING;


CREATE UNIQUE INDEX USR_ENE.PK_COD_LINIA_ESTVALMITJA ON USR_ENE.EST_VALOR_MITJA
(COD_LINIA, ANYO)
LOGGING
TABLESPACE ENERGIA_IND
NOPARALLEL;


-----------------------------------
--ALTER TABLES
-----------------------------------


ALTER TABLE USR_ENE.CENTRAL_DISTRIBUCIO ADD (
  CONSTRAINT PK_COD_C_DISTRIBUCIO
 PRIMARY KEY
 (COD_C_DISTRIBUCIO)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.CENTRAL_PRODUCCIO ADD (
  CONSTRAINT PK_COD_C_PRODUCCIO
 PRIMARY KEY
 (COD_C_PRODUCCIO)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.ALIMENTA_CENTRAL ADD (
  CONSTRAINT PK_ALIMENTA_CENTRAL
 PRIMARY KEY
 (COD_LINIA, COD_C_DISTRIBUCIO)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.ALIMENTA_CENTRAL ADD (
  CONSTRAINT FK_COD_C_PRODUCCIO_ALIM_CENT 
 FOREIGN KEY (COD_C_PRODUCCIO) 
 REFERENCES USR_ENE.CENTRAL_PRODUCCIO (COD_C_PRODUCCIO));


ALTER TABLE USR_ENE.ENERGIA_ABSOLUTA ADD (
  CONSTRAINT PK_COD_ENE_ABSOLUTA
 PRIMARY KEY
 (COD_ENE_ABSOLUTA)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.ENERGIA_ABSOLUTA ADD (
  CONSTRAINT FK_COD_C_PRODUCCIO_ENE_ABS 
 FOREIGN KEY (COD_C_PRODUCCIO) 
 REFERENCES USR_ENE.CENTRAL_PRODUCCIO (COD_C_PRODUCCIO));


ALTER TABLE USR_ENE.CLIENT ADD (
  CONSTRAINT PK_COD_CLIENT
 PRIMARY KEY
 (COD_CLIENT)
    USING INDEX 
    TABLESPACE ENERGIA_IND);

ALTER TABLE USR_ENE.CLIENT ADD (
  CONSTRAINT UK_CLIENT
 UNIQUE (NIF)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.CONTRACTE ADD (
  CONSTRAINT FK_COD_CLIENT 
 FOREIGN KEY (COD_CLIENT) 
 REFERENCES USR_ENE.CLIENT (COD_CLIENT));


ALTER TABLE USR_ENE.LINIA ADD (
  CONSTRAINT PK_COD_LINIA
 PRIMARY KEY
 (COD_LINIA)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.CARBO ADD (
  CONSTRAINT PK_COD_C_PRODUCCIO_CARBO
 PRIMARY KEY
 (COD_C_PRODUCCIO)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.CONTRACTE ADD (
  CONSTRAINT PK_COD_CONTRACTE
 PRIMARY KEY
 (COD_CONTRACTE)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.LECTURA_CONSUM ADD (
  CONSTRAINT PK_COD_LECT_CONSUM
 PRIMARY KEY
 (COD_LECT_CONSUM)
    USING INDEX 
    TABLESPACE ENERGIA_IND);



ALTER TABLE USR_ENE.MODEL_COMPTADOR ADD (
  CONSTRAINT PK_COD_MODEL
 PRIMARY KEY
 (COD_MODEL)
    USING INDEX 
    TABLESPACE ENERGIA_IND);

ALTER TABLE USR_ENE.MODEL_COMPTADOR ADD (
  CONSTRAINT UK_REFERENCIA
 UNIQUE (REFERENCIA)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.COMPTADOR ADD (
  CONSTRAINT PK_COD_COMPTADOR
 PRIMARY KEY
 (COD_COMPTADOR)
    USING INDEX 
    TABLESPACE ENERGIA_IND);

ALTER TABLE USR_ENE.COMPTADOR ADD (
  CONSTRAINT UK_REFERENCIA_COMPT
 UNIQUE (REFERENCIA)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.COMPTADOR ADD (
  CONSTRAINT FK_COD_MODEL 
 FOREIGN KEY (COD_MODEL) 
 REFERENCES USR_ENE.MODEL_COMPTADOR (COD_MODEL));


ALTER TABLE USR_ENE.CONTRACTE ADD (
  CONSTRAINT FK_COD_COMPTADOR 
 FOREIGN KEY (COD_COMPTADOR) 
 REFERENCES USR_ENE.COMPTADOR (COD_COMPTADOR));

ALTER TABLE USR_ENE.LECTURA_CONSUM ADD (
  CONSTRAINT FK_COD_COMP_LECTURA_CONSUM 
 FOREIGN KEY (COD_COMPTADOR) 
 REFERENCES USR_ENE.COMPTADOR (COD_COMPTADOR));


ALTER TABLE USR_ENE.CONNECTA_COMPTADOR ADD (
  CONSTRAINT PK_CONNECTA_COMPTADOR
 PRIMARY KEY
 (COD_COMPTADOR, COD_C_DISTRIBUCIO)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.EOLICA ADD (
  CONSTRAINT PK_COD_C_PRODUCCIO_EOLICA
 PRIMARY KEY
 (COD_C_PRODUCCIO)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.LOG ADD (
  CONSTRAINT PK_COD_LOG
 PRIMARY KEY
 (COD_LOG)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.NUCLEAR ADD (
  CONSTRAINT PK_COD_C_PRODUCCIO_NUCLEAR
 PRIMARY KEY
 (COD_C_PRODUCCIO)
    USING INDEX 
    TABLESPACE ENERGIA_IND);



ALTER TABLE USR_ENE.SOLAR ADD (
  CONSTRAINT PK_COD_C_PRODUCCIO_SOLAR
 PRIMARY KEY
 (COD_C_PRODUCCIO)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.TERMICA ADD (
  CONSTRAINT PK_COD_C_PRODUCCIO_TERMICA
 PRIMARY KEY
 (COD_C_PRODUCCIO)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.EST_CONSUM_COMPTADOR ADD (
  CONSTRAINT PK_COD_C_PRODUCCIO_ESTCOMCOMPT
 PRIMARY KEY
 (COD_C_PRODUCCIO)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.EST_CONSUM_MIG ADD (
  CONSTRAINT PK_COD_CLIENT_ESTCONSMIG
 PRIMARY KEY
 (NIF)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.EST_LINIA_CARREGADA ADD (
  CONSTRAINT PK_COD_LINIA_ESTLINCARREGADA
 PRIMARY KEY
 (COD_LINIA)
    USING INDEX 
    TABLESPACE ENERGIA_IND);



ALTER TABLE USR_ENE.EST_NOMBRE_CENTRAL_PRODUCCIO ADD (
  CONSTRAINT PK_ANYO_ESTNOMCENTPROD
 PRIMARY KEY
 (ANYO)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.EST_PERCENTATGE_LINIES ADD (
  CONSTRAINT PK_ANYO_ESTPERCENTLINIES
 PRIMARY KEY
 (ANYO)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.EST_TOP_TEN ADD (
  CONSTRAINT PK_COD_COMPTADOR_ESTTOPTEN
 PRIMARY KEY
 (COD_COMPTADOR)
    USING INDEX 
    TABLESPACE ENERGIA_IND);


ALTER TABLE USR_ENE.EST_VALOR_MITJA ADD (
  CONSTRAINT PK_COD_LINIA_ESTVALMITJA
 PRIMARY KEY
 (COD_LINIA, ANYO)
    USING INDEX 
    TABLESPACE ENERGIA_IND);
