
----------------------------------------------------------------
--ESTADISTICA 1
----------------------------------------------------------------
-- getCodProduccio

CREATE OR REPLACE FUNCTION USR_ENE.getCodProduccio (n_cod_comptador INTEGER)

RETURN INTEGER
IS

  err_msg            VARCHAR2 (100);
  rst                VARCHAR2 (100);
  a_cod_c_produccio  INTEGER;
  
BEGIN

  --Obtenim el codi de la central de produccio a partir del
  --codi de comptador de la taula lectura_consum
  SELECT AC.COD_C_PRODUCCIO into a_cod_c_produccio
  FROM COMPTADOR C, CONNECTA_COMPTADOR CC, ALIMENTA_CENTRAL AC
  WHERE C.COD_COMPTADOR = n_cod_comptador AND
  		C.COD_COMPTADOR = CC.COD_COMPTADOR AND CC.ESTAT = 'actiu' AND
	    CC.COD_C_DISTRIBUCIO = AC.COD_C_DISTRIBUCIO AND AC.ESTAT = 'actiu';
	 

  --Retornem el codi de central de producci
  RETURN a_cod_c_produccio;
  
  
  EXCEPTION
  WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
      p_insertLog ('Func. getCodProduccio',  
             	   'cod_comptador:   ' || n_cod_comptador || ', ' ||
				   'cod_c_produccio:   ' || a_cod_c_produccio, rst);
      ROLLBACK;

END getCodProduccio;
/

--p_altaEstConsumComptador

CREATE OR REPLACE PROCEDURE p_altaEstConsumComptador (
	   n_cod_c_produccio IN INTEGER,
	   n_consum			 IN NUMBER
	   


)
IS

campsBuits             EXCEPTION;
err_msg                VARCHAR2 (100);
rst              	   VARCHAR2(2000);


BEGIN

--Verifiquem que els camps no siguin nuls

     IF (n_cod_c_produccio IS NULL) OR (n_consum IS NULL)  THEN 
     RAISE campsBuits;
   END IF; 

   --Inserim el nou registre
   INSERT INTO EST_CONSUM_COMPTADOR (cod_c_produccio, consum) 
          VALUES (n_cod_c_produccio, n_consum);
   
   -- Fin de les operacions
   rst := 'OK: S''ha donat d''alta el consum';

   p_insertLog('p_altaEstConsumComptador', 
               'n_cod_c_produccio: ' || n_cod_c_produccio ||
               'n_consum: ' || n_consum, rst);
   COMMIT;
   
EXCEPTION

   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps nuls que no haurien de ser nuls';
   p_insertLog('p_altaEstConsumComptador', 
               'n_cod_c_produccio: ' || n_cod_c_produccio ||
               'n_consum: ' || n_consum, rst);
      ROLLBACK;  
	  
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_altaEstConsumComptador', 
               'n_cod_c_produccio: ' || n_cod_c_produccio ||
               'n_consum: ' || n_consum, rst);
      ROLLBACK;

END p_altaEstConsumComptador;
/


--p_modEstConsumComptador

CREATE OR REPLACE PROCEDURE         p_modEstConsumComptador (
   n_cod_c_produccio  IN INTEGER,
      n_consum    IN NUMBER
    


)
IS

campsBuits             EXCEPTION;
err_msg                VARCHAR2 (100);
rst                  VARCHAR2(2000);


BEGIN

--Verifiquem que els camps no siguin nuls

     IF (n_cod_c_produccio IS NULL) OR (n_consum IS NULL)  THEN 
     RAISE campsBuits;
   END IF; 

   --Actualitzem el nou registre
   UPDATE EST_CONSUM_COMPTADOR
   SET consum = (consum + n_consum) 
   WHERE cod_c_produccio = n_cod_c_produccio;
   
   -- Fin de les operacions
   rst := 'OK: S''ha modificat el consum';

   p_insertLog('p_modEstConsumComptador', 
               'n_cod_c_produccio: ' || n_cod_c_produccio ||
               'n_consum: ' || n_consum, rst);
   COMMIT;
   
EXCEPTION

   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps nuls que no haurien de ser nuls';
   p_insertLog('p_modEstConsumComptador', 
               'n_cod_c_produccio: ' || n_cod_c_produccio ||
               'n_consum: ' || n_consum, rst);
      ROLLBACK;  
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modEstConsumComptador', 
               'n_cod_c_produccio: ' || n_cod_c_produccio ||
               'n_consum: ' || n_consum, rst);
      ROLLBACK;


END p_modEstConsumComptador;
/

--p_estConsumComptador

CREATE OR REPLACE PROCEDURE USR_ENE.p_estConsumComptador (
	   	  n_cod_comptador IN INTEGER,
		  n_lectura       IN NUMBER,		  
		  lect		      IN NUMBER



)

IS

a_cod_c_produccio    INTEGER;
existe_codpro        INTEGER;


BEGIN


 --Obtenim el cod_c_produccio que pertany al codi_comptador que donem d'alta la lectura
   a_cod_c_produccio := getCodProduccio (n_cod_comptador);
   
 
  --Omplim la taula EST_CONSUM_COMPTADOR
  --Verifiquem que existeix el cod_c_produccio
  SELECT COUNT (*)
  INTO EXISTE_CODPRO
  FROM EST_CONSUM_COMPTADOR EC
  WHERE EC.COD_C_PRODUCCIO = a_cod_c_produccio;
  
  --Si no existeix
  IF (EXISTE_CODPRO = 0) THEN
  	  --Donem d'alta el consum del comptador a la taula EST_CONSUM_COMPTADOR
  	   p_altaEstConsumComptador (a_cod_c_produccio, (n_lectura - lect));

 	    --si existeix actualitzem la taula EST_CONSUM_COMPTADOR
  ELSE
  	   p_modEstConsumComptador (a_cod_c_produccio, (n_lectura - lect));
  
  END IF;



END p_estConsumComptador;
/


----------------------------------------------------------------
--ESTADISTICA 2
----------------------------------------------------------------
--getFilesByComptadorAndYear

CREATE OR REPLACE FUNCTION USR_ENE.getFilesByComptadorAndYear (n_cod_comptador INTEGER, n_year INTEGER)

RETURN INTEGER
IS

  err_msg VARCHAR2 (100);
  rst     VARCHAR2 (100);
  a_files NUMBER;
  
BEGIN

SELECT COUNT (*) INTO  a_files
FROM (
	 select EXTRACT (MONTH FROM LC.DATA_LECTURA) AS MES, LC.COD_COMPTADOR, EXTRACT (YEAR FROM LC.DATA_LECTURA) AS ANYO
	 from COMPTADOR C, CONNECTA_COMPTADOR CC, ALIMENTA_CENTRAL AC, LECTURA_CONSUM LC
	 WHERE LC.COD_COMPTADOR = n_cod_comptador AND EXTRACT (YEAR FROM LC.DATA_LECTURA) = n_year and
	 	   C.COD_COMPTADOR = CC.COD_COMPTADOR AND
	 	   CC.COD_C_DISTRIBUCIO = AC.COD_C_DISTRIBUCIO AND AC.ESTAT = 'actiu'
     GROUP BY LC.COD_COMPTADOR, EXTRACT (MONTH FROM LC.DATA_LECTURA), EXTRACT (YEAR FROM LC.DATA_LECTURA)) 
A;


RETURN a_files;

EXCEPTION
WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
      p_insertLog ('Func. getFiles ',  
             	   'cod_comptador:   ' || n_cod_comptador || ', ' ||
				   'files:   ' || a_files, rst);
      ROLLBACK;


END getFilesByComptadorAndYear;
/


--p_altaEstValorMitja

CREATE OR REPLACE PROCEDURE  p_altaEstValorMitja IS


err_msg                VARCHAR2 (100);
rst                  VARCHAR2(2000);
cod_linia      INTEGER;
anyo           NUMBER;
valor_mitja      NUMBER;

BEGIN


   --Inserim el nou registre a la taula EST_VALOR_MIJT amb totes les dades ja calculades
   --tenin en compte sempre que la linia introduida a LECTURA_COSUM sempre tigui el camp
   --valid = 'si'
   
   --Como mejora de proyecto se podra optimizar esta query para evitar hacer una recarga
   --de tabla en cada ocasin
   
   DELETE FROM est_valor_mitja;

   insert into est_valor_mitja (cod_linia, anyo, valor_mitja) 
   select a.a_cod_linia cod_linia, a.anyo anyo, sum(mitja) valor_mitja from (
   SELECT AC.COD_LINIA as a_cod_linia, l.COD_COMPTADOR,
    to_char(l.DATA_LECTURA, 'YYYY') anyo,
    round(sum (l.CONSUM_MES)/getFilesByComptadorAndYear(l.COD_COMPTADOR, to_char(l.DATA_LECTURA, 'YYYY')),2) mitja 
    FROM COMPTADOR C, CONNECTA_COMPTADOR CC, ALIMENTA_CENTRAL AC, lectura_consum l
    WHERE 
      C.COD_COMPTADOR = CC.COD_COMPTADOR AND
   CC.COD_C_DISTRIBUCIO = AC.COD_C_DISTRIBUCIO AND AC.ESTAT = 'actiu' and
   l.COD_COMPTADOR = cc.COD_COMPTADOR AND L.ESTAT = 'actiu'
   group by AC.COD_LINIA, l.COD_COMPTADOR, to_char(l.DATA_LECTURA, 'YYYY')
order by AC.COD_LINIA) a
group by a.a_cod_linia, a.anyo
order by a.a_cod_linia;
   
   
   
   -- Fin de les operacions
   rst := 'OK: S''ha donat d''alta el consum mitj d''un any';

   p_insertLog('p_altaEstValorMitja', 
               'cod_linia: ' || cod_linia || ',' ||
               'anyo: ' || anyo || ',' ||
      'valor_mitja: ' || valor_mitja, rst);
   COMMIT;
   
EXCEPTION


   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
  p_insertLog('p_altaEstValorMitja', 
               'cod_linia: ' || cod_linia || ',' ||
               'anyo: ' || anyo || ',' ||
      'valor_mitja: ' || valor_mitja, rst);
  ROLLBACK;
END p_altaEstValorMitja;
/






----------------------------------------------------------------
--ESTADISTICA 3
----------------------------------------------------------------

--p_estLiniaCarregada

CREATE OR REPLACE PROCEDURE         p_estLiniaCarregada IS

err_msg                VARCHAR2 (100);
rst                  VARCHAR2(2000);
cod_linia              INTEGER;
carrega                NUMBER;


BEGIN



   --Inserim el nou registre a la taula EST_VALOR_MITJA
   
   DELETE FROM EST_LINIA_CARREGADA;
   INSERT INTO  EST_LINIA_CARREGADA
    SELECT * FROM
        (SELECT A.COD_LINIA as COD_LINIA , SUM (CONSUM) CARREGA 
        FROM
           (SELECT SUM (LC.CONSUM_MES) CONSUM, LC.COD_COMPTADOR, AC.COD_LINIA
      FROM CONNECTA_COMPTADOR CC, ALIMENTA_CENTRAL AC, LECTURA_CONSUM LC
      WHERE LC.COD_COMPTADOR = CC.COD_COMPTADOR AND   
         CC.ESTAT = 'actiu' AND LC.ESTAT = 'actiu'  AND
         CC.COD_C_DISTRIBUCIO = AC.COD_C_DISTRIBUCIO AND AC.ESTAT = 'actiu'
      GROUP BY LC.COD_COMPTADOR, AC.COD_LINIA
      ORDER BY CONSUM DESC)A
    GROUP BY A.COD_LINIA
             ORDER BY CARREGA DESC)B
       WHERE ROWNUM =1;


 -- Fin de les operacions
   rst := 'OK: S''ha donat d''alta la linia ms carregada';

   p_insertLog('p_estLiniaCarregada ', 
               'cod_linia: ' || cod_linia || ',' ||
               'carrega: ' || carrega , rst);
   COMMIT;
   
EXCEPTION


   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_estLiniaCarregada ', 
               'cod_linia: ' || cod_linia || ',' ||
               'carrega: ' || carrega , rst);
  ROLLBACK;



END p_estLiniaCarregada;
/



----------------------------------------------------------------
--ESTADISTICA 4
----------------------------------------------------------------
--p_altaEstPercentatgeLinies

CREATE OR REPLACE PROCEDURE p_altaEstPercentatgeLinies (
    n_anyo    IN number   


)
IS

campsBuits             EXCEPTION;
err_msg                VARCHAR2 (100);
rst                  VARCHAR2(2000);
n_percentatge_linies   NUMBER;
linies_superen     NUMBER;
linies       NUMBER;

BEGIN



   --Mirem el numero de linies que superen el 50% de l'energia consumida
--tenint en compte la seva capacitat_maxima (aquesta dada ve donada en valors anuals)
SELECT COUNT (*) INTO linies_superen 
FROM (  
      SELECT AC.COD_LINIA,  L.CAPACITAT_MAXIMA, SUM (LC.CONSUM_MES)
      FROM ALIMENTA_CENTRAL AC, LINIA L, CONNECTA_COMPTADOR CC, LECTURA_CONSUM LC
      WHERE AC.COD_LINIA = L.COD_LINIA AND
            AC.COD_C_DISTRIBUCIO = CC.COD_C_DISTRIBUCIO AND
            CC.COD_COMPTADOR = LC.COD_COMPTADOR AND LC.ESTAT = 'actiu' AND
            CC.ESTAT = 'actiu' AND AC.ESTAT = 'actiu' AND EXTRACT (YEAR FROM LC.DATA_LECTURA) = n_anyo
      GROUP BY AC.COD_LINIA, L.CAPACITAT_MAXIMA
      HAVING SUM (LC.CONSUM_MES) >= L.CAPACITAT_MAXIMA * 0.5);



--Mirem el numero total de linies que te la taula LINIES      
   
SELECT COUNT (*) INTO linies
FROM LINIA;



--Fem el caulcul del %

     n_percentatge_linies := round(((linies_superen/linies) * 100),2);
  


--Inserim el nou registre


   INSERT INTO EST_PERCENTATGE_LINIES (anyo, percentatge_linies) 
          VALUES (n_anyo, n_percentatge_linies);
   
   -- Fin de les operacions
   rst := 'OK: S''ha donat d''alta el percentatge';

   p_insertLog('p_altaEstPercentatgeLinies', 
               'anyo: ' || n_anyo ||
               'percentatge_linies: ' || n_percentatge_linies, rst);
   COMMIT;
   
EXCEPTION

   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps nuls que no haurien de ser nuls';
   p_insertLog('p_altaEstPercentatgeLinies', 
               'anyo: ' || n_anyo ||
               'percentatge_linies: ' || n_percentatge_linies, rst);
      ROLLBACK;  
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_altaEstPercentatgeLinies', 
               'anyo: ' || n_anyo ||
               'percentatge_linies: ' || n_percentatge_linies, rst);
      ROLLBACK;


END p_altaEstPercentatgeLinies;
/






-- p_modEstPercentatgeLinies

CREATE OR REPLACE PROCEDURE p_modEstPercentatgeLinies (
    n_anyo   IN number
 
    


)
IS


err_msg                VARCHAR2 (100);
rst                    VARCHAR2(2000);
linies_superen         NUMBER;
linies                 NUMBER;
n_percentatge_linies   NUMBER;

BEGIN

--Mirem el numero de linies que superen el 50% de l'energia consumida
--tenint en compte la seva capacitat_maxima (aquesta dada ve donada en valors anuals)
SELECT COUNT (*) INTO linies_superen 
FROM (  
      SELECT AC.COD_LINIA,  L.CAPACITAT_MAXIMA, SUM (LC.CONSUM_MES)
      FROM ALIMENTA_CENTRAL AC, LINIA L, CONNECTA_COMPTADOR CC, LECTURA_CONSUM LC
      WHERE AC.COD_LINIA = L.COD_LINIA AND
            AC.COD_C_DISTRIBUCIO = CC.COD_C_DISTRIBUCIO AND
            CC.COD_COMPTADOR = LC.COD_COMPTADOR AND LC.ESTAT = 'actiu' AND
            CC.ESTAT = 'actiu' AND AC.ESTAT = 'actiu' AND EXTRACT (YEAR FROM LC.DATA_LECTURA) = n_anyo
      GROUP BY AC.COD_LINIA, L.CAPACITAT_MAXIMA
      HAVING SUM (LC.CONSUM_MES) >= L.CAPACITAT_MAXIMA * 0.5);


--Mirem el numero total de linies que te la taula LINIES      
   
SELECT COUNT (*) INTO linies
FROM LINIA;

--Fem el caulcul del %

     n_percentatge_linies := ((LINIES_SUPEREN/LINIES) * 100);
 
   
   
   
   --Actualitzem el nou registre
   UPDATE EST_PERCENTATGE_LINIES
   SET percentatge_linies =  n_percentatge_linies
   WHERE anyo = n_anyo;
   
   
   
   
   
   -- Fin de les operacions
   rst := 'OK: S''ha modificat el percentatge';

   p_insertLog('p_modEstPercentatgeLinies', 
               'anyo: ' || n_anyo ||
               'percentatge_linies: ' || n_percentatge_linies, rst);
   COMMIT;
   
EXCEPTION

   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modEstPercentatgeLinies', 
               'anyo: ' || n_anyo ||
               'percentatge_linies: ' || n_percentatge_linies, rst);
      ROLLBACK;

END p_modEstPercentatgeLinies;
/


--p_estPercentatgeLinies

CREATE OR REPLACE PROCEDURE         p_estPercentatgeLinies (
    n_anyo IN NUMBER
)

IS


exist_n_anyo  INTEGER;


BEGIN   
 
  --Omplim la taula EST_PERCENTATGE_LINIES
  --Verifiquem que existeix l'any
  SELECT COUNT (*)
  INTO exist_n_anyo
  FROM EST_PERCENTATGE_LINIES EP
  WHERE ep.ANYO = n_anyo;
  
  --Si no existeix
  IF (exist_n_anyo = 0) THEN
     --Donem d'alta el percentatge a la taula EST_PERCENTATGE_LINIES
      p_altaEstPercentatgeLinies (n_anyo);

      --si existeix actualitzem la taula EST_PERCENTATGE_LINIES
  ELSE
      p_modEstPercentatgeLinies (n_anyo);
  
  END IF;




END p_estPercentatgeLinies;
/


----------------------------------------------------------------
--ESTADISTICA 5
----------------------------------------------------------------


--p_altaEstNomCentProduccio

CREATE OR REPLACE PROCEDURE p_altaEstNomCentProduccio (
   n_anyo    IN number   


)
IS

campsBuits                 EXCEPTION;
err_msg                    VARCHAR2 (100);
rst                      VARCHAR2(2000);
n_percentatge_produccio    NUMBER;
cenprod_superen            NUMBER;
central                    NUMBER; 


BEGIN



--Mirem el numero de centrals que generen menys del 30% de la produccio
--tenint en compte la seva capacitat_maxima (aquesta dada ve donada en valors anuals)
SELECT COUNT (*) INTO  cenprod_superen  
FROM   
(SELECT SUM(EM.ENERGIA_PROD), EM.COD_C_PRODUCCIO, CP.CAPACITAT_MAXIMA
FROM ENERGIA_ABSOLUTA EM, CENTRAL_PRODUCCIO CP
WHERE EM.COD_C_PRODUCCIO = CP.COD_C_PRODUCCIO AND
      EXTRACT (YEAR FROM EM.DATA_PROD) = n_anyo AND
   EM.VALID = 'si'
GROUP BY EM.COD_C_PRODUCCIO, CP.CAPACITAT_MAXIMA
HAVING SUM(EM.ENERGIA_PROD) <= CP.CAPACITAT_MAXIMA * 0.3);


--Mirem el numero total de centrals que te la taula CENTRAL_PRODUCCIO      
   

SELECT COUNT (*) INTO central
FROM CENTRAL_PRODUCCIO;

 
   

--Inserim el nou registre
   INSERT INTO EST_NOMBRE_CENTRAL_PRODUCCIO (anyo, num_central_produccio) 
          VALUES (n_anyo, CENPROD_SUPEREN);
   
   -- Fin de les operacions
   rst := 'OK: S''ha donat d''alta el nombre de centrals';

   p_insertLog('p_estNomCentPro', 
               'anyo: ' || n_anyo ||
               'num_central_produccio: ' || cenprod_superen, rst);
   COMMIT;
   
EXCEPTION

   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps nuls que no haurien de ser nuls';
   
  p_insertLog('p_estNomCentPro', 
               'anyo: ' || n_anyo ||
               'num_central_produccio: ' || cenprod_superen, rst);
      ROLLBACK;  
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
  p_insertLog('p_estNomCentPro', 
               'anyo: ' || n_anyo ||
               'num_central_produccio: ' || cenprod_superen, rst);
      ROLLBACK;


END p_altaEstNomCentProduccio;
/




-- p_modEstNomCentProduccio
CREATE OR REPLACE PROCEDURE         p_modEstNomCentProduccio (
   n_anyo                        IN number
)
IS


err_msg                VARCHAR2 (100);
rst                  VARCHAR2(2000);
cenprod_superen        NUMBER;
central                NUMBER;


BEGIN



--Mirem el numero de centrals que generen menys del 30% de la produccio
--tenint en compte la seva capacitat_maxima (aquesta dada ve donada en valors anuals)
SELECT COUNT (*) into  cenprod_superen 
FROM   
(SELECT SUM(EM.ENERGIA_PROD), EM.COD_C_PRODUCCIO, CP.CAPACITAT_MAXIMA
FROM ENERGIA_ABSOLUTA EM, CENTRAL_PRODUCCIO CP
WHERE EM.COD_C_PRODUCCIO = CP.COD_C_PRODUCCIO AND
      EXTRACT (YEAR FROM EM.DATA_PROD) = 2011 AND
   EM.VALID = 'si'
GROUP BY EM.COD_C_PRODUCCIO, CP.CAPACITAT_MAXIMA
HAVING SUM(EM.ENERGIA_PROD) <= CP.CAPACITAT_MAXIMA * 0.3);


--Mirem el numero total de centrals que te la taula CENTRAL_PRODUCCIO      
   

SELECT COUNT (*) INTO central
FROM CENTRAL_PRODUCCIO;

 
   
   
   --Actualitzem el nou registre
   UPDATE EST_NOMBRE_CENTRAL_PRODUCCIO
   SET num_central_produccio =  cenprod_superen
   WHERE anyo = n_anyo;
   
     
   
   
   -- Fin de les operacions
   rst := 'OK: S''ha modificat els numeros de centrals';

   p_insertLog('p_modEstNomCentProduccio', 
               'anyo: ' || n_anyo ||
               'num_central_produccio: ' || cenprod_superen, rst);
   COMMIT;
   
EXCEPTION


   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modEstNomCentProduccio', 
               'anyo: ' || n_anyo ||
               'num_central_produccio: ' || cenprod_superen, rst);
      ROLLBACK;
END p_modEstNomCentProduccio ;
/

--p_estNomCentPro

CREATE OR REPLACE PROCEDURE         p_estNomCentPro  (
    n_anyo IN NUMBER
)

IS


exist_n_anyo          INTEGER;


BEGIN

  
 
  --Omplim la taula EST_CENTRAL_PRODUCCIO
  --Verifiquem que existeix l'any
  SELECT COUNT (*)
  INTO exist_n_anyo
  FROM EST_NOMBRE_CENTRAL_PRODUCCIO EP
  WHERE ep.ANYO = n_anyo;
  
  --Si no existeix
  IF (exist_n_anyo = 0) THEN
     --Donem d'alta el percentatge a la taula EST_PERCENTATGE_LINIES
      p_altaEstNomCentProduccio  (n_anyo);

      --si existeix actualitzem la taula EST_PERCENTATGE_LINIES
  ELSE
      p_modEstNomCentProduccio  (n_anyo);
  
  END IF;


END p_estNomCentPro ;
/



----------------------------------------------------------------
--ESTADISTICA 6
----------------------------------------------------------------

--p_estTopTen

CREATE OR REPLACE PROCEDURE p_estTopTen IS

err_msg                VARCHAR2 (100);
rst                  VARCHAR2(2000);
consum       NUMBER;
cod_comptador     NUMBER;


BEGIN



   --Inserim el nou registre a la taula EST_VALOR_MITJA
   
   DELETE FROM EST_TOP_TEN;
   INSERT INTO  EST_TOP_TEN
    SELECT * FROM (
       SELECT C.COD_COMPTADOR AS COD_COMPTADOR, C.CONSUM_ABSOLUT AS CONSUM
     FROM COMPTADOR C
     ORDER BY C.CONSUM_ABSOLUT DESC) A
          WHERE ROWNUM <= 10;


 -- Fin de les operacions
   rst := 'OK: S''ha actualitzat la taula top_ten';

   p_insertLog('p_estTopTen ', 
               'cod_comptador: ' || COD_COMPTADOR || ',' ||
               'consum: ' || CONSUM , rst);
   COMMIT;
   
EXCEPTION


   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_estTopTen ', 
               'cod_comptador: ' || COD_COMPTADOR || ',' ||
               'consum: ' || CONSUM , rst);
      
  ROLLBACK;

END p_estTopTen;
/





----------------------------------------------------------------
--ESTADISTICA 7
----------------------------------------------------------------

-- getConsByComptDates


CREATE OR REPLACE FUNCTION USR_ENE.getConsByComptDates (
		  N_COD_COMPTADOR INTEGER,
		  N_DATA_ALTA DATE,
		  N_DATA_FI DATE)


RETURN INTEGER IS

CONSUM INTEGER;

BEGIN


	 IF (N_DATA_FI IS NULL) THEN

	 
	   SELECT SUM(LC.CONSUM_MES) INTO CONSUM
	   FROM LECTURA_CONSUM LC
	   WHERE LC.DATA_LECTURA >= N_DATA_ALTA AND
	         LC.COD_COMPTADOR = N_COD_COMPTADOR; 
			 
			

	 ELSE
	   
	  SELECT SUM(LC.CONSUM_MES) INTO CONSUM
	  FROM LECTURA_CONSUM LC
	  WHERE LC.COD_COMPTADOR = N_COD_COMPTADOR AND
	  	 LC.DATA_LECTURA >= N_DATA_ALTA AND
	  	 LC.DATA_LECTURA <= N_DATA_FI; 

	 END IF;
	   
	 RETURN CONSUM;

END getConsByComptDates;
/


-- getMesos

CREATE OR REPLACE FUNCTION USR_ENE.getMesos (
	   	  N_NIF            IN VARCHAR2,
		  N_COD_COMPTADOR  IN INTEGER,
		  N_DATA_FI        IN DATE
		
)


RETURN NUMBER IS

MESOS NUMBER;

BEGIN

	 IF (N_DATA_FI IS NULL) THEN

	 	SELECT round (MONTHS_BETWEEN ((sysdate), CT.DATA_ALTA), 0) INTO MESOS
		FROM CONTRACTE CT, CLIENT CL
		WHERE CT.COD_CLIENT = CL.COD_CLIENT AND
	  		  CL.NIF = N_NIF AND CT.COD_COMPTADOR = N_COD_COMPTADOR;
			  
			 

	 ELSE
	 
	 	 SELECT round (MONTHS_BETWEEN (CT.DATA_FI, CT.DATA_ALTA), 0) INTO MESOS
		 FROM CONTRACTE CT, CLIENT CL
		 WHERE CT.COD_CLIENT = CL.COD_CLIENT AND
	  		   CL.NIF = N_NIF AND CT.COD_COMPTADOR = N_COD_COMPTADOR;
			   
			  
		 
	 END IF;
	 
	 RETURN MESOS; 
		 
		 
END getMesos;
/


--p_estConsumMig

CREATE OR REPLACE PROCEDURE         p_estConsumMig IS

    a_consum   NUMBER;
    a_mesos    NUMBER;
    a_mitja    NUMBER;


BEGIN  


  DECLARE CURSOR c_registros IS

  --Mostrem el NIF del client, el seu cod_contracte, fecha_inici i fecha_fic del contracte
  --i el condi del comptador
  
 SELECT CL.NIF, CT.COD_CONTRACTE, CT.COD_COMPTADOR, CT.DATA_ALTA, CT.DATA_FI
 FROM CONTRACTE CT, CLIENT CL
 WHERE CT.COD_CLIENT = CL.COD_CLIENT;
 
 v_reg c_registros%rowtype;
    
  BEGIN
  DELETE FROM EST_CONSUM_MIG;
   OPEN c_registros;
   LOOP
      FETCH c_registros INTO v_reg;
   exit when c_registros%NOTFOUND;
      

   a_consum := getConsByComptDates (v_reg.COD_COMPTADOR, v_reg.DATA_ALTA, v_reg.DATA_FI); 
   a_mesos := getMesos (v_reg.NIF, v_reg.COD_COMPTADOR, v_reg.DATA_FI);   
  
   
   --Dividim el consum total d'un comptador pels mesos que porta consumint
   --desde que es va donar d'alta el seu contracte fins a la seva data fi o
   --fins la data actual si el contracte encara est vigent.
   
   a_mitja := round ((a_consum/a_mesos),2);   
   
   
         INSERT INTO  EST_CONSUM_MIG (NIF, COD_COMPTADOR, CONSUM_MIG)
   VALUES (v_reg.NIF, v_reg.COD_COMPTADOR, a_mitja);
  
    END LOOP;
           
  CLOSE c_registros;   
 END; 





END p_estConsumMig;
/











