-------------------------------------
--p_altaAlimentaCentral
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_altaAlimentaCentral ( 
   n_cod_c_distribucio   IN INTEGER,
   n_cod_linia           IN INTEGER,
   n_cod_c_produccio     IN INTEGER,
   n_estat               IN VARCHAR2

)
IS
--variables locals-- 
   rst                            VARCHAR2(2000);
   cod_linia_test                 INTEGER;
   cod_c_distribucio_test         INTEGER;
   cod_c_produccio_test           INTEGER;
   err_msg                        VARCHAR2(100);
   campsBuits                     EXCEPTION;
   noExisteix_cod_linia           EXCEPTION;
   noExisteix_cod_c_distribucio   EXCEPTION;
   noExisteix_cod_c_produccio     EXCEPTION;
   errorEscriptura                EXCEPTION;


BEGIN

   --verifiquem que els camps no siguin nuls
   IF (n_cod_c_distribucio IS NULL) OR (n_cod_linia IS NULL) OR (n_cod_c_produccio IS NULL) 
      OR (n_estat IS NULL) THEN
     RAISE campsBuits;
   END IF; 
   
    -- Verifiquem que el cod_linia existeix a la taula LINIA
   
   SELECT COUNT (*)
     INTO cod_linia_test
     FROM LINIA
   WHERE cod_linia = n_cod_linia;

   IF (cod_linia_test = 0)
   THEN
      RAISE noExisteix_cod_linia;
   END IF;

  --verifiquem que no existeix el cod_c_distribucio a la taula CENTRAL_DISTRIBUCIO
  
   SELECT COUNT (*)
     INTO cod_c_distribucio_test
     FROM CENTRAL_DISTRIBUCIO
   WHERE cod_c_distribucio = n_cod_c_distribucio;

   IF (cod_c_distribucio_test = 0)
   THEN
      RAISE noExisteix_cod_c_distribucio;
   END IF;

     --verifiquem que no existeix el cod_c_produccio a la taula CENTRAL_PRODUCCIO
  
   SELECT COUNT (*)
     INTO cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
   WHERE cod_c_produccio = n_cod_c_produccio;

   IF (cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
   
    -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;

   -- inserim la relacio entre centrals de distribuci, la linia i les centrals de producci
  INSERT INTO ALIMENTA_CENTRAL (cod_c_distribucio, cod_linia, cod_c_produccio, estat) 
         VALUES (n_cod_c_distribucio, n_cod_linia, n_cod_c_produccio, lower(n_estat));
   
   -- Fi de les operacions
 
   rst := 'OK: S''ha inserit la relaci correctament';
   
   p_insertLog('p_altaAlimentaCentral',  
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
          'n_cod_linia: ' || n_cod_linia || ', ' ||
           'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
        'n_estat: ' || n_estat , rst);

   COMMIT;
   EXCEPTION
   
      WHEN noExisteix_cod_c_distribucio    
   THEN
      rst := 'ERROR: No existeix el codi de la central de distribucio';
   
   p_insertLog('p_altaAlimentaCentral',  
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
          'n_cod_linia: ' || n_cod_linia || ', ' ||
           'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
        'n_estat: ' || n_estat , rst);



      ROLLBACK; 
   
     WHEN noExisteix_cod_linia
     THEN
     rst := 'ERROR: No existeix el codi de la linia';
   
   p_insertLog('p_altaAlimentaCentral',  
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
          'n_cod_linia: ' || n_cod_linia || ', ' ||
           'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
        'n_estat: ' || n_estat , rst);


      ROLLBACK;
   
   
      
     WHEN noExisteix_cod_c_produccio
     THEN
     rst := 'ERROR: No existeix el codi de central de produccio';
   
   p_insertLog('p_altaAlimentaCentral',  
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
          'n_cod_linia: ' || n_cod_linia || ', ' ||
           'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
        'n_estat: ' || n_estat , rst);

      ROLLBACK;
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
   p_insertLog('p_altaAlimentaCentral',  
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
          'n_cod_linia: ' || n_cod_linia || ', ' ||
           'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
        'n_estat: ' || n_estat , rst);

      ROLLBACK;  
   
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
   p_insertLog('p_altaAlimentaCentral',  
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
          'n_cod_linia: ' || n_cod_linia || ', ' ||
           'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
        'n_estat: ' || n_estat , rst);

      ROLLBACK;  
   
   

   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   p_insertLog('p_altaAlimentaCentral',  
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
          'n_cod_linia: ' || n_cod_linia || ', ' ||
           'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
        'n_estat: ' || n_estat , rst);


   ROLLBACK;

END p_altaAlimentaCentral;
/



-------------------------------------
--p_altaCarbo
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_altaCarbo(

   n_cod_c_produccio    IN INTEGER,
   n_emissio_co2        IN NUMBER
 
)
IS
--variables locals--

   rst                            VARCHAR2(2000);
   err_msg                        VARCHAR2(100);
   cod_c_produccio_test           INTEGER;
   campsBuits                     EXCEPTION;
   noExisteix_cod_c_produccio     EXCEPTION;


BEGIN


   --verifiquem que els camps no siguin nuls
   IF (n_cod_c_produccio IS NULL) OR 
   (n_emissio_co2 IS NULL) 
   THEN
       RAISE campsBuits;
   END IF; 

  -- Verifiquem que el codi_c_produccio existeix a la taula CENTRAL_PRODUCCIO   
   SELECT COUNT (*)
     INTO cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
   WHERE cod_c_produccio = n_cod_c_produccio;

   IF (cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
  
   -- inserim  la central de carbo
   INSERT INTO CARBO  (cod_c_produccio, emissio_co2) 
          VALUES (n_cod_c_produccio, n_emissio_co2);
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit la central carbo correctament';
   
    p_insertLog('p_altaCarbo',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_emissio_co2: ' || n_emissio_co2, rst);
   COMMIT;
   
   EXCEPTION
    
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
   
    p_insertLog('p_altaCarbo',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_emissio_co2: ' || n_emissio_co2, rst);
      ROLLBACK;  
   
   WHEN noExisteix_cod_c_produccio
      THEN
      rst := 'ERROR: El cod_c_produccio no existeix';
   
   
    p_insertLog('p_altaCarbo',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_emissio_co2: ' || n_emissio_co2, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:' || err_msg;
    
   
    p_insertLog('p_altaCarbo',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_emissio_co2: ' || n_emissio_co2, rst);
      ROLLBACK;
   

END p_altaCarbo;
/





-------------------------------------
--p_altaCentralDistribucio
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_altaCentralDistribucio (
   n_maxim_energia       IN NUMBER,
   n_adrea              IN VARCHAR2,
   n_localitat           IN VARCHAR2,
   n_cp                  IN VARCHAR2,
   n_pais                IN VARCHAR2,
   n_estat               IN VARCHAR2
)
IS
--variables locals--
     rst                        VARCHAR2(2000);
     err_msg                    VARCHAR2(100);
     campsBuits                 EXCEPTION;
     errorEscriptura            EXCEPTION;

BEGIN
   --verifiquem que els camps no siguin nuls
   IF (n_maxim_energia IS NULL) OR (n_adrea IS NULL) OR (n_localitat IS NULL) OR (n_pais IS NULL)
      OR (n_estat IS NULL) THEN 
     RAISE campsBuits;
   END IF; 

 -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;
     
   -- inserim el nou client
   INSERT INTO CENTRAL_DISTRIBUCIO (maxim_energia, adrea, localitat, cp, pais, estat) 
          VALUES (n_maxim_energia, n_adrea, n_localitat, n_cp, n_pais, lower(n_estat));
   
   -- Fi de les operacions
 
   rst := 'OK: S''ha donat d''alta la central de distribuci';
   
   p_insertLog('p_altaCentralDistribucio', 'n_maxim_energia: ' || n_maxim_energia || ', ' || 'n_adrea:' || n_adrea || ', ' || 
               'n_localitat:' || n_localitat || ', ' || 'n_cp:' || n_cp || ', ' || 'n_pais' || n_pais || ', ' ||
       'n_estat:' || n_estat, rst);
   COMMIT;
   
EXCEPTION
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps nuls que no haurien de ser nuls';
      p_insertLog('p_altaCentralDistribucio', 'n_maxim_energia: ' || n_maxim_energia || ', ' || 'n_adrea:' || n_adrea || ', ' || 
                  'n_localitat:' || n_localitat || ', ' || 'n_cp:' || n_cp || ', ' || 'n_pais' || n_pais || ', ' ||
         'n_estat:' || n_estat, rst);
      ROLLBACK;  
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
    p_insertLog('p_altaCentralDistribucio', 'n_maxim_energia: ' || n_maxim_energia || ', ' || 'n_adrea:' || n_adrea || ', ' || 
                'n_localitat:' || n_localitat || ', ' || 'n_cp:' || n_cp || ', ' || 'n_pais' || n_pais || ', ' ||
       'n_estat:' || n_estat, rst);

   ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      p_insertLog('p_altaCentralDistribucio', 'n_maxim_energia: ' || n_maxim_energia || ', ' || 'n_adrea:' || n_adrea || ', ' || 
                  'n_localitat:' || n_localitat || ', ' || 'n_cp:' || n_cp || ', ' || 'n_pais' || n_pais || ', ' ||
         'n_estat:' || n_estat, rst);
   ROLLBACK;

END p_altaCentralDistribucio;
/





-------------------------------------
--p_altaCentralProduccio
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_altaCentralProduccio (
   n_energia_maxima     IN NUMBER,
   n_data_inspeccio     IN DATE,
   n_capacitat_maxima   IN NUMBER, 
   n_adrea             IN VARCHAR2,
   n_localitat          IN VARCHAR2,
   n_cp                 IN VARCHAR2,
   n_pais               IN VARCHAR2, 
   n_estat       IN VARCHAR2
)
IS
--variables locals--

   n_adrea_ins                   VARCHAR (100 CHAR);
   n_localitat_ins                VARCHAR (100 CHAR);
   n_cp_ins              VARCHAR (5);
   n_pais_ins           VARCHAR (50 CHAR);
   rst                            VARCHAR2(2000);
   err_msg                        VARCHAR2(100);
   campsBuits                     EXCEPTION;
   noExisteixDataInspeccio        EXCEPTION;
   errorEscriptura      EXCEPTION;


BEGIN

   n_adrea_ins := SUBSTR (n_adrea, 1, 100); 
   n_localitat_ins := SUBSTR (n_localitat, 1, 100);
   n_cp_ins := SUBSTR (n_cp, 1, 5);
   n_pais_ins  := SUBSTR (n_pais, 1, 50);
  


   --verifiquem que els camps no siguin nuls
   IF (n_data_inspeccio IS NULL) OR
   (n_capacitat_maxima IS NULL) OR
   (n_adrea IS NULL) OR
   (n_localitat IS NULL) OR
   (n_pais IS NULL) OR
   (n_estat IS NULL)
   THEN
       RAISE campsBuits;
   END IF; 

     
   -- Verifiquem que la data_inspeccio existeix (no pot ser superior  a la data del sistema(sysdate))
   IF (n_data_inspeccio > sysdate) THEN 
     RAISE noExisteixDataInspeccio;
   END IF; 
  
   -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;
  
   -- inserim  la nova central de producci
   INSERT INTO CENTRAL_PRODUCCIO (energia_maxima, data_inspeccio, capacitat_maxima,  adrea, localitat, cp, pais, estat) 
         VALUES (n_energia_maxima, n_data_inspeccio, n_capacitat_maxima, n_adrea, n_localitat, n_cp, n_pais, lower(n_estat));
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit la central de producci correctament';
   
    p_insertLog('p_altaCentralProduccio',  
             'n_energia_maxima: ' || n_energia_maxima || ', ' ||
    'n_data_inspeccio: ' || n_data_inspeccio || ', ' ||
    'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais || ', ' ||
    'n_estat: ' || n_estat, rst);
   COMMIT;
   
   EXCEPTION
    
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
    p_insertLog('p_altaCentralProduccio',  
             'n_energia_maxima: ' || n_energia_maxima || ', ' ||
    'n_data_inspeccio: ' || n_data_inspeccio || ', ' ||
    'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;  
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
    p_insertLog('p_altaCentralProduccio',  
             'n_energia_maxima: ' || n_energia_maxima || ', ' ||
    'n_data_inspeccio: ' || n_data_inspeccio || ', ' ||
    'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;  
   
   WHEN noExisteixDataInspeccio
      THEN
      rst := 'ERROR: La data_inspeccio introduda no pot ser superior a la data actual';
   
    p_insertLog('p_altaCentralProduccio',  
             'n_energia_maxima: ' || n_energia_maxima || ', ' ||
    'n_data_inspeccio: ' || n_data_inspeccio || ', ' ||
    'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:' || err_msg;
    
    p_insertLog('p_altaCentralProduccio',  
             'n_energia_maxima: ' || n_energia_maxima || ', ' ||
    'n_data_inspeccio: ' || n_data_inspeccio || ', ' ||
    'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;


END p_altaCentralProduccio;
/





-------------------------------------
--p_altaClient
-------------------------------------
CREATE OR REPLACE PROCEDURE p_altaClient (
   n_nif       IN VARCHAR2,
   n_nom       IN VARCHAR2,
   n_cognoms   IN VARCHAR2,
   n_adrea    IN VARCHAR2,
   n_localitat IN VARCHAR2,
   n_cp        IN VARCHAR2,
   n_pais      IN VARCHAR2,
   n_telefon   IN VARCHAR2,
   n_estat     IN VARCHAR
)
IS
--variables locals--
  
     rst                        VARCHAR2(2000);
     nifClient_test             VARCHAR2(20);
     err_msg                    VARCHAR2(100);
     existeix_client            EXCEPTION;
     campsBuits                 EXCEPTION;
     errorEscriptura            EXCEPTION;
	 no_actiu                   EXCEPTION;

BEGIN
   --verifiquem que els camps no siguin nuls
   IF (n_nif IS NULL) OR (n_nom IS NULL) OR (n_cognoms IS NULL) OR (n_adrea IS NULL) OR (n_localitat IS NULL) OR 
      (n_cp IS NULL) OR (n_pais IS NULL) OR (n_telefon IS NULL) OR (n_estat IS NULL) THEN 
     RAISE campsBuits;
   END IF; 

  --verifiquem que no existeix el client
  SELECT COUNT (*)
     INTO nifClient_test
     FROM CLIENT
    WHERE nif = n_nif;

   IF (nifClient_test > 0)
   THEN
      RAISE existeix_client;
   END IF;
   
  
   -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;

     
   -- inserim el nou client
   INSERT INTO CLIENT (nif, nom, cognoms, adrea, localitat, cp, pais, telefon, estat) 
          VALUES (n_nif, n_nom, n_cognoms, n_adrea, n_localitat, n_cp, n_pais, n_telefon, lower(n_estat));
   
   -- Fi de les operacions
 
   rst := 'OK: S''ha donat d''alta el client';
   
   p_insertLog('p_altaClient', 'n_nif: ' || n_nif || ', ' ||
                               'n_nom: ' || n_nom || ', ' || 
                               'n_cognoms: ' || n_cognoms || ', ' ||  
          'n_adrea:' || n_adrea || ', ' || 
          'n_localitat:' || n_localitat || ', ' || 
                               'n_cp:' || n_cp || ', ' ||  
          'n_pais' || n_pais || ', ' || 
          'n_telefon:' || n_telefon || ', ' ||
          'n_estat: ' || n_estat, rst);
   COMMIT;
   
EXCEPTION
   WHEN existeix_client
   THEN
    rst := 'ERROR: El client ja existeix';
   p_insertLog('p_altaClient', 'n_nif: ' || n_nif || ', ' ||
                               'n_nom: ' || n_nom || ', ' || 
                               'n_cognoms: ' || n_cognoms || ', ' ||  
          'n_adrea:' || n_adrea || ', ' || 
          'n_localitat:' || n_localitat || ', ' || 
                               'n_cp:' || n_cp || ', ' ||  
          'n_pais' || n_pais || ', ' || 
          'n_telefon:' || n_telefon || ', ' ||
          'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
 
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
   p_insertLog('p_altaClient', 'n_nif: ' || n_nif || ', ' ||
                               'n_nom: ' || n_nom || ', ' || 
                               'n_cognoms: ' || n_cognoms || ', ' ||  
          'n_adrea:' || n_adrea || ', ' || 
          'n_localitat:' || n_localitat || ', ' || 
                               'n_cp:' || n_cp || ', ' ||  
          'n_pais' || n_pais || ', ' || 
          'n_telefon:' || n_telefon || ', ' ||
          'n_estat: ' || n_estat, rst);

   ROLLBACK;
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: hi ha camps buits que no poden ser nuls';
   p_insertLog('p_altaClient', 'n_nif: ' || n_nif || ', ' ||
                               'n_nom: ' || n_nom || ', ' || 
                               'n_cognoms: ' || n_cognoms || ', ' ||  
          'n_adrea:' || n_adrea || ', ' || 
          'n_localitat:' || n_localitat || ', ' || 
                               'n_cp:' || n_cp || ', ' ||  
          'n_pais' || n_pais || ', ' || 
          'n_telefon:' || n_telefon || ', ' ||
          'n_estat: ' || n_estat, rst);
      ROLLBACK;  
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   p_insertLog('p_altaClient', 'n_nif: ' || n_nif || ', ' ||
                               'n_nom: ' || n_nom || ', ' || 
                               'n_cognoms: ' || n_cognoms || ', ' ||  
          'n_adrea:' || n_adrea || ', ' || 
          'n_localitat:' || n_localitat || ', ' || 
                               'n_cp:' || n_cp || ', ' ||  
          'n_pais' || n_pais || ', ' || 
          'n_telefon:' || n_telefon || ', ' ||
          'n_estat: ' || n_estat, rst);
 
      ROLLBACK;
END p_altaClient;
/





-------------------------------------
--p_altaComptador
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_altaComptador (
   n_cod_model       IN INTEGER,
   n_referencia      IN VARCHAR2,
   n_data_instal     IN DATE,
   n_consum_absolut  IN NUMBER,
   n_adrea          IN VARCHAR2,
   n_localitat       IN VARCHAR2, 
   n_cp              IN VARCHAR2,
   n_pais            IN VARCHAR2,
   n_estat           IN VARCHAR2
)
IS
--variables locals--

   rst                        VARCHAR2(2000);
   referencia_test            VARCHAR2(20);
   cod_model_test             INTEGER;
   err_msg                    VARCHAR2(100);
   existeix_referencia        EXCEPTION;
   campsBuits                 EXCEPTION;
   noExisteix_cod_model       EXCEPTION;
   noExisteixData             EXCEPTION;
   errorEscriptura            EXCEPTION;

BEGIN

   --verifiquem que els camps no siguin nuls
   IF (n_cod_model IS NULL) OR (n_referencia IS NULL) OR (n_adrea IS NULL) 
   OR (n_localitat IS NULL) OR (n_cp IS NULL) OR (n_pais IS NULL) OR (n_estat IS NULL) THEN
     RAISE campsBuits;
   END IF; 
   
    -- Verifiquem que el codi_model existeix a la taula MODEL_COMPTADOR
   
   SELECT COUNT (*)
     INTO cod_model_test
     FROM MODEL_COMPTADOR
   WHERE cod_model = n_cod_model;

   IF (cod_model_test = 0)
   THEN
      RAISE noExisteix_cod_model;
   END IF;

  --verifiquem que no existeix la referencia de comptador
    SELECT COUNT (*)
     INTO referencia_test 
     FROM COMPTADOR
    WHERE referencia = n_referencia;

   IF (referencia_test > 0)
   THEN
      RAISE existeix_referencia;
   END IF;

  
   -- Verifiquem que la data existeix (no pot ser superior  a la data del sistema(sysdate))
  IF (n_data_instal > sysdate) THEN 
    RAISE noExisteixData;
  END IF; 
  
   -- Verifiquem que el camp estat est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;
  
   -- inserim el nou comptador
  INSERT INTO COMPTADOR (cod_model, referencia, data_instal, consum_absolut, adrea, localitat, cp, pais, estat) 
         VALUES (n_cod_model, n_referencia, n_data_instal, n_consum_absolut, n_adrea, n_localitat, n_cp, n_pais, lower (n_estat));
   
   
   
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit comptador correctament';
   
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
                'n_data_instal: ' || n_data_instal || ', ' ||
                'n_consum_absolut: ' || n_consum_absolut || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
   COMMIT;
   EXCEPTION
   
      WHEN noExisteix_cod_model    
   THEN
      rst := 'ERROR: No existeix el codi del model';
   
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
                'n_data_instal: ' || n_data_instal || ', ' ||
                'n_consum_absolut: ' || n_consum_absolut || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);

      ROLLBACK; 
   
     WHEN existeix_referencia
     THEN
     rst := 'ERROR: La referencia del comptador ja existeix';
   
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
                'n_data_instal: ' || n_data_instal || ', ' ||
                'n_consum_absolut: ' || n_consum_absolut || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
      ROLLBACK;
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
                'n_data_instal: ' || n_data_instal || ', ' ||
                'n_consum_absolut: ' || n_consum_absolut || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
      ROLLBACK;  
   
   WHEN noExisteixData
   THEN
      rst := 'ERROR: La data d''installaci introduda no pot ser superior a la data actual';
   
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
                'n_data_instal: ' || n_data_instal || ', ' ||
                'n_consum_absolut: ' || n_consum_absolut || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);

      ROLLBACK;
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp estat no s''ha escrit correctament';
   
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
                'n_data_instal: ' || n_data_instal || ', ' ||
                'n_consum_absolut: ' || n_consum_absolut || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);

      ROLLBACK;
   
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
    
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
                'n_data_instal: ' || n_data_instal || ', ' ||
                'n_consum_absolut: ' || n_consum_absolut || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
      ROLLBACK;

END p_altaComptador;
/





-------------------------------------
--p_altaConnectaComptador
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_altaConnectaComptador (

   n_cod_comptador         IN INTEGER,
   n_cod_c_distribucio     IN INTEGER,
   n_estat       IN VARCHAR2

)
IS
--variables locals-- 
   rst                            VARCHAR2(2000);
   cod_comptador_test             INTEGER;
   cod_c_distribucio_test         INTEGER;
   err_msg                        VARCHAR2(100);
   campsBuits                     EXCEPTION;
   noExisteix_cod_comptador       EXCEPTION;
   noExisteix_cod_c_distribucio   EXCEPTION;
   errorEscriptura      EXCEPTION;


BEGIN

   --verifiquem que els camps no siguin nuls
   IF (n_cod_comptador IS NULL) OR (n_cod_c_distribucio IS NULL) THEN
     RAISE campsBuits;
   END IF; 
   
    -- Verifiquem que el codi_comptador existeix a la taula COMPTADOR
   
   SELECT COUNT (*)
     INTO cod_comptador_test
     FROM COMPTADOR
   WHERE cod_comptador = n_cod_comptador;

   IF (cod_comptador_test = 0)
   THEN
      RAISE noExisteix_cod_comptador;
   END IF;

  --verifiquem que no existeix el cod_c_distribucio a la taula CENTRAL_DISTRIBUCIO
  
   SELECT COUNT (*)
     INTO cod_c_distribucio_test
     FROM CENTRAL_DISTRIBUCIO
   WHERE cod_c_distribucio = n_cod_c_distribucio;

   IF (cod_c_distribucio_test = 0)
   THEN
      RAISE noExisteix_cod_c_distribucio;
   END IF;
   
   -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;


   -- inserim la relacio entre centrals de distribuci i comptadors
  INSERT INTO CONNECTA_COMPTADOR (cod_comptador, cod_c_distribucio, estat) 
         VALUES (n_cod_comptador, n_cod_c_distribucio, n_estat);
   
   -- Fi de les operacions
 
   rst := 'OK: S''ha inserit la relaci correctament';
   
   p_insertLog('p_altaConnectaComptador',  
               'n_cod_comptador: ' || n_cod_comptador || ', ' ||
         'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
      'n_estat: ' || n_estat, rst);

   COMMIT;
   EXCEPTION
   
      WHEN noExisteix_cod_comptador    
   THEN
      rst := 'ERROR: No existeix el codi del comptador';
   
   p_insertLog('p_altaConnectaComptador',  
               'n_cod_comptador: ' || n_cod_comptador || ', ' ||
         'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
      'n_estat: ' || n_estat, rst);


      ROLLBACK; 
   
     WHEN noExisteix_cod_c_distribucio
     THEN
     rst := 'ERROR: No existeix el codi de la central de distribuci';
   
   p_insertLog('p_altaConnectaComptador',  
               'n_cod_comptador: ' || n_cod_comptador || ', ' ||
         'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
      'n_estat: ' || n_estat, rst);


      ROLLBACK;
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
   p_insertLog('p_altaConnectaComptador',  
               'n_cod_comptador: ' || n_cod_comptador || ', ' ||
         'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
      'n_estat: ' || n_estat, rst);


      ROLLBACK;  
   
 
   WHEN errorEscriptura
   THEN
        rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
   p_insertLog('p_altaConnectaComptador',  
               'n_cod_comptador: ' || n_cod_comptador || ', ' ||
         'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
      'n_estat: ' || n_estat, rst);


      ROLLBACK;    
   
 
   

   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   p_insertLog('p_altaConnectaComptador',  
               'n_cod_comptador: ' || n_cod_comptador || ', ' ||
         'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
      'n_estat: ' || n_estat, rst);

   ROLLBACK;


END p_altaConnectaComptador;
/




-------------------------------------
--p_altaContracte
-------------------------------------
CREATE OR REPLACE PROCEDURE p_altaContracte (
   n_cod_comptador   IN INTEGER,
   n_cod_client      IN INTEGER,
   n_data_alta       IN DATE,
   n_data_fi         IN DATE,
   n_potencia        IN NUMBER,
   n_estat    IN VARCHAR2
)
IS
--variables locals--

   rst                            VARCHAR2(2000);
   codi_comptador_test            INTEGER;
   codi_client_test               INTEGER;
   cod_comptador_test			  INTEGER;
   err_msg                        VARCHAR2(100);
   campsBuits                     EXCEPTION;
   noExisteix_cod_comptador       EXCEPTION;
   noExisteix_cod_client          EXCEPTION;
   noExisteixDataAlta             EXCEPTION;
   errorEscriptura                EXCEPTION;
   noDonarAlta					  EXCEPTION;


BEGIN

   --verifiquem que els camps no siguin nuls
   IF (n_cod_comptador IS NULL) OR 
   (n_cod_client IS NULL) OR
   (n_data_alta IS NULL) OR
   (n_potencia IS NULL) OR
   (n_estat IS NULL)
   THEN
       RAISE campsBuits;
   END IF; 

   -- Verifiquem que el codi_comptador existeix a la taula COMPTADOR   
   SELECT COUNT (*)
     INTO codi_comptador_test
     FROM COMPTADOR
   WHERE cod_comptador = n_cod_comptador;

   IF (codi_comptador_test = 0)
   THEN
      RAISE noExisteix_cod_comptador;
   END IF;
   
     -- Verifiquem que el codi_client existeix a la taula CLIENT  
   SELECT COUNT (*)
     INTO codi_client_test
     FROM CLIENT
   WHERE cod_client = n_cod_client;

   IF (codi_client_test = 0)
   THEN
      RAISE noExisteix_cod_client;
   END IF;
   
 
   -- Verifiquem que la data d'alta existeix (no pot ser superior  a la data del sistema(sysdate))
   IF (n_data_alta > sysdate) THEN 
     RAISE noExisteixDataAlta;
   END IF; 
   
   -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;
  
   
   -- No es pot assignar un comptador a ms d'un contracte actiu
    SELECT COUNT (*)
     INTO cod_comptador_test
     FROM CONTRACTE
     WHERE cod_comptador = n_cod_comptador AND
	       n_estat = 'actiu';

   IF (cod_comptador_test > 0) 
   THEN
      RAISE noDonarAlta;
   END IF;
   
  
   -- inserim el nou contracte
   INSERT INTO CONTRACTE (cod_comptador, cod_client, data_alta, data_fi, potencia, estat) 
         VALUES (n_cod_comptador, n_cod_client, n_data_alta, n_data_fi, n_potencia, lower(n_estat));
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit contracte correctament';
   
    p_insertLog('p_altaContracte',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
       'n_cod_client: ' || n_cod_client || ', ' ||
                'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
       'n_potencia: ' || n_potencia || ', ' ||
    'n_estat: ' || n_estat, rst);
   COMMIT;
   
  --- La data_alta d'un contracte ser la mateixa que la data_instal d'un comptador. 
  -- Actualitzem aquesta dada en la taula comptador.
  UPDATE COMPTADOR
  SET data_instal = n_data_alta
  where cod_comptador = n_cod_comptador;
    
   

   EXCEPTION
     WHEN noExisteix_cod_comptador
     THEN
     rst := 'ERROR: No exiteix el codi del comptador';
   
    p_insertLog('p_altaContracte',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
       'n_cod_client: ' || n_cod_client || ', ' ||
                'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
       'n_potencia: ' || n_potencia || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
  
  
       WHEN noDonarAlta
     THEN
     rst := 'ERROR: Un comptador no pot tenir ms d''un contracte';
   
    p_insertLog('p_altaContracte',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
       'n_cod_client: ' || n_cod_client || ', ' ||
                'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
       'n_potencia: ' || n_potencia || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
  
  
  
   WHEN noExisteix_cod_client
     THEN
     rst := 'ERROR: No exiteix el codi del client';
   
    p_insertLog('p_altaContracte',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
       'n_cod_client: ' || n_cod_client || ', ' ||
                'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
       'n_potencia: ' || n_potencia || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
    p_insertLog('p_altaContracte',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
       'n_cod_client: ' || n_cod_client || ', ' ||
                'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
       'n_potencia: ' || n_potencia || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;  
   
   WHEN noExisteixDataAlta
   THEN
      rst := 'ERROR: La data_alta introduda no pot ser superior a la data actual';
   
    p_insertLog('p_altaContracte',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
       'n_cod_client: ' || n_cod_client || ', ' ||
                'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
       'n_potencia: ' || n_potencia || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
      WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
    p_insertLog('p_altaContracte',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
       'n_cod_client: ' || n_cod_client || ', ' ||
                'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
       'n_potencia: ' || n_potencia || ', ' ||
    'n_estat: ' || n_estat, rst);

   ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:' || err_msg;
    
    p_insertLog('p_altaContracte',  
                'err_msg: ' || err_msg, rst);
      ROLLBACK;




END p_altaContracte;
/




-------------------------------------
--p_altaEnergiaAbsoluta
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_altaEnergiaAbsoluta(
    n_cod_c_produccio IN INTEGER,
    n_data_prod       IN DATE,
    n_energia_prod    IN NUMBER,
 n_valid     IN VARCHAR

)
IS

--Variables locals

   rst                            VARCHAR2(2000);
   cod_c_produccio_test           INTEGER;
   err_msg                        VARCHAR2(100);
   existe                         INTEGER;
   n_data             DATE;
   eprod        NUMBER;
   n_anyo        NUMBER;
   campsBuits                     EXCEPTION;
   noExisteix_cod_c_produccio     EXCEPTION;
   noExisteixDataProd             EXCEPTION;
   datesDuplicades        EXCEPTION;
   errorValid       EXCEPTION;


BEGIN

 
   --verifiquem que els camps no siguin nuls
   IF (n_cod_c_produccio IS NULL) OR 
   (n_data_prod IS NULL) OR
   (n_energia_prod IS NULL) OR
   (n_valid IS NULL)
   THEN
       RAISE campsBuits;
   END IF; 

   -- Verifiquem que el cod_c_produccio existeix a la taula CENTRAL_PRODUCCIO  
   SELECT COUNT (*)
     INTO cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
   WHERE cod_c_produccio = n_cod_c_produccio;

   IF (cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
     
   -- Verifiquem que el camp 'valid' est escrit correctament: si o no

   IF LOWER (n_valid) NOT IN ('si', 'no') THEN
      RAISE errorValid;
   END IF;
   
   
   -- Verifiquem que la data_consum existeix (no pot ser superior  a la data del sistema(sysdate))
   IF (n_data_prod > sysdate) THEN 
     RAISE noExisteixDataProd;
   END IF; 
   
   -- No es pot introdur la mateixa data_prod pel mateix codi_c_produccio

   SELECT COUNT (*)
     INTO existe
     FROM ENERGIA_ABSOLUTA
   WHERE cod_c_produccio = n_cod_c_produccio AND data_prod = n_data_prod;

   IF (existe = 1)
   THEN
      RAISE datesDuplicades;
   END IF;
   
   

   -- inserim el registre
   INSERT INTO ENERGIA_ABSOLUTA (cod_c_produccio, data_prod, energia_prod, valid) 
         VALUES (n_cod_c_produccio, n_data_prod, n_energia_prod, lower (n_valid));
   
   
  ---------------------------------------------------------------------------------------
   --EST_NOMBRE_CENRAL_PRODUCCIO 
  
  --Obtenim el l'any de la prod de la taula energia_absoluta per EST_NOMBRE_CENRAL_PRODUCCIO 
  SELECT EXTRACT (YEAR FROM EM.DATA_PROD) into  n_anyo
  FROM ENERGIA_ABSOLUTA EM 
  WHERE EM.DATA_PROD =  n_data_prod  
  GROUP BY EM.DATA_PROD;

 
  --Cridem al procediment qu far tots els calculs de l'estadistica EST_PERCENTATGE_LINIES
 
       p_estNomCentPro (n_anyo);
 ----------------------------------------------------------------------------------------
   
   
   
   
   
   
   
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit el registre correctament';
   
    p_insertLog('p_altaEnergiaAbsoluta',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
       'n_data_prod: ' || n_data_prod || ', ' ||
                'n_energia_prod: ' || n_energia_prod || ', ' ||
    'n_valid: ' || n_valid, rst);
   COMMIT;
   
    
 
 -- Actualitzem el camp energia_maxima de la taula central_produccio
 --aquest camp ser la suma de les energies prduides al llarg de la vida de la central
 
 SELECT SUM (ENERGIA_PROD) INTO EPROD
 FROM ENERGIA_ABSOLUTA
 WHERE cod_c_produccio = n_cod_c_produccio and
       valid = 'si';
 
 UPDATE CENTRAL_PRODUCCIO
 SET ENERGIA_MAXIMA = EPROD
 WHERE cod_c_produccio = n_cod_c_produccio;
 
 
 
 
   
   EXCEPTION
     WHEN noExisteix_cod_c_produccio
     THEN
     rst := 'ERROR: No exiteix el codi de la central de produccio';
   
    p_insertLog('p_altaEnergiaAbsoluta',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
       'n_data_prod: ' || n_data_prod || ', ' ||
                'n_energia_prod: ' || n_energia_prod || ', ' ||
    'n_valid: ' || n_valid, rst);
      ROLLBACK;
  
  WHEN errorValid
  THEN
  rst := 'ERROR: El camp ''valid'' no s''ha informat correctament';
   
    p_insertLog('p_altaEnergiaAbsoluta',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
       'n_data_prod: ' || n_data_prod || ', ' ||
                'n_energia_prod: ' || n_energia_prod || ', ' ||
    'n_valid: ' || n_valid, rst);
      ROLLBACK;
  
  
  
  
     
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
    p_insertLog('p_altaEnergiaAbsoluta',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
       'n_data_prod: ' || n_data_prod || ', ' ||
                'n_energia_prod: ' || n_energia_prod || ', ' ||
    'n_valid: ' || n_valid, rst);
      ROLLBACK;  
   
   WHEN noExisteixDataProd
   THEN
      rst := 'ERROR: La data_prod introduda no pot ser superior a la data actual';
   
    p_insertLog('p_altaEnergiaAbsoluta',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
       'n_data_prod: ' || n_data_prod || ', ' ||
                'n_energia_prod: ' || n_energia_prod || ', ' ||
    'n_valid: ' || n_valid, rst);
      ROLLBACK;
   
  WHEN datesDuplicades
 THEN
     rst := 'ERROR: No es poden donar d''alta dos data_prod pel matix cod_c_produccio';
   
    p_insertLog('p_altaEnergiaAbsoluta',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
       'n_data_prod: ' || n_data_prod || ', ' ||
                'n_energia_prod: ' || n_energia_prod || ', ' ||
    'n_valid: ' || n_valid, rst);
    ROLLBACK;
 
 
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:' || err_msg;
    
    p_insertLog('p_altaEnergiaAbsoluta',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
       'n_data_prod: ' || n_data_prod || ', ' ||
                'n_energia_prod: ' || n_energia_prod || ', ' ||
    'n_valid: ' || n_valid, rst);
      ROLLBACK;


END p_altaEnergiaAbsoluta;
/





-------------------------------------
--p_altaEolica
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_altaEolica (
   n_cod_c_produccio   IN INTEGER,
   n_num_molins        IN NUMBER
 
)
IS
--variables locals--

   rst                            VARCHAR2(2000);
   err_msg                        VARCHAR2(100);
   cod_c_produccio_test           INTEGER;
   campsBuits                     EXCEPTION;
   noExisteix_cod_c_produccio     EXCEPTION;


BEGIN


   --verifiquem que els camps no siguin nuls
   IF (n_cod_c_produccio IS NULL) OR 
   (n_num_molins IS NULL) 
   THEN
       RAISE campsBuits;
   END IF; 

  -- Verifiquem que el codi_c_produccio existeix a la taula CENTRAL_PRODUCCIO   
   SELECT COUNT (*)
     INTO cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
   WHERE cod_c_produccio = n_cod_c_produccio;

   IF (cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
  
   -- inserim  la central eolica
   INSERT INTO EOLICA (cod_c_produccio, num_molins) 
          VALUES (n_cod_c_produccio, n_num_molins);
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit la central eolica correctament';
   
    p_insertLog('p_altaEolica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_num_molins: ' || n_num_molins, rst);
   COMMIT;
   
   EXCEPTION
    
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
   
    p_insertLog('p_altaEolica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_num_molins: ' || n_num_molins, rst);
      ROLLBACK;  
   
   WHEN noExisteix_cod_c_produccio
      THEN
      rst := 'ERROR: El cod_c_produccio no existeix';
   
   
    p_insertLog('p_altaEolica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_num_molins: ' || n_num_molins, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:' || err_msg;
    
    p_insertLog('p_altaEolica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_num_molins: ' || n_num_molins, rst);
      ROLLBACK;
   


END p_altaEolica;
/




-------------------------------------
--p_altaLecturaConsum
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_altaLecturaConsum (
    n_cod_comptador  IN INTEGER,
    n_data_lectura   IN DATE,
    n_consum_mes     IN NUMBER,
    n_lectura        IN NUMBER,
    n_modalitat      IN VARCHAR2,
  n_estat          IN VARCHAR2
    
)
IS

--variables locals--

   rst                      VARCHAR2(2000);
   cod_comptador_test          INTEGER;
   err_msg                     VARCHAR2(100);
   lect                NUMBER;
   existe             INTEGER;
   n_data             DATE;
   n_data_duplicada            DATE;
   n_data_alta               DATE;
   compt_consum_abs      NUMBER;
   estat_comptador      VARCHAR2(10);
   n_anyo                      number; 
   campsBuits                  EXCEPTION;
   noExisteix_cod_comptador    EXCEPTION;
   noExisteixDataAlta          EXCEPTION;  
   errorEscriptura             EXCEPTION;
   datesDuplicades             EXCEPTION;
   dataLectura                 EXCEPTION;
   errorEscriptura2            EXCEPTION;
   noLectura       EXCEPTION;
   
   

BEGIN

--Verifiquem que els camps no siguin nuls

   IF (n_cod_comptador IS NULL) OR (n_data_lectura IS NULL) OR (n_lectura IS NULL)
      OR (n_modalitat IS NULL) OR (n_estat IS NULL) THEN 
     RAISE campsBuits;
   END IF; 

   
-- Verifiquem que el codi_comptador existeix a la taula COMPTADOR
   
   SELECT COUNT (*)
     INTO cod_comptador_test
     FROM COMPTADOR
   WHERE cod_comptador = n_cod_comptador;

   IF (cod_comptador_test = 0)
   THEN
      RAISE noExisteix_cod_comptador;
   END IF;
   
 
-- Verifiquem que el camp 'modalitat' est escrit correctament: telematica o presencial

   IF LOWER (n_modalitat) NOT IN ('telematica', 'presencial') THEN
      RAISE errorEscriptura;
   END IF;
   
 -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura2;
   END IF;
   
   
-- No es pot introdur una lectura que sigui anterior a al data d'alta de contracte

SELECT C.DATA_ALTA into n_data_alta
FROM CONTRACTE C   
WHERE cod_comptador=n_cod_comptador;

IF (n_data_lectura < n_data_alta) THEN
RAISE dataLectura;
END IF;


-- No es pot fer una lectura d'un comptador que estigui com 'inactiu'

SELECT estat into estat_comptador
FROM comptador
WHERE cod_comptador = n_cod_comptador;

IF (estat_comptador = 'inactiu') THEN
RAISE noLectura;
END IF;


   
--No es poden fer dos lectures en el mateix dia

 SELECT count(*)
 INTO existe
 FROM LECTURA_CONSUM 
 WHERE cod_comptador=n_cod_comptador AND DATA_LECTURA=n_data_lectura AND n_estat = 'actiu';

 IF (existe = 1) THEN
  RAISE datesDuplicades; 
 END IF;

 IF (existe = 0) THEN
 
 -- Calculem el consum_mes
 -- Guardem dintre la variable 'lect' la ultima data del mes
 
 
 SELECT lectura INTO lect FROM LECTURA_CONSUM  l
 WHERE l.data_lectura IN (SELECT MAX(data_lectura ) FROM LECTURA_CONSUM WHERE cod_comptador = n_cod_comptador) AND 
       cod_comptador=n_cod_comptador AND n_estat = 'actiu'; 
   
    
  
    
  
 -- Inserim la nova lectura
 
    INSERT INTO LECTURA_CONSUM (cod_comptador, data_lectura, consum_mes, lectura, modalitat, estat) 
           VALUES (n_cod_comptador, n_data_lectura, (n_lectura - lect), n_lectura, lower(n_modalitat), lower (n_estat));
  
  
  -----------------------------------------------------------------------------------------
  --EST_CONSUM_COMPTADOR: Cridem al procediment de l'estadisctica p_estConsumComptador--  
     p_estConsumComptador (n_cod_comptador, n_lectura, lect);
  ----------------------------------------------------------------------------------------
  
 
  ----------------------------------------------------------------------------------------
  --EST_VALOR_MITJA 
  --Cridem al procediment que far tots els clculs per l'estadistica EST_VALOR_MITJA

      p_altaEstValorMitja();  
  ----------------------------------------------------------------------------------------
 
   ---------------------------------------------------------------------------------------
   --EST_LINIA_CARREGADA
   --Cridem al procediment qu far tots els calculs de l'estadistica EST_LINIA CARREGADA
   
     p_estLiniaCarregada();
   ----------------------------------------------------------------------------------------
 
    ---------------------------------------------------------------------------------------
   --EST_CONSUM_MIG
   
   --Cridem al procediment qu far tots els calculs de l'estadistica EST_CONSUM_MIG
      p_estConsumMig();
   ----------------------------------------------------------------------------------------
 
  ---------------------------------------------------------------------------------------
   --EST_PERCENTATGE_LINIES   
  
  --Obtenim el l'any de la data_lectura de la taula lectura_consum per EST_PERCENTATGE_LINIES
  SELECT EXTRACT (YEAR FROM LC.DATA_LECTURA) INTO n_anyo
  FROM LECTURA_CONSUM LC
  WHERE LC.DATA_LECTURA = n_data_lectura
  GROUP BY LC.DATA_LECTURA;
 
  --Cridem al procediment qu far tots els calculs de l'estadistica EST_PERCENTATGE_LINIES
 
       p_estPercentatgeLinies (n_anyo);
 ----------------------------------------------------------------------------------------
 
 
  
 -- Fi de les operacions
  
    rst := 'OK: S''ha donat d''alta la lectura';
    
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);
    COMMIT;
 
END IF;

-- Actualitzem el camp consum_absolut de la taula comptador

   select sum (consum_mes) into compt_consum_abs
   from lectura_consum
   where cod_comptador = n_cod_comptador and estat = 'actiu';
   
   UPDATE COMPTADOR
   SET consum_absolut = compt_consum_abs
   where cod_comptador = n_cod_comptador;
   
 
 
 ---------------------------------------------------------------------------------------
   --EST_TOP_TEN
   --Cridem al procediment qu far tots els calculs de l'estadistica EST_TOP_TEN 
   
    p_estTopTen();  
 ---------------------------------------------------------------------------------------
 

EXCEPTION
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps nuls que no haurien de ser nuls';
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);
      ROLLBACK;  
   
   WHEN errorEscriptura  
   THEN
      rst := 'ERROR: El camp ''modalitat'' no s''ha escrit correctament';
   
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);

   ROLLBACK;
   
   WHEN noLectura 
   THEN
      rst := 'ERROR: no es pot fer una lectura d''un comptador inactiu';
   
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);

   ROLLBACK;
   
   
   
   
   WHEN errorEscriptura2  
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);

   ROLLBACK;
   
   
   
   WHEN noExisteix_cod_comptador
   THEN
       rst := 'ERROR: no existeix codi_comtpador';
    
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);

   ROLLBACK;
   
   WHEN dataLectura
   THEN
      rst := 'ERROR: La data_lectura no pot ser anterior a la data_alta del contracte';
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);
      ROLLBACK; 
   
   
   
   
   
 WHEN datesDuplicades
 THEN
     rst := 'ERROR: No es poden fer dues lectures en el mateix dia d''un mateix comptador';
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);
    ROLLBACK;  
 
 WHEN NO_DATA_FOUND 
  THEN
   
  INSERT INTO LECTURA_CONSUM (cod_comptador, data_lectura, consum_mes, lectura, modalitat, estat) 
        VALUES (n_cod_comptador, n_data_lectura, (n_lectura), n_lectura, lower(n_modalitat), lower(n_estat));
   
   -- Actualitzem el camp consum_absolut de la taula comptador
   select sum (consum_mes) into compt_consum_abs
   from lectura_consum
   where cod_comptador = n_cod_comptador;
   
   UPDATE COMPTADOR
   SET consum_absolut = compt_consum_abs
   where cod_comptador = n_cod_comptador;
   
    
 rst := 'OK: S''ha donat d''alta la lectura';
   
   
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);
  COMMIT;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
    
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);

   ROLLBACK;  

END p_altaLecturaConsum;
/




-------------------------------------
--p_altaLinia
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_altaLinia (
  n_capacitat_maxima     IN NUMBER,
  n_estat                IN VARCHAR2

)
IS
--variables locals-- 
   rst                            VARCHAR2(2000);
   err_msg                        VARCHAR2(100);
   campsBuits                     EXCEPTION;
   errorEscriptura                EXCEPTION;
  

BEGIN

   -- Verifiquem que els camps no siguin nuls
   IF (n_capacitat_maxima IS NULL) OR (n_estat IS NULL) THEN
     RAISE campsBuits;
   END IF; 
   
  -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;
   -- Inserim la linia
  INSERT INTO LINIA (capacitat_maxima, estat) 
         VALUES (n_capacitat_maxima, lower(n_estat));
   
   -- Fi de les operacions
 
   rst := 'OK: S''ha inserit la linia correctament';
   
   p_insertLog('p_altaLinia', 'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||
               'n_estat: ' || n_estat, rst);

   COMMIT;
   
   EXCEPTION
     
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
        p_insertLog('p_altaLinia', 'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||
                    'n_estat: ' || n_estat, rst);
 
      ROLLBACK;  
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
       p_insertLog('p_altaLinia', 'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||
                   'n_estat: ' || n_estat, rst);

   ROLLBACK;
   

   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
    
   p_insertLog('p_altaLinia', 'n_capacitat_maxima: ' || n_capacitat_maxima, rst);
    
   ROLLBACK;

END p_altaLinia;
/




-------------------------------------
--p_altaModelComptador
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_altaModelComptador (
   n_data_fab      IN DATE,
   n_referencia    IN VARCHAR2,
   n_empresa_fab   IN VARCHAR2,
   n_descripcio    IN VARCHAR2,
   n_adrea        IN VARCHAR2,
   n_localitat     IN VARCHAR2,
   n_cp            IN VARCHAR2,
   n_pais          IN VARCHAR2,
   n_telefon       IN VARCHAR2
)
IS
--variables locals--

     rst                         VARCHAR2(2000);
     referencia_test             VARCHAR2(20);
     err_msg                     VARCHAR2(100);
     existeix_referencia         EXCEPTION;
     campsBuits                  EXCEPTION;
     noExisteixData              EXCEPTION;

BEGIN
   --verifiquem que els camps no siguin nuls
   IF (n_data_fab IS NULL) OR (n_referencia IS NULL) OR (n_empresa_fab IS NULL) OR (n_adrea IS NULL) OR (n_localitat IS NULL) OR 
      (n_cp IS NULL) OR (n_pais IS NULL) OR (n_telefon IS NULL) THEN 
     RAISE campsBuits;
   END IF; 

  --verifiquem que no existeix  la referencia del model de comptador
    SELECT COUNT (*)
     INTO referencia_test 
     FROM MODEL_COMPTADOR
    WHERE referencia = n_referencia;

   IF (referencia_test > 0)
   THEN
      RAISE existeix_referencia;
   END IF;

   --verifiquem que la data existeix (no pot ser superior  a la data del sistema(sysdate))
  IF (n_data_fab > sysdate) THEN 
    RAISE noExisteixData;
  END IF; 
  
  
   -- inserim el nou model de comptador
  INSERT INTO MODEL_COMPTADOR (data_fab, referencia, empresa_fab, descripcio, adrea, localitat, cp, pais, telefon) 
          VALUES (n_data_fab, n_referencia, n_empresa_fab, n_descripcio, n_adrea, n_localitat, n_cp, n_pais, n_telefon);
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit model de comptador';
   
    p_insertLog('p_altaModelComptador',  
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_empresa_fab: ' || n_empresa_fab || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
    'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
   COMMIT;
   
EXCEPTION
   WHEN existeix_referencia
   THEN
      rst := 'ERROR: El codi de model de comptador ja existeix';
     p_insertLog('p_altaModelComptador',  
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_empresa_fab: ' || n_empresa_fab || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
    'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
      ROLLBACK;
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
     p_insertLog('p_altaModelComptador',  
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_empresa_fab: ' || n_empresa_fab || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
    'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
      ROLLBACK;  
   
   WHEN noExisteixData
   THEN
      rst := 'ERROR: La data introduda no pot ser superior a la data actual';
   
    p_insertLog('p_altaModelComptador',  
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_empresa_fab: ' || n_empresa_fab || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
    'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);

      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   p_insertLog('p_altaModelComptador',  
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_empresa_fab: ' || n_empresa_fab || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
    'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
      ROLLBACK;

END p_altaModelComptador;
/





-------------------------------------
--p_altaNuclear
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_altaNuclear (
   n_cod_c_produccio   IN INTEGER,
   n_energia_minima    IN NUMBER,
   n_rebuig            IN NUMBER
)
IS
--variables locals--

   rst                            VARCHAR2(2000);
   err_msg                        VARCHAR2(100);
   cod_c_produccio_test           INTEGER;
   campsBuits                     EXCEPTION;
   noExisteix_cod_c_produccio     EXCEPTION;


BEGIN


   --verifiquem que els camps no siguin nuls
   IF (n_cod_c_produccio IS NULL) OR 
   (n_energia_minima IS NULL) OR
   (n_rebuig IS NULL) 
   THEN
       RAISE campsBuits;
   END IF; 

  -- Verifiquem que el codi_c_produccio existeix a la taula CENTRAL_PRODUCCIO   
   SELECT COUNT (*)
     INTO cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
   WHERE cod_c_produccio = n_cod_c_produccio;

   IF (cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
  
   -- inserim  la central nuclear
   INSERT INTO NUCLEAR  (cod_c_produccio, energia_minima, rebuig) 
          VALUES (n_cod_c_produccio, n_energia_minima, n_rebuig);
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit la central nuclear correctament';
   
    p_insertLog('p_altaNuclear',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_energia_minima: ' || n_energia_minima || ', ' ||
                'n_rebuig): ' || n_rebuig, rst);
   COMMIT;
   
   EXCEPTION
    
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
    p_insertLog('p_altaNuclear',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_energia_minima: ' || n_energia_minima || ', ' ||
                'n_rebuig): ' || n_rebuig, rst);
      ROLLBACK;  
   
   WHEN noExisteix_cod_c_produccio
      THEN
      rst := 'ERROR: El codi_c_produccio no existeix';
   
    p_insertLog('p_altaNuclear',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_energia_minima: ' || n_energia_minima || ', ' ||
                'n_rebuig): ' || n_rebuig, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:' || err_msg;
    
    p_insertLog('p_altaNuclear',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_energia_minima: ' || n_energia_minima || ', ' ||
                'n_rebuig): ' || n_rebuig, rst);
      ROLLBACK;

END p_altaNuclear;
/




-------------------------------------
--p_altaSolar
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_altaSolar (
   n_cod_c_produccio   IN INTEGER,
   n_num_panells       IN NUMBER
 
)
IS
--variables locals--

   rst                            VARCHAR2(2000);
   err_msg                        VARCHAR2(100);
   cod_c_produccio_test           INTEGER;
   campsBuits                     EXCEPTION;
   noExisteix_cod_c_produccio     EXCEPTION;


BEGIN


   --verifiquem que els camps no siguin nuls
   IF (n_cod_c_produccio IS NULL) OR 
   (n_num_panells IS NULL) 
   THEN
       RAISE campsBuits;
   END IF; 

  -- Verifiquem que el codi_c_produccio existeix a la taula CENTRAL_PRODUCCIO   
   SELECT COUNT (*)
     INTO cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
   WHERE cod_c_produccio = n_cod_c_produccio;

   IF (cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
  
   -- inserim  la central solar
   INSERT INTO SOLAR (cod_c_produccio, num_panells) 
          VALUES (n_cod_c_produccio, n_num_panells);
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit la central solar correctament';
   
    p_insertLog('p_altaSolar',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_num_panells: ' || n_num_panells, rst);
   COMMIT;
   
   EXCEPTION
    
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
   
    p_insertLog('p_altaSolar',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_num_panells: ' || n_num_panells, rst);
      ROLLBACK;  
   
   WHEN noExisteix_cod_c_produccio
      THEN
      rst := 'ERROR: El cod_c_produccio no existeix';
   
    p_insertLog('p_altaSolar',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_num_panells: ' || n_num_panells, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:' || err_msg;
    
    p_insertLog('p_altaSolar',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_num_panells: ' || n_num_panells, rst);
      ROLLBACK;
   


END p_altaSolar;
/





-------------------------------------
--p_altaTermica
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_altaTermica ( 
   n_cod_c_produccio   IN INTEGER,
   n_emissioco2        IN NUMBER
 
)
IS
--variables locals--

   rst                            VARCHAR2(2000);
   err_msg                        VARCHAR2(100);
   cod_c_produccio_test           INTEGER;
   campsBuits                     EXCEPTION;
   noExisteix_cod_c_produccio     EXCEPTION;


BEGIN


   --verifiquem que els camps no siguin nuls
   IF (n_cod_c_produccio IS NULL) OR 
   (n_emissioco2 IS NULL) 
   THEN
       RAISE campsBuits;
   END IF; 

  -- Verifiquem que el codi_c_produccio existeix a la taula CENTRAL_PRODUCCIO   
   SELECT COUNT (*)
     INTO cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
   WHERE cod_c_produccio = n_cod_c_produccio;

   IF (cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
  
   -- inserim  la central termica
   INSERT INTO TERMICA  (cod_c_produccio, emissioco2) 
          VALUES (n_cod_c_produccio, n_emissioco2);
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit la central termica correctament';
   
    p_insertLog('p_altaTermica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_emissioco2: ' || n_emissioco2, rst);
   COMMIT;
   
   EXCEPTION
    
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
    p_insertLog('p_altaTermica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_emissioco2: ' || n_emissioco2, rst);
      ROLLBACK;  
   
   WHEN noExisteix_cod_c_produccio
      THEN
      rst := 'ERROR: El cod_c_produccio no existeix';
   
    p_insertLog('p_altaTermica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_emissioco2: ' || n_emissioco2, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:' || err_msg;
    
    p_insertLog('p_altaTermica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_emissioco2: ' || n_emissioco2, rst);
      ROLLBACK;
   
END p_altaTermica;
/





-------------------------------------
--p_baixaAlimentaCentral
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_baixaAlimentaCentral (
       n_cod_c_distribucio  IN INTEGER,
       n_cod_linia          IN INTEGER,
       n_cod_c_produccio    IN INTEGER,
    n_estat    IN VARCHAR2
)
IS  

--Variables locals
   cod_c_distribucio_test               INTEGER;
   n_cod_linia_test                     INTEGER;   
   n_cod_c_produccio_test               INTEGER;
   err_msg                              VARCHAR2(100);
   rst                                  VARCHAR2(100);
   noExisteix_cod_c_distribucio         EXCEPTION;
   noExisteix_cod_linia                 EXCEPTION;
   noExisteix_cod_c_produccio           EXCEPTION;
   campsBuits                           EXCEPTION;
   
BEGIN

  --El codis de central distribucio, linia i central 
  --de producci no poden estar buits
  
  IF ( n_cod_c_distribucio IS NULL) OR (n_cod_linia IS NULL) OR (n_cod_c_produccio IS NULL)
  THEN 
    RAISE campsBuits;
  END IF; 
  
    --El codi de la central de distribuci ha d'existir
  SELECT COUNT (*)
     INTO cod_c_distribucio_test
     FROM CENTRAL_DISTRIBUCIO
     WHERE cod_c_distribucio = n_cod_c_distribucio;

   IF (cod_c_distribucio_test = 0)
   THEN
      RAISE noExisteix_cod_c_distribucio;
   END IF;
   
     --El codi de la linia ha d'existir
  SELECT COUNT (*)
     INTO n_cod_linia_test
     FROM LINIA
     WHERE cod_linia = n_cod_linia;

   IF (n_cod_linia_test = 0)
   THEN
      RAISE noExisteix_cod_linia;
   END IF;
   
     --El codi de la central de producci ha d'existir
  SELECT COUNT (*)
     INTO n_cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
     WHERE cod_c_produccio = n_cod_c_produccio;

   IF (n_cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
  
   --Esborrem la relaci
   DELETE FROM ALIMENTA_CENTRAL
   WHERE cod_c_distribucio = n_cod_c_distribucio AND
         cod_linia = n_cod_linia AND
         cod_c_produccio = n_cod_c_produccio;
   
   -- Fi de les operacions
   rst := 'OK: S''ha donat de baixa la relaci';

    p_insertLog('p_baixaAlimentaCentral', 
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
               'n_cod_linia: ' || n_cod_linia || ', ' ||
               'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_c_distribucio
   THEN
      rst := 'ERROR: El codi de la central de distribuci no existeix';
      
    p_insertLog('p_baixaAlimentaCentral', 
             'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
    'n_cod_linia: ' || n_cod_linia || ', ' ||
             'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;
   
   WHEN noExisteix_cod_linia
   THEN
      rst := 'ERROR: El codi de la linia no existeix';
      
    p_insertLog('p_baixaAlimentaCentral', 
             'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
    'n_cod_linia: ' || n_cod_linia || ', ' ||
             'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
 ROLLBACK;
 
   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de produccio no existeix';
      
    p_insertLog('p_baixaAlimentaCentral', 
             'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
    'n_cod_linia: ' || n_cod_linia || ', ' ||
             'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
 ROLLBACK;
       
   WHEN campsBuits
   THEN
      rst := 'ERROR: Els camps buits no poden estar buits';
      
    p_insertLog('p_baixaAlimentaCentral', 
             'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
    'n_cod_linia: ' || n_cod_linia || ', ' ||
             'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;
 

END p_baixaAlimentaCentral;
/





-------------------------------------
--p_baixaCarbo
-------------------------------------
CREATE OR REPLACE PROCEDURE USR_ENE.p_baixaCarbo (
    n_cod_c_produccio  IN INTEGER  
)
IS  

--Variables locals
   n_cod_c_produccio_test          INTEGER;
   err_msg                         VARCHAR2(100);
   rst                             VARCHAR2(100);
   noExisteix_cod_c_produccio      EXCEPTION;
   esNul_cod_c_produccio           EXCEPTION;
   
BEGIN

  --Verifiquem que el ccod_c_produccio no sigui nul
  IF ( n_cod_c_produccio  IS NULL) 
  THEN 
    RAISE esNul_cod_c_produccio;
  END IF; 
  
  --El codi de la central de producci ha d'existir
  SELECT COUNT (*)
     INTO n_cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
     WHERE cod_c_produccio = n_cod_c_produccio;

   IF (n_cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
  
   --Esborrem la central termica
   DELETE FROM CARBO 
   WHERE cod_c_produccio = n_cod_c_produccio;
   
   -- Fi de les operacions
   rst := 'OK: S''ha donat de baixa la central de carb';

   p_insertLog('p_baixaCarbo', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de producci no existeix';
      
            p_insertLog('p_baixaCarbo', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;
   
   WHEN esNul_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de producci no pot ser nul';
      
          p_insertLog('p_baixaCarbo', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
        p_insertLog('p_baixaCarbo', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;


END p_baixaCarbo;
/



-------------------------------------
--p_baixaCentralDistribucio
-------------------------------------
CREATE OR REPLACE PROCEDURE p_baixaCentralDistribucio (
    n_cod_c_distribucio  IN INTEGER,
 n_estat              IN VARCHAR2  
)
IS  

--Variables locals
   n_cod_c_distribucio_test        INTEGER;
   err_msg                         VARCHAR2(100);
   rst                             VARCHAR2(100);
   noExisteix_cod_c_distribucio    EXCEPTION;
   esNul_cod_c_distribucio         EXCEPTION;
   errorEscriptura                 EXCEPTION;
   no_actiu						   EXCEPTION;
   
BEGIN

  --Verifiquem que el cod_c_distribucio no sigui nul
  IF ( n_cod_c_distribucio  IS NULL) 
  THEN 
    RAISE esNul_cod_c_distribucio;
  END IF; 
  
  --El codi de la central de distribuci ha d'existir
  SELECT COUNT (*)
     INTO n_cod_c_distribucio_test
     FROM CENTRAL_DISTRIBUCIO
     WHERE cod_c_distribucio = n_cod_c_distribucio;

   IF (n_cod_c_distribucio_test = 0)
   THEN
      RAISE noExisteix_cod_c_distribucio;
   END IF;
   
   --No es pot activar un central de distirbucio inactiva
   --en aquest cas s'hauria de fer una alta nova amb un nou codi de central
   
   IF (n_estat = 'actiu') THEN
   RAISE no_actiu;
   END IF;
  
   -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;
  
  
   --Fem una baixa lgica de central de distribuci actualitzant
   --el camp estat d'actiu a inactiu
   UPDATE CENTRAL_DISTRIBUCIO
   SET estat = n_estat
   WHERE cod_c_distribucio = n_cod_c_distribucio;
   
   -- Fi de les operacions
   rst := 'OK: La central de distribuc est inactiva';

   p_insertLog('p_baixaCentralDistribucio', 'n_cod_c_distribucio: ' || n_cod_c_distribucio ||','||
                                            'n_estat: ' || n_estat, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_c_distribucio
   THEN
      rst := 'ERROR: El codi de la central de distribuci no existeix';
      
   p_insertLog('p_baixaCentralDistribucio', 'n_cod_c_distribucio: ' || n_cod_c_distribucio ||','||
                                            'n_estat: ' || n_estat, rst);
      ROLLBACK;
	  
   WHEN no_actiu
   THEN
      rst := 'ERROR: No es pot activar una central de distribuci inactiva';
      
   p_insertLog('p_baixaCentralDistribucio', 'n_cod_c_distribucio: ' || n_cod_c_distribucio ||','||
                                            'n_estat: ' || n_estat, rst);
      ROLLBACK;	  
   
   WHEN esNul_cod_c_distribucio
   THEN
      rst := 'ERROR: El codi de la central de distribuci no pot ser nul';
      
   p_insertLog('p_baixaCentralDistribucio', 'n_cod_c_distribucio: ' || n_cod_c_distribucio ||','||
                                            'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
      
   p_insertLog('p_baixaCentralDistribucio', 'n_cod_c_distribucio: ' || n_cod_c_distribucio ||','||
                                            'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_baixaCentralDistribucio', 'n_cod_c_distribucio: ' || n_cod_c_distribucio ||','||
                                            'n_estat: ' || n_estat, rst);
      ROLLBACK;




END p_baixaCentralDistribucio;
/




-------------------------------------
--p_baixaCentralProduccio
-------------------------------------
CREATE OR REPLACE PROCEDURE p_baixaCentralProduccio (
    n_cod_c_produccio  IN INTEGER,
 n_estat      IN VARCHAR2  
)
IS  

--Variables locals
   n_cod_c_produccio_test          INTEGER;
   err_msg                         VARCHAR2(100);
   rst                             VARCHAR2(100);
   noExisteix_cod_c_produccio      EXCEPTION;
   esNul_cod_c_produccio           EXCEPTION;
   errorEscriptura                 EXCEPTION;
   no_actiu						   EXCEPTION;
   
BEGIN

  --Verifiquem que el ccod_c_produccio no sigui nul
  IF ( n_cod_c_produccio  IS NULL) 
  THEN 
    RAISE esNul_cod_c_produccio;
  END IF; 
  
  --El codi de la central de producci ha d'existir
  SELECT COUNT (*)
     INTO n_cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
     WHERE cod_c_produccio = n_cod_c_produccio;

   IF (n_cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
   
   	-- Una central de produccio no pot passar d'inactiva a activa
	--en aquest cas, s'haur de fer una alta nova i tenir un altre codi
	
	IF (n_estat = 'actiu') THEN
	RAISE no_actiu;
	END IF;  
	
    -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('inactiu') THEN
      RAISE errorEscriptura;
   END IF;
        
   --Fem la baixa logica de la central de produccio
   UPDATE CENTRAL_PRODUCCIO
   SET estat = n_estat
   WHERE cod_c_produccio = n_cod_c_produccio;
   
   --Si s'ha fet una baixa d'una central de produccio, les seves
   --produccions d'energies hauran d'estar inactives, per tant el camp valid
   --de la taula energia_Absoluta estar com a 'no'
   
   
  IF (n_estat = 'inactiu') then  
    UPDATE ENERGIA_ABSOLUTA
  SET valid = 'no'
  WHERE cod_c_produccio = n_cod_c_produccio;
  END IF;
   
   -- Fi de les operacions
   rst := 'OK: La central de producci est inactiva';

         p_insertLog('p_baixaCentralProduccio', 'n_cod_c_produccio: ' || n_cod_c_produccio ||','||
                'n_estat: ' || n_estat, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de producci no existeix';
      
       p_insertLog('p_baixaCentralProduccio', 'n_cod_c_produccio: ' || n_cod_c_produccio ||','||
                'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN esNul_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de producci no pot ser nul';
      
              p_insertLog('p_baixaCentralProduccio', 'n_cod_c_produccio: ' || n_cod_c_produccio ||','||
                'n_estat: ' || n_estat, rst);
      ROLLBACK;
	  
   WHEN no_actiu
   THEN
      rst := 'ERROR: No es pot activar una central de producci inactiva';
      
              p_insertLog('p_baixaCentralProduccio', 'n_cod_c_produccio: ' || n_cod_c_produccio ||','||
                'n_estat: ' || n_estat, rst);
   ROLLBACK;
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
      
       p_insertLog('p_baixaCentralProduccio', 'n_cod_c_produccio: ' || n_cod_c_produccio ||','||
                'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
       p_insertLog('p_baixaCentralProduccio', 'n_cod_c_produccio: ' || n_cod_c_produccio ||','||
                'n_estat: ' || n_estat, rst);
      ROLLBACK;

END p_baixaCentralProduccio;
/




-------------------------------------
--p_baixaClient
-------------------------------------
CREATE OR REPLACE PROCEDURE p_baixaClient (
    n_cod_client        IN INTEGER,
    n_estat             IN VARCHAR2
)
IS

--Variables locals
   n_cod_client_test               VARCHAR2(100);
   rst                             VARCHAR2(100);
   err_msg                         VARCHAR2(100);
   noExisteix_cod_client           EXCEPTION;
   esNul_cod_client                EXCEPTION;
   no_actiu						   EXCEPTION;

   
BEGIN

  --Verifiquem que el cod_client no sigui nul
  IF ( n_cod_client  IS NULL) 
  THEN 
    RAISE esNul_cod_client;
  END IF; 
  
  --El codi del client ha d'existir
  SELECT COUNT (*)
     INTO n_cod_client_test  
     FROM CLIENT
     WHERE cod_client = n_cod_client;

   IF (n_cod_client_test = 0)
   THEN
      RAISE noExisteix_cod_client;
   END IF;
  
   --No es pot activar un client que estigu inactiu. Haur de tenir un altre codi_client
   IF (n_estat = 'actiu') THEN
   	  RAISE no_actiu;
   END IF; 
  
   --Fem la baixa lgica del resigre
   UPDATE CLIENT
   SET estat  = n_estat            
   WHERE cod_client = n_cod_client;
   
  
  -- Si el registre d'un client ha quedat inactiu, el contracte associat tamb ho haur d'estar
  
  IF (n_estat = 'inactiu') then
  
    UPDATE CONTRACTE
  SET estat = n_estat
  WHERE cod_client = n_cod_client;
  END IF;
  
   
   
   -- Fi de les operacions
   rst := 'OK: El client s inactiu';

   p_insertLog('p_baixaClient', 'n_cod_client: ' || n_cod_client || ', ' ||
               'n_estat: ' || n_estat, rst);
   COMMIT; 
   
   
   
EXCEPTION
   WHEN noExisteix_cod_client
   THEN
      rst := 'ERROR: El codi_client no existeix';
      
   p_insertLog('p_baixaClient', 'n_cod_client: ' || n_cod_client || ', ' ||
               'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN esNul_cod_client
   THEN
      rst := 'ERROR: El cod_client no pot ser nul';
      
   p_insertLog('p_baixaClient', 'n_cod_client: ' || n_cod_client || ', ' ||
               'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN no_actiu
   THEN
      rst := 'ERROR: No es pot activar un client inactiu';
      
   p_insertLog('p_baixaClient', 'n_cod_client: ' || n_cod_client || ', ' ||
               'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_baixaClient', 'n_cod_client: ' || n_cod_client || ', ' ||
               'n_estat: ' || n_estat, rst);
      ROLLBACK;



END p_baixaClient;
/





-------------------------------------
--p_baixaComptador
-------------------------------------
CREATE OR REPLACE PROCEDURE p_baixaComptador (
    n_cod_comptador IN INTEGER,
 n_estat         IN VARCHAR2
  
)
IS  

--Variables locals
   n_cod_comptador_test        INTEGER;
   err_msg                     VARCHAR2(100);
   rst                         VARCHAR2(100);
   errorEscriptura             EXCEPTION;
   noExisteix_cod_comptador    EXCEPTION;
   esNul_cod_comptador         EXCEPTION;
   no_actiu					   EXCEPTION;
   
BEGIN

  --El cod_comptador no pot ser nul
  IF ( n_cod_comptador  IS NULL) 
  THEN 
    RAISE esNul_cod_comptador;
  END IF; 
  
  --El codi del comptador ha d'existir
  SELECT COUNT (*)
     INTO n_cod_comptador_test
     FROM COMPTADOR
     WHERE cod_comptador = n_cod_comptador;

   IF (n_cod_comptador_test = 0)
   THEN
      RAISE noExisteix_cod_comptador;
   END IF;
   
   --No es pot activar un comptador inactiu, s'haur de fer una
   --alta nova amb un nou codi comptador
   
   IF (n_estat = 'actiu') THEN
   RAISE no_actiu;
   END IF;
   
   -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;
  
   --Fem la baixa lgica: actualitzem el camp estat a inactiu
   UPDATE COMPTADOR 
   SET estat = n_estat
   WHERE cod_comptador = n_cod_comptador;
   
   --Si el registre del comptador s inactiu, el seu contracte
   --associat ha d'estar inactiu
   
     IF (n_estat = 'inactiu') then
  
    UPDATE CONTRACTE
  SET estat = n_estat
  WHERE cod_comptador = n_cod_comptador;
  END IF;
   
   --Si el registra del comptador s inacitu, totes les
   --lectures associades haurn d'estar inactives per aquest comptador
   
  IF (n_estat = 'inactiu') then  
    UPDATE LECTURA_CONSUM
  SET estat = n_estat
  WHERE cod_comptador = n_cod_comptador;
  END IF;
   
   -- Fi de les operacions
   
   rst := 'OK: El registre del comptador s inactiu';

    p_insertLog('p_baixaComptador', 'n_cod_comptador: ' || n_cod_comptador ||','||
                                 'n_estat: ' || n_estat, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_comptador
   THEN
      rst := 'ERROR: El codi del comptador no existeix';
      
    p_insertLog('p_baixaComptador', 'n_cod_comptador: ' || n_cod_comptador ||','||
                                 'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN no_actiu
   THEN
      rst := 'ERROR: No es pot activar un comptador inactiu';
      
    p_insertLog('p_baixaComptador', 'n_cod_comptador: ' || n_cod_comptador ||','||
                                 'n_estat: ' || n_estat, rst);
   ROLLBACK;
   
   
   WHEN esNul_cod_comptador
   THEN
      rst := 'ERROR: El codi del comptador s nul';
      
          p_insertLog('p_baixaComptador', 'n_cod_comptador: ' || n_cod_comptador ||','||
                                 'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
      WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
        p_insertLog('p_baixaComptador', 'n_cod_comptador: ' || n_cod_comptador ||','||
                                 'n_estat: ' || n_estat, rst);

   ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
          p_insertLog('p_baixaComptador', 'n_cod_comptador: ' || n_cod_comptador ||','||
                                 'n_estat: ' || n_estat, rst);
      ROLLBACK;


END p_baixaComptador;
/





-------------------------------------
--p_baixaConnectaComptador
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_baixaConnectaComptador (
     n_cod_comptador       IN INTEGER,
     n_cod_c_distribucio   IN INTEGER,
  n_estat      IN VARCHAR2
  
)
IS  

--Variables locals
   n_cod_comptador_test               INTEGER;
   n_cod_c_distribucio_test           INTEGER;   
   err_msg                            VARCHAR2(100);
   rst                                VARCHAR2(100);
   noExisteix_cod_comptador           EXCEPTION;
   esNul_cod_comptador                EXCEPTION;
   noExisteix_cod_c_distribucio       EXCEPTION;
   esNul_cod_c_distribucio            EXCEPTION;
   
BEGIN

  --El cod_comptador no pot ser nuls
  IF ( n_cod_comptador IS NULL) 
  THEN 
    RAISE esNul_cod_comptador;
  END IF; 
  
  --El cod_c_distribucio  no pot ser nul
  IF ( n_cod_c_distribucio IS NULL) 
  THEN 
    RAISE esNul_cod_c_distribucio;
  END IF; 
  
  --El codi del comtpador ha d'existir
  SELECT COUNT (*)
     INTO n_cod_comptador_test
     FROM COMPTADOR
     WHERE cod_comptador = n_cod_comptador;

   IF (n_cod_comptador_test = 0)
   THEN
      RAISE noExisteix_cod_comptador;
   END IF;
   
  --El codi de la central de distribuci ha d'existir
  SELECT COUNT (*)
     INTO n_cod_c_distribucio_test
     FROM CENTRAL_DISTRIBUCIO
     WHERE cod_c_distribucio = n_cod_c_distribucio;

   IF (n_cod_c_distribucio_test = 0)
   THEN
      RAISE noExisteix_cod_c_distribucio;
   END IF;
  
   --Esborrem la relaci
   DELETE FROM CONNECTA_COMPTADOR 
   WHERE cod_comptador = n_cod_comptador AND
         cod_c_distribucio = n_cod_c_distribucio;
   
   -- Fi de les operacions
   rst := 'OK: S''ha donat de baixa la relaci';

    p_insertLog(' p_baixaConnectaComptador', 'n_cod_comptador: ' || n_cod_comptador || ', ' ||
               'n_cod_c_distribucio: ' || n_cod_c_distribucio , rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_comptador
   THEN
      rst := 'ERROR: El codi del comptador no existeix';
      
       p_insertLog('p_baixaConnectaComptador', 'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_c_distribucio: ' || n_cod_c_distribucio , rst);
      ROLLBACK;
   
   WHEN noExisteix_cod_c_distribucio
   THEN
      rst := 'ERROR: El codi de la central de distribucio no existeix';
      
       p_insertLog('p_baixaConnectaComptador', 'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_c_distribucio: ' || n_cod_c_distribucio , rst);
       
   WHEN esNul_cod_comptador
   THEN
      rst := 'ERROR: El codi del comptador s nul';
      
      p_insertLog('p_baixaConnectaComptador', 'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_c_distribucio: ' || n_cod_c_distribucio , rst);
      ROLLBACK;
   
   WHEN esNul_cod_c_distribucio
   THEN
      rst := 'ERROR: El codi de la central de distribucio s nul';
      
      p_insertLog('p_baixaConnectaComptador', 'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_c_distribucio: ' || n_cod_c_distribucio , rst);
    ROLLBACK;
    
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
      p_insertLog('p_baixaConnectaComptador', 'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_c_distribucio: ' || n_cod_c_distribucio , rst);
      ROLLBACK;

END p_baixaConnectaComptador;
/





-------------------------------------
--p_baixaContracte
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_baixaContracte (
    n_cod_contracte IN INTEGER  
)
IS  

--Variables locals
   n_cod_contracte_test        INTEGER;
   err_msg                     VARCHAR2(100);
   rst                         VARCHAR2(100);
   noExisteix_cod_contracte    EXCEPTION;
   esNul_cod_contracte         EXCEPTION;
   
BEGIN

  --El cod_contracte pot ser nul
  IF ( n_cod_contracte  IS NULL) 
  THEN 
    RAISE esNul_cod_contracte;
  END IF; 
  
  --El codi del comptador ha d'existir
  SELECT COUNT (*)
     INTO n_cod_contracte_test
     FROM CONTRACTE
     WHERE cod_contracte = n_cod_contracte;

   IF (n_cod_contracte_test = 0)
   THEN
      RAISE noExisteix_cod_contracte;
   END IF;
  
   --Esborrem el contracte noms si l'estat de registre est inactiu
   DELETE FROM CONTRACTE 
   WHERE cod_contracte = n_cod_contracte and
         estat = 'inactiu';
   
   -- Fi de les operacions
   rst := 'OK: S''ha donat de baixa el contracte';

    p_insertLog('p_baixaContracte', 'n_cod_contracte: ' || n_cod_contracte, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_contracte
   THEN
      rst := 'ERROR: El codi del contracte no existeix';
      
      p_insertLog('p_baixaContracte', 'n_cod_contracte: ' || n_cod_contracte, rst);
      ROLLBACK;
   
   WHEN esNul_cod_contracte
   THEN
      rst := 'ERROR: El codi del contracte no pot ser nul';
      
      p_insertLog('p_baixaContracte', 'n_cod_contracte: ' || n_cod_contracte, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
      p_insertLog('p_baixaContracte', 'n_cod_contracte: ' || n_cod_contracte, rst);
      ROLLBACK;


END p_baixaContracte;
/




-------------------------------------
--p_baixaEnergiaAbsoluta
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_baixaEnergiaAbsoluta (
     n_cod_ene_absoluta  IN INTEGER, 
       n_valid             IN VARCHAR2
)
IS

--Variables locals
   n_cod_ene_absoluta_test         INTEGER;
   err_msg                         VARCHAR2(100);
   rst                    VARCHAR2(100);
   n_data_prod        DATE;
   n_anyo         NUMBER;
   noExisteix_cod_ene_absoluta     EXCEPTION;
   esNul_cod_ene_absoluta          EXCEPTION;
   
BEGIN

  --Verifiquem que el cod_ene_absoluta no sigui nul
  IF ( n_cod_ene_absoluta  IS NULL) 
  THEN 
    RAISE esNul_cod_ene_absoluta;
  END IF; 
  
  --El codi de ENERGIA_ABSOLUTA ha d'existir
  SELECT COUNT (*)
     INTO n_cod_ene_absoluta_test
     FROM ENERGIA_ABSOLUTA
     WHERE cod_ene_absoluta = n_cod_ene_absoluta;

   IF (n_cod_ene_absoluta_test = 0)
   THEN
      RAISE noExisteix_cod_ene_absoluta;
   END IF;
  
   --Fem la baixa lgica del resigre
   UPDATE ENERGIA_ABSOLUTA
   SET valid  = n_valid            
   WHERE cod_ene_absoluta = n_cod_ene_absoluta;
   
   ----------------------------------------------------------------------------------------------
     --EST_NOMBRE_CENTRAL_PRODUCCIO
   
   --Hem de saber la data_prod de produccio que es donar de baixa per tal de poder
   --cridar al procediment p_modEstNomCentProduccio(n_anyo);
   
   SELECT EA.DATA_PROD INTO n_data_prod
   FROM ENERGIA_ABSOLUTA EA
   WHERE EA.COD_ENE_ABSOLUTA = n_cod_ene_absoluta;
   
     
    --Obtenim el l'any de la data_prod de la taula energia_absoluta per EST_NOMBRE_CENTRAL_PRODUCCIO
    SELECT EXTRACT (YEAR FROM EA.DATA_PROD) INTO n_anyo
    FROM ENERGIA_ABSOLUTA EA
    WHERE EA.DATA_PROD = n_data_prod
    GROUP BY EA.DATA_PROD;
 
    --Cridem al procediment qu far tots els calculs de l'estadistica EST_NOMBRE_CENTRAL_PRODUCCIO
 
       p_modEstNomCentProduccio(n_anyo);  
   
   ---------------------------------------------------------------------------------------------- 
   
   -- Fi de les operacions
   rst := 'OK: S''ha actualitzat el camp ''valid'' correctament';

   p_insertLog('p_baixaEnergiaAbsoluta', 'n_cod_ene_absoluta: ' || n_cod_ene_absoluta || ', ' ||
               'n_valid: ' || n_valid, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_ene_absoluta
   THEN
      rst := 'ERROR: El codi cod_ene_absoluta no existeix';
      
   p_insertLog('p_baixaEnergiaAbsoluta', 'n_cod_ene_absoluta: ' || n_cod_ene_absoluta || ', ' ||
               'n_valid: ' || n_valid, rst);
      ROLLBACK;
   
   WHEN esNul_cod_ene_absoluta
   THEN
      rst := 'ERROR: El codi de cod_ene_absoluta no pot ser nul';
      
   p_insertLog('p_baixaEnergiaAbsoluta', 'n_cod_ene_absoluta: ' || n_cod_ene_absoluta || ', ' ||
               'n_valid: ' || n_valid, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_baixaEnergiaAbsoluta', 'n_cod_ene_absoluta: ' || n_cod_ene_absoluta || ', ' ||
               'n_valid: ' || n_valid, rst);
      ROLLBACK;


   
END p_baixaEnergiaAbsoluta;
/





-------------------------------------
--p_baixaEolica
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_baixaEolica (
    n_cod_c_produccio  IN INTEGER  
)
IS  

--Variables locals
   n_cod_c_produccio_test          INTEGER;
   err_msg                         VARCHAR2(100);
   rst                             VARCHAR2(100);
   noExisteix_cod_c_produccio      EXCEPTION;
   esNul_cod_c_produccio           EXCEPTION;
   
BEGIN

  --Verifiquem que el ccod_c_produccio no sigui nul
  IF ( n_cod_c_produccio  IS NULL) 
  THEN 
    RAISE esNul_cod_c_produccio;
  END IF; 
  
  --El codi de la central de producci ha d'existir
  SELECT COUNT (*)
     INTO n_cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
     WHERE cod_c_produccio = n_cod_c_produccio;

   IF (n_cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
  
   --Esborrem la central termica
   DELETE FROM EOLICA 
   WHERE cod_c_produccio = n_cod_c_produccio;
   
   -- Fi de les operacions
   rst := 'OK: S''ha donat de baixa la central eolica';

   p_insertLog('p_baixaEolica', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de producci no existeix';
      
       p_insertLog('p_baixaEolica', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;
   
   WHEN esNul_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de producci no pot ser nul';
      
      p_insertLog('p_baixaEolica', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
       p_insertLog('p_baixaEolica', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;


END p_baixaEolica;
/





-------------------------------------
--p_baixaLecturaConsum
-------------------------------------
CREATE OR REPLACE PROCEDURE p_baixaLecturaConsum (
    n_cod_lect_consum   IN INTEGER,
    n_estat             IN VARCHAR2
)
IS

--Variables locals
   n_cod_lect_consum_test          INTEGER;
   err_msg                         VARCHAR2(100);
   rst                   VARCHAR2(100);
   n_cod_comptador       INTEGER;
   a_cod_c_produccio      INTEGER;
   EXISTE_CODPRO       INTEGER;
   consum_mes        NUMBER;
   n_data_lectura       DATE;
   n_anyo         NUMBER;
   noExisteix_cod_lect_consum      EXCEPTION;
   esNul_cod_lect_consum           EXCEPTION;
   
BEGIN

  --Verifiquem que el cod_lect_consum no sigui nul
  IF ( n_cod_lect_consum  IS NULL) 
  THEN 
    RAISE esNul_cod_lect_consum;
  END IF; 
  
  --El codi de la LECTURA_CONSUM ha d'existir
  SELECT COUNT (*)
     INTO n_cod_lect_consum_test
     FROM LECTURA_CONSUM
     WHERE cod_lect_consum = n_cod_lect_consum;

   IF (n_cod_lect_consum_test = 0)
   THEN
      RAISE noExisteix_cod_lect_consum;
   END IF;
  
   --Fem la baixa lgica del resigre
   UPDATE LECTURA_CONSUM
   SET estat  = n_estat            
   WHERE cod_lect_consum = n_cod_lect_consum;
   
   
   
  --ESTADISTICA: EST_CONSUM_COMTPADOR---------------------------------------------------------------------------
  
  --Obtenim el codi del comptador a partir del codi de la lectura  
  SELECT COD_COMPTADOR INTO n_cod_comptador
  FROM LECTURA_CONSUM 
  WHERE COD_LECT_CONSUM = n_cod_lect_consum;
  
  --Obtenim el consum del comptador a partir a partir del codi de la lectura 
  
  SELECT CONSUM_MES INTO consum_mes
  FROM LECTURA_CONSUM 
  WHERE COD_LECT_CONSUM = n_cod_lect_consum;
  
  
  --Obtenim el cod_c_produccio corresponent al cod_comptador
   a_cod_c_produccio := getCodProduccio (n_cod_comptador);
   
   IF (n_estat = 'inactiu') THEN
      --Actualitzem la taula EST_CONSUM_COMPTADOR
     --Verifiquem que existeix el cod_c_produccio
     SELECT COUNT (*)
      INTO EXISTE_CODPRO
      FROM EST_CONSUM_COMPTADOR EC
      WHERE EC.COD_C_PRODUCCIO = a_cod_c_produccio;
  
       p_modEstConsumComptador (a_cod_c_produccio, (consum_mes *(-1)));
  
  END IF;
  
 --------------------------------------------------------------------------------------
 --ESTADISTICA: EST_VALOR_MITJA-------------------------------------------------------
 --En el moment que es fa una baixa lgica de la taula LECTURA_CONSUM s'actualitza
 --tota la taula EST_VALOR_MITJA
 
    p_altaEstValorMitja();  
 --------------------------------------------------------------------------------------
   
   
   --------------------------------------------------------------------------------------
 --ESTADISTICA: EST_lINIA CARREGADA------------------------------------------------------
 --En el moment que es fa una baixa lgica de la taula LECTURA_CONSUM s'actualitza
 --tota la taula EST_LINIA_CARREGADA
 
    p_estLiniaCarregada(); 
 -------------------------------------------------------------------------------------- 
   
 
  --------------------------------------------------------------------------------------
   --EST_PERCENTATGE_LINIES
   
   --Hem de saber la data_lectura de la linia que es donar de baixa per tal de poder
   --cridar al procediment p_modPercentatgeLinies(n_anyo);
   
   SELECT LC.DATA_LECTURA INTO n_data_lectura
   FROM LECTURA_CONSUM LC
   WHERE LC.COD_LECT_CONSUM = n_cod_lect_consum;
   
   
    --Obtenim el l'any de la data_lectura de la taula lectura_consum per EST_PERCENTATGE_LINIES
    SELECT EXTRACT (YEAR FROM LC.DATA_LECTURA) INTO n_anyo
    FROM LECTURA_CONSUM LC
    WHERE LC.DATA_LECTURA = n_data_lectura
    GROUP BY LC.DATA_LECTURA;
 
 
   --Cridem al procediment qu far tots els calculs de l'estadistica EST_PERCENTATGE_LINIES
 
       p_modEstPercentatgeLinies (n_anyo);  
  
  -------------------------------------------------------------------------------------- 
   
   
   
   -- Fi de les operacions
   rst := 'OK: S''ha actualitzat el camp ''estat'' correctament';

   p_insertLog('p_baixaLecturaConsum', 'n_cod_lect_consum: ' || n_cod_lect_consum || ', ' ||
               'n_estat: ' || n_estat, rst);
   COMMIT; 
   
   
   
EXCEPTION
   WHEN noExisteix_cod_lect_consum
   THEN
      rst := 'ERROR: El codi lectura_consum no existeix';
      
  p_insertLog('p_baixaLecturaConsum', 'n_cod_lect_consum: ' || n_cod_lect_consum || ', ' ||
               'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN esNul_cod_lect_consum
   THEN
      rst := 'ERROR: El codi de lectura_consum no pot ser nul';
      
  p_insertLog('p_baixaLecturaConsum', 'n_cod_lect_consum: ' || n_cod_lect_consum || ', ' ||
               'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
  p_insertLog('p_baixaLecturaConsum', 'n_cod_lect_consum: ' || n_cod_lect_consum || ', ' ||
               'n_estat: ' || n_estat, rst);
      ROLLBACK;

END p_baixaLecturaConsum;
/










-------------------------------------
--p_baixaLinia
-------------------------------------
CREATE OR REPLACE PROCEDURE p_baixaLinia ( 
   n_cod_linia IN INTEGER ,
   n_estat    IN VARCHAR2 
)
IS  

--Variables locals
   n_cod_linia_test        INTEGER;
   err_msg                 VARCHAR2(100);
   rst                     VARCHAR2(100);
   noExisteix_cod_linia    EXCEPTION;
   esNul_cod_linia         EXCEPTION;
   errorEscriptura         EXCEPTION;
   no_actiu				   EXCEPTION;
   
BEGIN

  --El cod_linia pot ser nul
  IF ( n_cod_linia  IS NULL) 
  THEN 
    RAISE esNul_cod_linia;
  END IF; 
  
  --El codi de la linia ha d'existir
  SELECT COUNT (*)
     INTO n_cod_linia_test
     FROM LINIA
     WHERE cod_linia = n_cod_linia;

   IF (n_cod_linia_test = 0)
   THEN
      RAISE noExisteix_cod_linia;
   END IF;
   
   --No es pot activar una linia inactiva, en aquest cas
   --s'haur de fer una alta nova
   IF (n_estat = 'actiu') THEN
   RAISE no_actiu;
   END IF;
   
   
   -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;
  
   --Fem la baixa logica de linia actualitzatn el camp estat d'actiu a inactiu
   UPDATE LINIA
   SET estat = n_estat
   WHERE cod_linia = n_cod_linia;
   
   -- Fi de les operacions
   rst := 'OK: La linia s inactiva';

    p_insertLog('p_baixaLinia', 'n_cod_linia: ' || n_cod_linia, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_linia
   THEN
      rst := 'ERROR: El codi de la linia no existeix';
      
      p_insertLog('p_baixaLinia', 'n_cod_linia: ' || n_cod_linia, rst);
      ROLLBACK;
	  
	  
   WHEN no_actiu
   THEN
      rst := 'ERROR: No es pot activa una linia inactiva';
      
      p_insertLog('p_baixaLinia', 'n_cod_linia: ' || n_cod_linia, rst);
      ROLLBACK;	  
   
   WHEN esNul_cod_linia
   THEN
      rst := 'ERROR: El codi de la linia no pot ser nul';
      
      p_insertLog('p_baixaLinia', 'n_cod_linia: ' || n_cod_linia, rst);
      ROLLBACK;
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
      
      p_insertLog('p_baixaLinia', 'n_cod_linia: ' || n_cod_linia, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
      p_insertLog('p_baixaLinia', 'n_cod_linia: ' || n_cod_linia, rst);
      ROLLBACK;

END p_baixaLinia;
/





-------------------------------------
--p_baixaModComptador
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_baixaModComptador (
   n_cod_model          IN    INTEGER
  
)
IS  

--Variables locasl
   n_cod_model_test         INTEGER;
   err_msg                  VARCHAR2(100);
   rst                      VARCHAR2(100);
   noExisteix_cod_model     EXCEPTION;
   esNul_cod_model          EXCEPTION;
   
BEGIN

  --El cod_model no pot ser nul
  IF ( n_cod_model  IS NULL) 
  THEN 
    RAISE esNul_cod_model;
  END IF; 
  
  --El codi del model ha d'existir
  SELECT COUNT (*)
     INTO n_cod_model_test
     FROM MODEL_COMPTADOR
     WHERE cod_model = n_cod_model;

   IF (n_cod_model_test = 0)
   THEN
      RAISE noExisteix_cod_model;
   END IF;
  
   --Esborrem el model
   DELETE FROM MODEL_COMPTADOR 
   WHERE cod_model = n_cod_model;
   
   -- Fi de les operacions
   rst := 'OK';

    p_insertLog('p_baixaModComptador', 'n_cod_model: ' || n_cod_model, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_model
   THEN
      rst := 'ERROR: El codi del model no existeix';
      
      p_insertLog('p_baixaModComptador', 'n_cod_model: ' || n_cod_model, rst);
      ROLLBACK;
   
   WHEN esNul_cod_model
   THEN
      rst := 'ERROR: El codi del model s nul';
      
      p_insertLog('p_baixaModComptador', 'n_cod_model: ' || n_cod_model, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
      p_insertLog('p_baixaModComptador', 'n_cod_model: ' || n_cod_model, rst);
      ROLLBACK;


END p_baixaModComptador;
/





-------------------------------------
--p_baixaNuclear
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_baixaNuclear (
    n_cod_c_produccio  IN INTEGER  
)
IS  

--Variables locals
   n_cod_c_produccio_test          INTEGER;
   err_msg                         VARCHAR2(100);
   rst                             VARCHAR2(100);
   noExisteix_cod_c_produccio      EXCEPTION;
   esNul_cod_c_produccio           EXCEPTION;
   
BEGIN

  --Verifiquem que el ccod_c_produccio no sigui nul
  IF ( n_cod_c_produccio  IS NULL) 
  THEN 
    RAISE esNul_cod_c_produccio;
  END IF; 
  
  --El codi de la central de producci ha d'existir
  SELECT COUNT (*)
     INTO n_cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
     WHERE cod_c_produccio = n_cod_c_produccio;

   IF (n_cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
  
   --Esborrem la central de producci
   DELETE FROM NUCLEAR
   WHERE cod_c_produccio = n_cod_c_produccio;
   
   -- Fi de les operacions
   rst := 'OK: S''ha donat de baixa la central nuclear';

   p_insertLog('p_baixaNuclear', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de producci no existeix';
      
         p_insertLog('p_baixaNuclear', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;
   
   WHEN esNul_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de producci no pot ser nul';
      
         p_insertLog('p_baixaNuclear', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
       p_insertLog('p_baixaNuclear', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;




END p_baixaNuclear;
/




-------------------------------------
--p_baixaSolar
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_baixaSolar (
    n_cod_c_produccio  IN INTEGER  
)
IS  

--Variables locals
   n_cod_c_produccio_test          INTEGER;
   err_msg                         VARCHAR2(100);
   rst                             VARCHAR2(100);
   noExisteix_cod_c_produccio      EXCEPTION;
   esNul_cod_c_produccio           EXCEPTION;
   
BEGIN

  --Verifiquem que el ccod_c_produccio no sigui nul
  IF ( n_cod_c_produccio  IS NULL) 
  THEN 
    RAISE esNul_cod_c_produccio;
  END IF; 
  
  --El codi de la central de producci ha d'existir
  SELECT COUNT (*)
     INTO n_cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
     WHERE cod_c_produccio = n_cod_c_produccio;

   IF (n_cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
  
   --Esborrem la central termica
   DELETE FROM SOLAR 
   WHERE cod_c_produccio = n_cod_c_produccio;
   
   -- Fi de les operacions
   rst := 'OK: S''ha donat de baixa la central solar';

   p_insertLog('p_baixaSolar', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de producci no existeix';
      
     p_insertLog('p_baixaSolar', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;
   
   WHEN esNul_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de producci no pot ser nul';
      
        p_insertLog('p_baixaSolar', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
         p_insertLog('p_baixaSolar', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;


END p_baixaSolar;
/





-------------------------------------
--p_baixaTermica
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_baixaTermica (
    n_cod_c_produccio  IN INTEGER  
)
IS  

--Variables locals
   n_cod_c_produccio_test          INTEGER;
   err_msg                         VARCHAR2(100);
   rst                             VARCHAR2(100);
   noExisteix_cod_c_produccio      EXCEPTION;
   esNul_cod_c_produccio           EXCEPTION;
   
BEGIN

  --Verifiquem que el ccod_c_produccio no sigui nul
  IF ( n_cod_c_produccio  IS NULL) 
  THEN 
    RAISE esNul_cod_c_produccio;
  END IF; 
  
  --El codi de la central de producci ha d'existir
  SELECT COUNT (*)
     INTO n_cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
     WHERE cod_c_produccio = n_cod_c_produccio;

   IF (n_cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
  
   --Esborrem la central termica
   DELETE FROM TERMICA 
   WHERE cod_c_produccio = n_cod_c_produccio;
   
   -- Fi de les operacions
   rst := 'OK: S''ha donat de baixa la central termica';

   p_insertLog('p_baixaTermica', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de producci no existeix';
      
         p_insertLog('p_baixaTermica', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;
   
   WHEN esNul_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de producci no pot ser nul';
      
        p_insertLog('p_baixaTermica', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
       p_insertLog('p_baixaTermica', 'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;


END p_baixaTermica;
/





-------------------------------------
--p_modCarbo
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_modCarbo (
  n_cod_c_produccio    IN INTEGER,
   n_emissio_co2       IN NUMBER
 

)
IS

-- Variables locals
        rst                             VARCHAR2(2000);
        cod_c_produccio_test            INTEGER;
        err_msg                         VARCHAR2(100);
        existeixenCampsNuls             EXCEPTION;
        noExisteix_cod_c_produccio      EXCEPTION;
    
     
BEGIN


  --verifiquem que els camps no nuls no siguin nuls
  IF (n_cod_c_produccio IS NULL) OR 
     (n_emissio_co2  IS NULL) 
  THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que el codi de la central de carb existeix
  SELECT COUNT (*)
  INTO cod_c_produccio_test
  FROM CARBO
  WHERE cod_c_produccio = n_cod_c_produccio;

  IF (cod_c_produccio_test = 0)
   THEN
  RAISE noExisteix_cod_c_produccio;
  END IF;

   
    -- Modifiquem la central de carb
   UPDATE CARBO
   SET emissio_co2  = n_emissio_co2           
   WHERE cod_c_produccio = n_cod_c_produccio;

   -- Fi de les operacions
   rst := 'OK: La central de carbo s''ha modificat correctament';

   p_insertLog('p_modCarbo',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_emissio_co2 : ' || n_emissio_co2, rst);
   COMMIT;

   EXCEPTION

   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi_c_produccio no existeix';
      
   p_insertLog('p_modCarbo',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_emissio_co2 : ' || n_emissio_co2, rst);
      ROLLBACK;

  
 
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
   p_insertLog('p_modCarbo',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_emissio_co2 : ' || n_emissio_co2, rst);
      ROLLBACK;

    
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modCarbo',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_emissio_co2 : ' || n_emissio_co2, rst);
      ROLLBACK;


END p_modCarbo;
/





-------------------------------------
--p_modCentralDistribucio
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_modCentralDistribucio (
    n_cod_c_distribucio   IN INTEGER,
    n_maxim_energia       IN NUMBER,
    n_adrea              IN VARCHAR2,
    n_localitat           IN VARCHAR2,
    n_cp                  IN VARCHAR2,
    n_pais                IN VARCHAR2

)
IS  

-- Variables locals
     n_maxim_energia_ins           NUMBER;
     n_adrea_ins                  VARCHAR2(100 CHAR);
     n_localitat_ins               VARCHAR2(100 CHAR); 
     n_cp_ins                      VARCHAR2(5 CHAR); 
     n_pais_ins                    VARCHAR2(50 CHAR);
     rst                           VARCHAR2(2000);
     cod_c_distribucio_test        INTEGER;
     err_msg                       VARCHAR2(100);
     noExisteix_cod_c_distribucio  EXCEPTION;
     existeixenCampsNuls           EXCEPTION;
  
  
   
BEGIN
   n_maxim_energia_ins := n_maxim_energia;
   n_adrea_ins := SUBSTR (n_adrea, 1, 100);
   n_localitat_ins  := SUBSTR (n_localitat, 1, 100);
   n_cp_ins := SUBSTR (n_cp, 1, 5);
   n_pais_ins := SUBSTR (n_pais, 1, 50);

   
  --verifiquem que els camps no nuls no siguin nuls
  IF (n_cod_c_distribucio IS NULL) 
     OR (n_maxim_energia IS NULL)
     OR (n_adrea IS NULL)
     OR (n_localitat IS NULL)
     OR (n_pais IS NULL)
  THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que la central de distribucio a modificar existeix
  SELECT COUNT (*)
     INTO cod_c_distribucio_test
     FROM CENTRAL_DISTRIBUCIO
     WHERE cod_c_distribucio = n_cod_c_distribucio;

   IF (cod_c_distribucio_test = 0)
   THEN
      RAISE noExisteix_cod_c_distribucio;
   END IF;
   

   
    -- modifiquem la central de distribucio
   UPDATE CENTRAL_DISTRIBUCIO
   SET maxim_energia = n_maxim_energia_ins, 
       adrea = n_adrea_ins, 
       localitat = n_localitat_ins, 
       cp = n_cp_ins, 
       pais = n_pais_ins
   WHERE cod_c_distribucio = n_cod_c_distribucio;

   
   -- Fi de les operacions
   rst := 'OK, central distribucio modificada correctament';

   p_insertLog('p_modCentralDistribucio',  
                'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
                'n_maxim_energia: ' || n_maxim_energia || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais, rst);
   COMMIT;
EXCEPTION
   WHEN noExisteix_cod_c_distribucio
   THEN
      rst := 'ERROR: El codi de la central de distriucio no existeix';
      
   p_insertLog('p_modCentralDistribucio',  
                'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
                'n_maxim_energia: ' || n_maxim_energia || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais, rst);
      ROLLBACK;
  
 
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
   p_insertLog('p_modCentralDistribucio',  
                'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
                'n_maxim_energia: ' || n_maxim_energia || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais, rst);
      ROLLBACK;
   
 
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modCentralDistribucio',  
                'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
                'n_maxim_energia: ' || n_maxim_energia || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais, rst);
      ROLLBACK;

END p_modCentralDistribucio;
/





-------------------------------------
--p_modCentralProduccio
-------------------------------------
CREATE OR REPLACE PROCEDURE p_modCentralProduccio (

    n_cod_c_produccio     IN INTEGER,
	n_data_inspeccio      IN DATE,
    n_capacitat_maxima    IN NUMBER,    
    n_adrea              IN VARCHAR2,
    n_localitat           IN VARCHAR2,
    n_cp                  IN VARCHAR2,
    n_pais                IN VARCHAR2 
)
IS

-- Variables locals
   n_adrea_ins                   VARCHAR (100 CHAR);
   n_localitat_ins                VARCHAR (100 CHAR);
   n_cp_ins                       VARCHAR (5);
   n_pais_ins                     VARCHAR (50);
   rst                            VARCHAR2(2000);
   err_msg                        VARCHAR2(100);
   cod_c_produccio_test           INTEGER;
   campsBuits                     EXCEPTION;
   noExisteixDataInspeccio        EXCEPTION;
   noExisteix_cod_c_produccio     EXCEPTION;
  
   
BEGIN

   n_adrea_ins := SUBSTR (n_adrea, 1, 100); 
   n_localitat_ins := SUBSTR (n_localitat, 1, 100);
   n_cp_ins := SUBSTR (n_cp, 1, 5);
   n_pais_ins  := SUBSTR (n_pais, 1, 50);

   --verifiquem que els camps no siguin nuls
   IF (n_data_inspeccio IS NULL) OR
   (n_capacitat_maxima IS NULL) OR
   (n_adrea IS NULL) OR
   (n_localitat IS NULL) OR
   (n_pais IS NULL)

   THEN
       RAISE campsBuits;
   END IF;  
  
  --verifiquem que la central de produccio a modificar existeix
  SELECT COUNT (*)
  INTO cod_c_produccio_test
  FROM CENTRAL_PRODUCCIO
  WHERE cod_c_produccio = n_cod_c_produccio;

  IF (cod_c_produccio_test = 0)
   THEN
  RAISE noExisteix_cod_c_produccio;
  END IF;
  
  
   -- Verifiquem que la data_inspeccio existeix (no pot ser superior  a la data del sistema(sysdate))
   IF (n_data_inspeccio > sysdate) THEN 
     RAISE noExisteixDataInspeccio;
   END IF; 
     
   -- Modifiquem la central de produccio
   UPDATE CENTRAL_PRODUCCIO
   SET 
    data_inspeccio = n_data_inspeccio,
    capacitat_maxima = n_capacitat_maxima,
    adrea = n_adrea,
    localitat = n_localitat,
    cp = n_cp,
    pais = n_pais
    
   WHERE cod_c_produccio = n_cod_c_produccio;

   -- Fi de les operacions
   rst := 'OK, central de producci modificada correctament';

    p_insertLog('p_altaCentralProduccio', 
       'n_data_inspeccio: ' || n_data_inspeccio || ', ' || 
          'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais, rst);
   COMMIT;

   EXCEPTION

   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de proudccio no existeix';
      
    p_insertLog('p_altaCentralProduccio', 
       'n_data_inspeccio: ' || n_data_inspeccio || ', ' || 
          'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais, rst);
      ROLLBACK;
   

   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
    p_insertLog('p_altaCentralProduccio', 
       'n_data_inspeccio: ' || n_data_inspeccio || ', ' || 
          'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais, rst);
      ROLLBACK;
   
   WHEN noExisteixDataInspeccio
   THEN
      rst := 'ERROR: La data_inspeccio introduda no s correcte';
      
    p_insertLog('p_altaCentralProduccio', 
       'n_data_inspeccio: ' || n_data_inspeccio || ', ' || 
          'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais, rst);
   ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
    p_insertLog('p_altaCentralProduccio', 
       'n_data_inspeccio: ' || n_data_inspeccio || ', ' || 
          'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais, rst);
      ROLLBACK;

END p_modCentralProduccio;
/






-------------------------------------
--p_modClient
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_modClient (
    n_cod_client   IN INTEGER,
    n_nif          IN VARCHAR2,
    n_nom          IN VARCHAR2,
    n_cognoms      IN VARCHAR2,
    n_adrea       IN VARCHAR2,
    n_localitat    IN VARCHAR2,
    n_cp           IN VARCHAR2,
    n_pais         IN VARCHAR2,
    n_telefon      IN VARCHAR2,
 n_estat     IN VARCHAR2
)
IS  

-- Variables locals
   n_nif_ins                     VARCHAR2(12 CHAR); 
   n_nom_ins                     VARCHAR2(15 CHAR); 
   n_cognoms_ins                 VARCHAR2(50 CHAR); 
   n_adrea_ins                  VARCHAR2(100 CHAR);
   n_localitat_ins               VARCHAR2(100 CHAR); 
   n_cp_ins                      VARCHAR2(5 CHAR); 
   n_pais_ins                    VARCHAR2(50 CHAR);
   n_telefon_ins                 VARCHAR2(20 CHAR);
   rst                           VARCHAR2(2000);
   cod_client_test               INTEGER;
   nif_test                      VARCHAR2(12 CHAR); 
   err_msg                       VARCHAR2(100);
   noExisteix_cod_client         EXCEPTION;
   existeixenCampsNuls           EXCEPTION;
  
   
BEGIN
   n_nif_ins := SUBSTR (n_nif, 1, 12);
   n_nom_ins := SUBSTR (n_nom, 1, 15);
   n_cognoms_ins := SUBSTR (n_cognoms, 1, 50);
   n_adrea_ins := SUBSTR (n_adrea, 1, 100);
   n_localitat_ins  := SUBSTR (n_localitat, 1, 100);
   n_cp_ins := SUBSTR (n_cp, 1, 5);
   n_pais_ins := SUBSTR (n_pais, 1, 50);
   n_telefon_ins := SUBSTR (n_telefon, 1, 20);
   
  --verifiquem que els camps no nuls no siguin nuls
  IF (n_nif IS NULL) 
     OR (n_nom IS NULL)
     OR (n_cognoms IS NULL)
     OR (n_adrea IS NULL)
     OR (n_localitat IS NULL)
     OR (n_cp IS NULL)
     OR (n_pais IS NULL)
     OR (n_telefon IS NULL)
  OR (n_estat IS NULL)
  THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que el client a modificar existeix
  SELECT COUNT (*)
     INTO cod_client_test
     FROM CLIENT
     WHERE cod_client = n_cod_client;

   IF (cod_client_test = 0)
   THEN
      RAISE noExisteix_cod_client;
   END IF;
   
    -- modifiquem el client
   UPDATE CLIENT
   SET nif = n_nif_ins, 
       nom = n_nom_ins,
       cognoms = n_cognoms_ins, 
       adrea = n_adrea_ins, 
       localitat = n_localitat_ins, 
       cp = n_cp_ins, 
       pais = n_pais_ins, 
       telefon = n_telefon_ins 
 WHERE cod_client = n_cod_client;


 
   -- Fin de las operaciones
   rst := 'OK, client modificat correctament';

   p_insertLog('p_modClient',  
                'n_cod_client: ' || n_cod_client || ', ' ||
                'n_nom: ' || n_nom || ', ' ||
                'n_cognoms: ' || n_cognoms || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ' ||
    'n_estat: ' || n_estat, rst);
   COMMIT;
EXCEPTION
   WHEN noExisteix_cod_client
   THEN
      rst := 'ERROR: El codi del client no existeix';
      
   p_insertLog('p_modClient',  
                'n_cod_client: ' || n_cod_client || ', ' ||
                'n_nom: ' || n_nom || ', ' ||
                'n_cognoms: ' || n_cognoms || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
  
 
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
   p_insertLog('p_modClient',  
                'n_cod_client: ' || n_cod_client || ', ' ||
                'n_nom: ' || n_nom || ', ' ||
                'n_cognoms: ' || n_cognoms || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modClient',  
                'n_cod_client: ' || n_cod_client || ', ' ||
                'n_nom: ' || n_nom || ', ' ||
                'n_cognoms: ' || n_cognoms || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
END p_modClient;
/




-------------------------------------
--p_modComptador
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_modComptador (
 n_cod_comptador    IN INTEGER,  
 n_cod_model        IN INTEGER,
 n_referencia       IN VARCHAR2,
 n_adrea           IN VARCHAR2,
 n_localitat        IN VARCHAR2,
 n_cp               IN VARCHAR2,
 n_pais             IN VARCHAR2,
 n_estat            IN VARCHAR2
)
IS

-- Variables locals
   n_cod_model_ins               INTEGER;
   n_referencia_ins              VARCHAR2(20 CHAR);
   n_adrea_ins                  VARCHAR2(100 CHAR);
   n_localitat_ins               VARCHAR2(100 CHAR);
   n_cp_ins                      VARCHAR2 (5 CHAR);
   n_pais_ins                    VARCHAR2 (50 CHAR);
   n_estat_ins                   VARCHAR2 (7 CHAR);
   rst                           VARCHAR2(2000);
   cod_comptador_test            INTEGER;
   cod_model_test                INTEGER;
   err_msg                       VARCHAR2(100);
   noExisteix_cod_comptador      EXCEPTION;
   existeixenCampsNuls           EXCEPTION;
   noExisteix_cod_model          EXCEPTION;
  
   
BEGIN
   n_cod_model_ins := n_cod_model; 
   n_referencia_ins := n_referencia;
   n_referencia_ins  := SUBSTR (n_referencia, 1, 20);
   n_adrea_ins := SUBSTR (n_adrea, 1, 100);
   n_localitat_ins := SUBSTR (n_localitaT, 1, 100);
   n_cp_ins := SUBSTR (n_cp, 1, 5);
   n_pais_ins  := SUBSTR (n_pais, 1, 50);
   n_estat_ins := SUBSTR (n_estat, 1, 7);
   
  --verifiquem que els camps no nuls no siguin nuls
  IF (n_cod_model IS NULL) OR (n_estat IS NULL) OR (n_referencia IS NULL)THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que el comptador a modificar existeix
  SELECT COUNT (*)
     INTO cod_comptador_test
     FROM COMPTADOR
     WHERE cod_comptador = n_cod_comptador;

   IF (cod_comptador_test = 0)
   THEN
      RAISE noExisteix_cod_comptador;
   END IF;
   
    -- Verifiquem que el codi_model existeix a la taula MODEL_COMPTADOR
   
   SELECT COUNT (*)
     INTO cod_model_test
     FROM MODEL_COMPTADOR
   WHERE cod_model = n_cod_model;

   IF (cod_model_test = 0)
   THEN
      RAISE noExisteix_cod_model;
   END IF;
   
    -- modifiquem el comptador
   UPDATE COMPTADOR
   SET cod_model = n_cod_model_ins, 
       referencia = n_referencia_ins,
   -- consum_mes = n_consum_mes,
   -- consum_absolut = n_consum_absolut,
   -- darrera_lectura = n_darrera_lectura,
    adrea  = n_adrea_ins,
    localitat = n_localitat_ins, 
       cp = n_cp_ins, 
       pais = n_pais_ins, 
       estat = n_estat_ins    
    WHERE cod_comptador = n_cod_comptador;

   -- Fi de les operacions
   rst := 'OK, comptador modificat correctament';

   p_insertLog('p_modComptador',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
    --'consum_mes: ' || n_consum_mes || ', ' ||
    --'consum_absolut: ' || n_consum_absolut || ', ' ||
    --'darrera_lectura: ' || n_darrera_lectura || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
   COMMIT;
EXCEPTION
   WHEN noExisteix_cod_comptador
   THEN
      rst := 'ERROR: El codi del comptador no existeix';
      
   p_insertLog('p_modComptador',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
    --'consum_mes: ' || n_consum_mes || ', ' ||
    --'consum_absolut: ' || n_consum_absolut || ', ' ||
    --'darrera_lectura: ' || n_darrera_lectura || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
      ROLLBACK;
  
        WHEN noExisteix_cod_model    
   THEN
      rst := 'ERROR: No existeix el codi del model';
   
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
               -- 'n_data: ' || n_data || ', ' ||
              --  'n_consum_mes: ' || n_consum_mes || ', ' ||
    --'n_consum_absolut: ' || n_consum_absolut || ', ' ||
    --'n_darrera_lectura: ' || n_darrera_lectura || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
    'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);

      ROLLBACK; 
 
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
   p_insertLog('p_modComptador',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
    --'consum_mes: ' || n_consum_mes || ', ' ||
    --'consum_absolut: ' || n_consum_absolut || ', ' ||
    --'darrera_lectura: ' || n_darrera_lectura || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
      ROLLBACK;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modComptador',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
    --'consum_mes: ' || n_consum_mes || ', ' ||
    --'consum_absolut: ' || n_consum_absolut || ', ' ||
    --'darrera_lectura: ' || n_darrera_lectura || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
      ROLLBACK;

END p_modComptador;
/





-------------------------------------
--p_modContracte
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_modContracte (
    n_cod_contracte    IN INTEGER,
    n_cod_comptador    IN INTEGER,  
    n_cod_client       IN INTEGER,
    n_data_alta        IN DATE,
    n_data_fi          IN DATE,
    n_potencia         IN NUMBER

)
IS

-- Variables locals
     rst                             VARCHAR2(2000);
     cod_contracte_test              INTEGER;
     cod_comptador_test              INTEGER;
     cod_client_test                 INTEGER;
     err_msg                         VARCHAR2(100);
     existeixenCampsNuls             EXCEPTION;
     noExisteix_cod_contracte        EXCEPTION;
     noExisteix_cod_comptador        EXCEPTION;
     noExisteix_cod_client           EXCEPTION; 
     noExisteixDataFi                EXCEPTION;
     noExisteixDataAlta              EXCEPTION;
     datesIncorrectes                EXCEPTION;
  
   
BEGIN


  --verifiquem que els camps no nuls no siguin nuls
  IF (n_cod_comptador IS NULL) OR 
     (n_cod_client IS NULL) OR 
  (n_data_alta IS NULL) OR
  (n_potencia IS NULL) 
  THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que el contracte a modificar existeix
  SELECT COUNT (*)
  INTO cod_contracte_test
  FROM CONTRACTE
  WHERE cod_contracte = n_cod_contracte;

  IF (cod_contracte_test = 0)
   THEN
  RAISE noExisteix_cod_contracte;
  END IF;
   
  -- Verifiquem que el codi_comptador existeix en la taula COMPTADOR   
  SELECT COUNT (*)
  INTO cod_comptador_test
  FROM COMPTADOR
  WHERE cod_comptador = n_cod_comptador;

   IF (cod_comptador_test = 0)
   THEN
      RAISE noExisteix_cod_comptador;
   END IF;
   
    -- Verifiquem que el codi_client existeix en la taula CLIENT   
   SELECT COUNT (*)
     INTO cod_client_test
     FROM CLIENT
   WHERE cod_client = n_cod_client;

   IF (cod_client_test = 0)
   THEN
      RAISE noExisteix_cod_client;
   END IF;
   
     -- Verifiquem que la data_alta existeix (no pot ser superior  a la data del sistema(sysdate))
   IF (n_data_alta > sysdate) THEN 
     RAISE noExisteixDataAlta;
   END IF; 
     
   -- Verifiquem que la data_fi existeix (no pot ser superior  a la data del sistema(sysdate))
   IF (n_data_fi > sysdate) THEN 
     RAISE noExisteixDataFi;
   END IF; 
   
   -- Verifiquem que la data_fi del contracte s superior a la data_alta
   IF (n_data_fi <= n_data_alta) THEN
      RAISE datesIncorrectes;
   END IF;
   
    -- Modifiquem el contracte
   UPDATE CONTRACTE
   SET cod_comptador = n_cod_comptador,
       cod_client = n_cod_client,
    data_alta = n_data_alta,
    data_fi = n_data_fi,
    potencia = n_potencia        
   WHERE cod_contracte = n_cod_contracte;

   -- Fi de les operacions
   rst := 'OK, contracte modificat correctament';

   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
   COMMIT;

   EXCEPTION

   WHEN noExisteix_cod_contracte
   THEN
      rst := 'ERROR: El codi del contracte no existeix';
      
   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
      ROLLBACK;

   WHEN noExisteix_cod_comptador
   THEN
      rst := 'ERROR: El codi del comptador no existeix';
      
   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
      ROLLBACK;
  
        WHEN noExisteix_cod_client    
   THEN
      rst := 'ERROR: No existeix el codi del client';
   
   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
      ROLLBACK; 
 
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
      ROLLBACK;
   
   WHEN noExisteixDataAlta
   THEN
      rst := 'ERROR: La data_alta introduda no s correcte';
      
   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
   ROLLBACK;
   
   WHEN noExisteixDataFi
   THEN
      rst := 'ERROR: La data fi introduda no s correcte';
      
   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
   ROLLBACK;
   
   WHEN datesIncorrectes
   THEN
      rst := 'ERROR: La data_fi no pot ser inferior a la data_fi del contracte';
      
    p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
   ROLLBACK;
    
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
      ROLLBACK;


END p_modContracte;
/





-------------------------------------
--p_modEolica
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_modEolica  (
   n_cod_c_produccio   IN INTEGER,
   n_num_molins        IN NUMBER


)
IS

-- Variables locals
        rst                             VARCHAR2(2000);
        cod_c_produccio_test            INTEGER;
        err_msg                         VARCHAR2(100);
        existeixenCampsNuls             EXCEPTION;
        noExisteix_cod_c_produccio      EXCEPTION;
    
     
BEGIN


  --verifiquem que els camps no nuls no siguin nuls
  IF (n_cod_c_produccio IS NULL) OR 
     (n_num_molins  IS NULL) 
  THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que el codi de la central de carb existeix
  SELECT COUNT (*)
  INTO cod_c_produccio_test
  FROM EOLICA
  WHERE cod_c_produccio = n_cod_c_produccio;

  IF (cod_c_produccio_test = 0)
   THEN
  RAISE noExisteix_cod_c_produccio;
  END IF;

   
    -- Modifiquem la central eolica
   UPDATE EOLICA
   SET num_molins  = n_num_molins            
   WHERE cod_c_produccio = n_cod_c_produccio;

   -- Fi de les operacions
   rst := 'OK: La central eolica s''ha modificat correctament';

   p_insertLog('p_modEolica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_num_molins : ' || n_num_molins, rst);
   COMMIT;

   EXCEPTION

   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi_c_produccio no existeix';
      
   p_insertLog('p_modEolica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_num_molins : ' || n_num_molins, rst);
      ROLLBACK;

  
 
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
   p_insertLog('p_modEolica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_num_molins : ' || n_num_molins, rst);
      ROLLBACK;

    
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modEolica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_num_molins : ' || n_num_molins, rst);
      ROLLBACK;


END p_modEolica;
/





-------------------------------------
--p_modLinia
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_modLinia (

    n_cod_linia        IN INTEGER,
    n_capacitat_maxima IN NUMBER 


)
IS

-- Variables locals
    rst                            VARCHAR2(2000);
    cod_linia_test                 INTEGER;
    err_msg                        VARCHAR2(100);
    existeixenCampsNuls            EXCEPTION;
    noExisteix_cod_linia           EXCEPTION;

   
BEGIN


  --verifiquem que els camps no nuls no siguin nuls
  IF (n_cod_linia IS NULL) OR 
     (n_capacitat_maxima IS NULL)  
  THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que la linia a modificar existeix
  SELECT COUNT (*)
  INTO cod_linia_test
  FROM LINIA
  WHERE cod_linia = n_cod_linia;

  IF (cod_linia_test = 0)
   THEN
  RAISE noExisteix_cod_linia;
  END IF;
 
  
    -- Modifiquem la linia
   UPDATE LINIA
   SET capacitat_maxima = n_capacitat_maxima         
   WHERE cod_linia = n_cod_linia;

   -- Fi de les operacions
   rst := 'OK: La linia s''ha modificat correctament';

   p_insertLog('p_modLinia',  
                'n_cod_linia: ' || n_cod_linia || ', ' ||
                'n_capacitat_maxima: ' || n_capacitat_maxima, rst);
   COMMIT;

   EXCEPTION

   WHEN noExisteix_cod_linia
   THEN
      rst := 'ERROR: El codi de la linia no existeix';
      
   p_insertLog('p_modLinia',  
                'n_cod_linia: ' || n_cod_linia || ', ' ||
                'n_capacitat_maxima: ' || n_capacitat_maxima, rst);
      ROLLBACK;

  
 
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
   p_insertLog('p_modLinia',  
                'n_cod_linia: ' || n_cod_linia || ', ' ||
                'n_capacitat_maxima: ' || n_capacitat_maxima, rst);
      ROLLBACK;
   
    
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modLinia',  
                'n_cod_linia: ' || n_cod_linia || ', ' ||
                'n_capacitat_maxima: ' || n_capacitat_maxima, rst);
      ROLLBACK;



END p_modLinia;
/




-------------------------------------
--p_modModelComptador
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_modModelComptador (
   n_cod_model     IN INTEGER,
   n_data_fab      IN DATE,
   n_referencia    IN VARCHAR2,
   n_empresa_fab   IN VARCHAR2,
   n_descripcio    IN VARCHAR2,
   n_adrea        IN VARCHAR2,
   n_localitat     IN VARCHAR2,
   n_cp            IN VARCHAR2,
   n_pais          IN VARCHAR2,
   n_telefon       IN VARCHAR2
   
)
IS   


-- Variables locals

   n_data_fab_ins                DATE; 
   n_referencia_ins              VARCHAR2(20 CHAR); 
   n_empresa_fab_ins             VARCHAR2(100 CHAR); 
   n_descripcio_ins              VARCHAR (100 CHAR);
   n_adrea_ins                  VARCHAR2(100 CHAR);
   n_localitat_ins               VARCHAR2(100 CHAR); 
   n_cp_ins                      VARCHAR2(5 CHAR); 
   n_pais_ins                    VARCHAR2(50 CHAR);
   n_telefon_ins                 VARCHAR2(20 CHAR);
   rst                           VARCHAR2(2000);
   cod_model_test                INTEGER;
   referencia_test               VARCHAR2(20 CHAR); 
   err_msg                       VARCHAR2(100);
   noExisteix_cod_model          EXCEPTION;
   existeixenCampsNuls           EXCEPTION;
   
  
  BEGIN
   n_referencia_ins := SUBSTR (n_referencia, 1, 20);
   n_empresa_fab_ins := SUBSTR (n_empresa_fab, 1, 100);
   n_descripcio_ins := SUBSTR (n_descripcio, 1, 100);
   n_adrea_ins := SUBSTR (n_adrea, 1, 100);
   n_localitat_ins  := SUBSTR (n_localitat, 1, 100);
   n_cp_ins := SUBSTR (n_cp, 1, 96);
   n_pais_ins := SUBSTR (n_pais, 1, 50);
   n_telefon_ins := SUBSTR (n_telefon, 1, 20);
   
  --verifiquem que els camps no nuls no siguin nuls
  IF (n_data_fab IS NULL) 
     OR (n_referencia IS NULL)
     OR (n_empresa_fab IS NULL)
  OR (n_descripcio IS NULL)
     OR (n_adrea IS NULL)
     OR (n_localitat IS NULL)
     OR (n_cp IS NULL)
  OR (n_pais IS NULL)
  OR (n_telefon IS NULL)
  THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que el model a modificar existeix
  SELECT COUNT (*)
     INTO cod_model_test
     FROM MODEL_COMPTADOR
     WHERE cod_model = n_cod_model;

   IF (cod_model_test = 0)
   THEN
      RAISE noExisteix_cod_model;
   END IF;
   
  
    -- modifiquem el model
   UPDATE MODEL_COMPTADOR
   SET data_fab = n_data_fab, 
       referencia = n_referencia,
       empresa_fab = n_empresa_fab,
    descripcio = n_descripcio, 
       adrea = n_adrea_ins, 
       localitat = n_localitat_ins, 
       cp = n_cp_ins, 
       pais = n_pais_ins, 
       telefon = n_telefon_ins 
    WHERE cod_model = n_cod_model;

   -- Fi de les operacions
   rst := 'OK, model modificat correctament';

   p_insertLog('p_modModelComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
   COMMIT;
EXCEPTION
   WHEN noExisteix_cod_model
   THEN
      rst := 'ERROR: El codi del model no existeix';
      
        p_insertLog('p_modModelComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
      ROLLBACK;
  
  
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
          p_insertLog('p_modModelComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
      ROLLBACK;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
       p_insertLog('p_modModelComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
      ROLLBACK;

END p_modModelComptador;
/





-------------------------------------
--p_modNuclear
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_modNuclear (
   n_cod_c_produccio   IN INTEGER,
   n_energia_minima    IN NUMBER,
   n_rebuig            IN NUMBER

)
IS

-- Variables locals
        rst                             VARCHAR2(2000);
        cod_c_produccio_test            INTEGER;
        err_msg                         VARCHAR2(100);
        existeixenCampsNuls             EXCEPTION;
        noExisteix_cod_c_produccio      EXCEPTION;
    
     
BEGIN


  --verifiquem que els camps no nuls no siguin nuls
  IF (n_cod_c_produccio IS NULL) OR 
     (n_energia_minima IS NULL) OR 
  (n_rebuig IS NULL)
  THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que el codi a modificar existeix
  SELECT COUNT (*)
  INTO cod_c_produccio_test
  FROM NUCLEAR
  WHERE cod_c_produccio = n_cod_c_produccio;

  IF (cod_c_produccio_test = 0)
   THEN
  RAISE noExisteix_cod_c_produccio;
  END IF;

   
    -- Modifiquem la central nuclar
   UPDATE NUCLEAR
   SET energia_minima = n_energia_minima,
       rebuig = n_rebuig          
   WHERE cod_c_produccio = n_cod_c_produccio;

   -- Fi de les operacions
   rst := 'OK: La central nuclear s''ha modificat correctament';

   p_insertLog('p_modNuclear',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_energia_minima: ' || n_energia_minima || ', ' ||
                'n_rebuig: ' || n_rebuig, rst);
   COMMIT;

   EXCEPTION

   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi_c_produccio no existeix';
      
   p_insertLog('p_modNuclear',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_energia_minima: ' || n_energia_minima || ', ' ||
                'n_rebuig: ' || n_rebuig, rst);
      ROLLBACK;

  
 
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
   p_insertLog('p_modNuclear',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_energia_minima: ' || n_energia_minima || ', ' ||
                'n_rebuig: ' || n_rebuig, rst);
      ROLLBACK;

    
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modNuclear',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_energia_minima: ' || n_energia_minima || ', ' ||
                'n_rebuig: ' || n_rebuig, rst);
      ROLLBACK;



END p_modNuclear;
/





-------------------------------------
--p_modSolar
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_modSolar (
   n_cod_c_produccio    IN INTEGER,
   n_num_panells        IN NUMBER

)
IS

-- Variables locals
        rst                             VARCHAR2(2000);
        cod_c_produccio_test            INTEGER;
        err_msg                         VARCHAR2(100);
        existeixenCampsNuls             EXCEPTION;
        noExisteix_cod_c_produccio      EXCEPTION;
    
     
BEGIN


  --verifiquem que els camps no nuls no siguin nuls
  IF (n_cod_c_produccio IS NULL) OR 
     (n_num_panells  IS NULL) 
  THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que el codi de la central de carb existeix
  SELECT COUNT (*)
  INTO cod_c_produccio_test
  FROM SOLAR
  WHERE cod_c_produccio = n_cod_c_produccio;

  IF (cod_c_produccio_test = 0)
   THEN
  RAISE noExisteix_cod_c_produccio;
  END IF;

   
    -- Modifiquem la central solars
   UPDATE SOLAR
   SET num_panells  = n_num_panells            
   WHERE cod_c_produccio = n_cod_c_produccio;

   -- Fi de les operacions
   rst := 'OK: La central solar s''ha modificat correctament';

   p_insertLog('p_modSolar',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_num_panells : ' || n_num_panells, rst);
   COMMIT;

   EXCEPTION

   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi_c_produccio no existeix';
      
   p_insertLog('p_modSolar',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_num_panells : ' || n_num_panells, rst);
      ROLLBACK;

  
 
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
   p_insertLog('p_modSolar',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_num_panells : ' || n_num_panells, rst);
      ROLLBACK;

    
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modSolar',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_num_panells : ' || n_num_panells, rst);
      ROLLBACK;



END p_modSolar;
/




-------------------------------------
--p_modTermica
-------------------------------------
CREATE OR REPLACE PROCEDURE         p_modTermica (
   n_cod_c_produccio   IN INTEGER,
   n_emissioco2        IN NUMBER
 

)
IS

-- Variables locals
        rst                             VARCHAR2(2000);
        cod_c_produccio_test            INTEGER;
        err_msg                         VARCHAR2(100);
        existeixenCampsNuls             EXCEPTION;
        noExisteix_cod_c_produccio      EXCEPTION;
    
     
BEGIN


  --verifiquem que els camps no nuls no siguin nuls
  IF (n_cod_c_produccio IS NULL) OR 
     (n_emissioco2  IS NULL) 
  THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que el codi de la central termica existeix
  SELECT COUNT (*)
  INTO cod_c_produccio_test
  FROM TERMICA
  WHERE cod_c_produccio = n_cod_c_produccio;

  IF (cod_c_produccio_test = 0)
   THEN
  RAISE noExisteix_cod_c_produccio;
  END IF;

   
    -- Modifiquem la central termica
   UPDATE TERMICA
   SET emissioco2  = n_emissioco2            
   WHERE cod_c_produccio = n_cod_c_produccio;

   -- Fi de les operacions
   rst := 'OK: La central termica s''ha modificat correctament';

   p_insertLog('p_modTermica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_emissioco2 : ' || n_emissioco2, rst);
   COMMIT;

   EXCEPTION

   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi_c_produccio no existeix';
      
   p_insertLog('p_modTermica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_emissioco2 : ' || n_emissioco2, rst);
      ROLLBACK;

  
 
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
   p_insertLog('p_modTermica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_emissioco2 : ' || n_emissioco2, rst);
      ROLLBACK;

    
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modTermica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
                'n_emissioco2 : ' || n_emissioco2, rst);
      ROLLBACK;


END p_modTermica;
/

-------------------------------------
--p_retornLog
-------------------------------------

CREATE OR REPLACE PROCEDURE p_retornLog IS

PARAM_OUT VARCHAR2(20000);

BEGIN
	 
	 SELECT L.PARAM_OUT INTO PARAM_OUT FROM LOG L
	 WHERE L.COD_LOG = (SELECT MAX(LL.COD_LOG) FROM LOG LL);

	 dbms_output.put_line (PARAM_OUT);

END p_retornLog;

/


