CREATE OR REPLACE PROCEDURE         p_altaAlimentaCentral ( 
   n_cod_c_distribucio   IN INTEGER,
   n_cod_linia           IN INTEGER,
   n_cod_c_produccio     IN INTEGER,
   n_estat               IN VARCHAR2

)
IS
--variables locals-- 
   rst                            VARCHAR2(2000);
   cod_linia_test                 INTEGER;
   cod_c_distribucio_test         INTEGER;
   cod_c_produccio_test           INTEGER;
   err_msg                        VARCHAR2(100);
   campsBuits                     EXCEPTION;
   noExisteix_cod_linia           EXCEPTION;
   noExisteix_cod_c_distribucio   EXCEPTION;
   noExisteix_cod_c_produccio     EXCEPTION;
   errorEscriptura                EXCEPTION;


BEGIN

   --verifiquem que els camps no siguin nuls
   IF (n_cod_c_distribucio IS NULL) OR (n_cod_linia IS NULL) OR (n_cod_c_produccio IS NULL) 
      OR (n_estat IS NULL) THEN
     RAISE campsBuits;
   END IF; 
   
    -- Verifiquem que el cod_linia existeix a la taula LINIA
   
   SELECT COUNT (*)
     INTO cod_linia_test
     FROM LINIA
   WHERE cod_linia = n_cod_linia;

   IF (cod_linia_test = 0)
   THEN
      RAISE noExisteix_cod_linia;
   END IF;

  --verifiquem que no existeix el cod_c_distribucio a la taula CENTRAL_DISTRIBUCIO
  
   SELECT COUNT (*)
     INTO cod_c_distribucio_test
     FROM CENTRAL_DISTRIBUCIO
   WHERE cod_c_distribucio = n_cod_c_distribucio;

   IF (cod_c_distribucio_test = 0)
   THEN
      RAISE noExisteix_cod_c_distribucio;
   END IF;

     --verifiquem que no existeix el cod_c_produccio a la taula CENTRAL_PRODUCCIO
  
   SELECT COUNT (*)
     INTO cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
   WHERE cod_c_produccio = n_cod_c_produccio;

   IF (cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
   
    -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;

   -- inserim la relacio entre centrals de distribuci, la linia i les centrals de producci
  INSERT INTO ALIMENTA_CENTRAL (cod_c_distribucio, cod_linia, cod_c_produccio, estat) 
         VALUES (n_cod_c_distribucio, n_cod_linia, n_cod_c_produccio, lower(n_estat));
   
   -- Fi de les operacions
 
   rst := 'OK: S''ha inserit la relaci correctament';
   
   p_insertLog('p_altaAlimentaCentral',  
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
          'n_cod_linia: ' || n_cod_linia || ', ' ||
           'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
        'n_estat: ' || n_estat , rst);

   COMMIT;
   EXCEPTION
   
      WHEN noExisteix_cod_c_distribucio    
   THEN
      rst := 'ERROR: No existeix el codi de la central de distribucio';
   
   p_insertLog('p_altaAlimentaCentral',  
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
          'n_cod_linia: ' || n_cod_linia || ', ' ||
           'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
        'n_estat: ' || n_estat , rst);



      ROLLBACK; 
   
     WHEN noExisteix_cod_linia
     THEN
     rst := 'ERROR: No existeix el codi de la linia';
   
   p_insertLog('p_altaAlimentaCentral',  
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
          'n_cod_linia: ' || n_cod_linia || ', ' ||
           'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
        'n_estat: ' || n_estat , rst);


      ROLLBACK;
   
   
      
     WHEN noExisteix_cod_c_produccio
     THEN
     rst := 'ERROR: No existeix el codi de central de produccio';
   
   p_insertLog('p_altaAlimentaCentral',  
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
          'n_cod_linia: ' || n_cod_linia || ', ' ||
           'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
        'n_estat: ' || n_estat , rst);

      ROLLBACK;
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
   p_insertLog('p_altaAlimentaCentral',  
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
          'n_cod_linia: ' || n_cod_linia || ', ' ||
           'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
        'n_estat: ' || n_estat , rst);

      ROLLBACK;  
   
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
   p_insertLog('p_altaAlimentaCentral',  
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
          'n_cod_linia: ' || n_cod_linia || ', ' ||
           'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
        'n_estat: ' || n_estat , rst);

      ROLLBACK;  
   
   

   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   p_insertLog('p_altaAlimentaCentral',  
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
          'n_cod_linia: ' || n_cod_linia || ', ' ||
           'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
        'n_estat: ' || n_estat , rst);


   ROLLBACK;

END p_altaAlimentaCentral;
/

