CREATE OR REPLACE PROCEDURE         p_altaCentralDistribucio (
   n_maxim_energia       IN NUMBER,
   n_adrea              IN VARCHAR2,
   n_localitat           IN VARCHAR2,
   n_cp                  IN VARCHAR2,
   n_pais                IN VARCHAR2,
   n_estat               IN VARCHAR2
)
IS
--variables locals--
     rst                        VARCHAR2(2000);
     err_msg                    VARCHAR2(100);
     campsBuits                 EXCEPTION;
     errorEscriptura            EXCEPTION;

BEGIN
   --verifiquem que els camps no siguin nuls
   IF (n_maxim_energia IS NULL) OR (n_adrea IS NULL) OR (n_localitat IS NULL) OR (n_pais IS NULL)
      OR (n_estat IS NULL) THEN 
     RAISE campsBuits;
   END IF; 

 -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;
     
   -- inserim el nou client
   INSERT INTO CENTRAL_DISTRIBUCIO (maxim_energia, adrea, localitat, cp, pais, estat) 
          VALUES (n_maxim_energia, n_adrea, n_localitat, n_cp, n_pais, lower(n_estat));
   
   -- Fi de les operacions
 
   rst := 'OK: S''ha donat d''alta la central de distribuci';
   
   p_insertLog('p_altaCentralDistribucio', 'n_maxim_energia: ' || n_maxim_energia || ', ' || 'n_adrea:' || n_adrea || ', ' || 
               'n_localitat:' || n_localitat || ', ' || 'n_cp:' || n_cp || ', ' || 'n_pais' || n_pais || ', ' ||
       'n_estat:' || n_estat, rst);
   COMMIT;
   
EXCEPTION
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps nuls que no haurien de ser nuls';
      p_insertLog('p_altaCentralDistribucio', 'n_maxim_energia: ' || n_maxim_energia || ', ' || 'n_adrea:' || n_adrea || ', ' || 
                  'n_localitat:' || n_localitat || ', ' || 'n_cp:' || n_cp || ', ' || 'n_pais' || n_pais || ', ' ||
         'n_estat:' || n_estat, rst);
      ROLLBACK;  
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
    p_insertLog('p_altaCentralDistribucio', 'n_maxim_energia: ' || n_maxim_energia || ', ' || 'n_adrea:' || n_adrea || ', ' || 
                'n_localitat:' || n_localitat || ', ' || 'n_cp:' || n_cp || ', ' || 'n_pais' || n_pais || ', ' ||
       'n_estat:' || n_estat, rst);

   ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      p_insertLog('p_altaCentralDistribucio', 'n_maxim_energia: ' || n_maxim_energia || ', ' || 'n_adrea:' || n_adrea || ', ' || 
                  'n_localitat:' || n_localitat || ', ' || 'n_cp:' || n_cp || ', ' || 'n_pais' || n_pais || ', ' ||
         'n_estat:' || n_estat, rst);
   ROLLBACK;

END p_altaCentralDistribucio;
/

