CREATE OR REPLACE PROCEDURE         p_altaCentralProduccio (
   n_energia_maxima     IN NUMBER,
   n_data_inspeccio     IN DATE,
   n_capacitat_maxima   IN NUMBER, 
   n_adrea             IN VARCHAR2,
   n_localitat          IN VARCHAR2,
   n_cp                 IN VARCHAR2,
   n_pais               IN VARCHAR2, 
   n_estat       IN VARCHAR2
)
IS
--variables locals--

   n_adrea_ins                   VARCHAR (100 CHAR);
   n_localitat_ins                VARCHAR (100 CHAR);
   n_cp_ins              VARCHAR (5);
   n_pais_ins           VARCHAR (50 CHAR);
   rst                            VARCHAR2(2000);
   err_msg                        VARCHAR2(100);
   campsBuits                     EXCEPTION;
   noExisteixDataInspeccio        EXCEPTION;
   errorEscriptura      EXCEPTION;


BEGIN

   n_adrea_ins := SUBSTR (n_adrea, 1, 100); 
   n_localitat_ins := SUBSTR (n_localitat, 1, 100);
   n_cp_ins := SUBSTR (n_cp, 1, 5);
   n_pais_ins  := SUBSTR (n_pais, 1, 50);
  


   --verifiquem que els camps no siguin nuls
   IF (n_data_inspeccio IS NULL) OR
   (n_capacitat_maxima IS NULL) OR
   (n_adrea IS NULL) OR
   (n_localitat IS NULL) OR
   (n_pais IS NULL) OR
   (n_estat IS NULL)
   THEN
       RAISE campsBuits;
   END IF; 

     
   -- Verifiquem que la data_inspeccio existeix (no pot ser superior  a la data del sistema(sysdate))
   IF (n_data_inspeccio > sysdate) THEN 
     RAISE noExisteixDataInspeccio;
   END IF; 
  
   -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;
  
   -- inserim  la nova central de producci
   INSERT INTO CENTRAL_PRODUCCIO (energia_maxima, data_inspeccio, capacitat_maxima,  adrea, localitat, cp, pais, estat) 
         VALUES (n_energia_maxima, n_data_inspeccio, n_capacitat_maxima, n_adrea, n_localitat, n_cp, n_pais, lower(n_estat));
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit la central de producci correctament';
   
    p_insertLog('p_altaCentralProduccio',  
             'n_energia_maxima: ' || n_energia_maxima || ', ' ||
    'n_data_inspeccio: ' || n_data_inspeccio || ', ' ||
    'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais || ', ' ||
    'n_estat: ' || n_estat, rst);
   COMMIT;
   
   EXCEPTION
    
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
    p_insertLog('p_altaCentralProduccio',  
             'n_energia_maxima: ' || n_energia_maxima || ', ' ||
    'n_data_inspeccio: ' || n_data_inspeccio || ', ' ||
    'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;  
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
    p_insertLog('p_altaCentralProduccio',  
             'n_energia_maxima: ' || n_energia_maxima || ', ' ||
    'n_data_inspeccio: ' || n_data_inspeccio || ', ' ||
    'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;  
   
   WHEN noExisteixDataInspeccio
      THEN
      rst := 'ERROR: La data_inspeccio introduda no pot ser superior a la data actual';
   
    p_insertLog('p_altaCentralProduccio',  
             'n_energia_maxima: ' || n_energia_maxima || ', ' ||
    'n_data_inspeccio: ' || n_data_inspeccio || ', ' ||
    'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:' || err_msg;
    
    p_insertLog('p_altaCentralProduccio',  
             'n_energia_maxima: ' || n_energia_maxima || ', ' ||
    'n_data_inspeccio: ' || n_data_inspeccio || ', ' ||
    'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;


END p_altaCentralProduccio;
/

