CREATE OR REPLACE PROCEDURE p_altaClient (
   n_nif       IN VARCHAR2,
   n_nom       IN VARCHAR2,
   n_cognoms   IN VARCHAR2,
   n_adrea    IN VARCHAR2,
   n_localitat IN VARCHAR2,
   n_cp        IN VARCHAR2,
   n_pais      IN VARCHAR2,
   n_telefon   IN VARCHAR2,
   n_estat     IN VARCHAR
)
IS
--variables locals--
  
     rst                        VARCHAR2(2000);
     nifClient_test             VARCHAR2(20);
     err_msg                    VARCHAR2(100);
     existeix_client            EXCEPTION;
     campsBuits                 EXCEPTION;
     errorEscriptura            EXCEPTION;
	 no_actiu                   EXCEPTION;

BEGIN
   --verifiquem que els camps no siguin nuls
   IF (n_nif IS NULL) OR (n_nom IS NULL) OR (n_cognoms IS NULL) OR (n_adrea IS NULL) OR (n_localitat IS NULL) OR 
      (n_cp IS NULL) OR (n_pais IS NULL) OR (n_telefon IS NULL) OR (n_estat IS NULL) THEN 
     RAISE campsBuits;
   END IF; 

  --verifiquem que no existeix el client
  SELECT COUNT (*)
     INTO nifClient_test
     FROM CLIENT
    WHERE nif = n_nif;

   IF (nifClient_test > 0)
   THEN
      RAISE existeix_client;
   END IF;
   
  
   -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;

     
   -- inserim el nou client
   INSERT INTO CLIENT (nif, nom, cognoms, adrea, localitat, cp, pais, telefon, estat) 
          VALUES (n_nif, n_nom, n_cognoms, n_adrea, n_localitat, n_cp, n_pais, n_telefon, lower(n_estat));
   
   -- Fi de les operacions
 
   rst := 'OK: S''ha donat d''alta el client';
   
   p_insertLog('p_altaClient', 'n_nif: ' || n_nif || ', ' ||
                               'n_nom: ' || n_nom || ', ' || 
                               'n_cognoms: ' || n_cognoms || ', ' ||  
          'n_adrea:' || n_adrea || ', ' || 
          'n_localitat:' || n_localitat || ', ' || 
                               'n_cp:' || n_cp || ', ' ||  
          'n_pais' || n_pais || ', ' || 
          'n_telefon:' || n_telefon || ', ' ||
          'n_estat: ' || n_estat, rst);
   COMMIT;
   
EXCEPTION
   WHEN existeix_client
   THEN
    rst := 'ERROR: El client ja existeix';
   p_insertLog('p_altaClient', 'n_nif: ' || n_nif || ', ' ||
                               'n_nom: ' || n_nom || ', ' || 
                               'n_cognoms: ' || n_cognoms || ', ' ||  
          'n_adrea:' || n_adrea || ', ' || 
          'n_localitat:' || n_localitat || ', ' || 
                               'n_cp:' || n_cp || ', ' ||  
          'n_pais' || n_pais || ', ' || 
          'n_telefon:' || n_telefon || ', ' ||
          'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
 
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
   p_insertLog('p_altaClient', 'n_nif: ' || n_nif || ', ' ||
                               'n_nom: ' || n_nom || ', ' || 
                               'n_cognoms: ' || n_cognoms || ', ' ||  
          'n_adrea:' || n_adrea || ', ' || 
          'n_localitat:' || n_localitat || ', ' || 
                               'n_cp:' || n_cp || ', ' ||  
          'n_pais' || n_pais || ', ' || 
          'n_telefon:' || n_telefon || ', ' ||
          'n_estat: ' || n_estat, rst);

   ROLLBACK;
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: hi ha camps buits que no poden ser nuls';
   p_insertLog('p_altaClient', 'n_nif: ' || n_nif || ', ' ||
                               'n_nom: ' || n_nom || ', ' || 
                               'n_cognoms: ' || n_cognoms || ', ' ||  
          'n_adrea:' || n_adrea || ', ' || 
          'n_localitat:' || n_localitat || ', ' || 
                               'n_cp:' || n_cp || ', ' ||  
          'n_pais' || n_pais || ', ' || 
          'n_telefon:' || n_telefon || ', ' ||
          'n_estat: ' || n_estat, rst);
      ROLLBACK;  
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   p_insertLog('p_altaClient', 'n_nif: ' || n_nif || ', ' ||
                               'n_nom: ' || n_nom || ', ' || 
                               'n_cognoms: ' || n_cognoms || ', ' ||  
          'n_adrea:' || n_adrea || ', ' || 
          'n_localitat:' || n_localitat || ', ' || 
                               'n_cp:' || n_cp || ', ' ||  
          'n_pais' || n_pais || ', ' || 
          'n_telefon:' || n_telefon || ', ' ||
          'n_estat: ' || n_estat, rst);
 
      ROLLBACK;
END p_altaClient;
/

