CREATE OR REPLACE PROCEDURE         p_altaComptador (
   n_cod_model       IN INTEGER,
   n_referencia      IN VARCHAR2,
   n_data_instal     IN DATE,
   n_consum_absolut  IN NUMBER,
   n_adrea          IN VARCHAR2,
   n_localitat       IN VARCHAR2, 
   n_cp              IN VARCHAR2,
   n_pais            IN VARCHAR2,
   n_estat           IN VARCHAR2
)
IS
--variables locals--

   rst                        VARCHAR2(2000);
   referencia_test            VARCHAR2(20);
   cod_model_test             INTEGER;
   err_msg                    VARCHAR2(100);
   existeix_referencia        EXCEPTION;
   campsBuits                 EXCEPTION;
   noExisteix_cod_model       EXCEPTION;
   noExisteixData             EXCEPTION;
   errorEscriptura            EXCEPTION;

BEGIN

   --verifiquem que els camps no siguin nuls
   IF (n_cod_model IS NULL) OR (n_referencia IS NULL) OR (n_adrea IS NULL) 
   OR (n_localitat IS NULL) OR (n_cp IS NULL) OR (n_pais IS NULL) OR (n_estat IS NULL) THEN
     RAISE campsBuits;
   END IF; 
   
    -- Verifiquem que el codi_model existeix a la taula MODEL_COMPTADOR
   
   SELECT COUNT (*)
     INTO cod_model_test
     FROM MODEL_COMPTADOR
   WHERE cod_model = n_cod_model;

   IF (cod_model_test = 0)
   THEN
      RAISE noExisteix_cod_model;
   END IF;

  --verifiquem que no existeix la referencia de comptador
    SELECT COUNT (*)
     INTO referencia_test 
     FROM COMPTADOR
    WHERE referencia = n_referencia;

   IF (referencia_test > 0)
   THEN
      RAISE existeix_referencia;
   END IF;

  
   -- Verifiquem que la data existeix (no pot ser superior  a la data del sistema(sysdate))
  IF (n_data_instal > sysdate) THEN 
    RAISE noExisteixData;
  END IF; 
  
   -- Verifiquem que el camp estat est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;
  
   -- inserim el nou comptador
  INSERT INTO COMPTADOR (cod_model, referencia, data_instal, consum_absolut, adrea, localitat, cp, pais, estat) 
         VALUES (n_cod_model, n_referencia, n_data_instal, n_consum_absolut, n_adrea, n_localitat, n_cp, n_pais, lower (n_estat));
   
   
   
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit comptador correctament';
   
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
                'n_data_instal: ' || n_data_instal || ', ' ||
                'n_consum_absolut: ' || n_consum_absolut || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
   COMMIT;
   EXCEPTION
   
      WHEN noExisteix_cod_model    
   THEN
      rst := 'ERROR: No existeix el codi del model';
   
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
                'n_data_instal: ' || n_data_instal || ', ' ||
                'n_consum_absolut: ' || n_consum_absolut || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);

      ROLLBACK; 
   
     WHEN existeix_referencia
     THEN
     rst := 'ERROR: La referencia del comptador ja existeix';
   
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
                'n_data_instal: ' || n_data_instal || ', ' ||
                'n_consum_absolut: ' || n_consum_absolut || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
      ROLLBACK;
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
                'n_data_instal: ' || n_data_instal || ', ' ||
                'n_consum_absolut: ' || n_consum_absolut || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
      ROLLBACK;  
   
   WHEN noExisteixData
   THEN
      rst := 'ERROR: La data d''installaci introduda no pot ser superior a la data actual';
   
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
                'n_data_instal: ' || n_data_instal || ', ' ||
                'n_consum_absolut: ' || n_consum_absolut || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);

      ROLLBACK;
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp estat no s''ha escrit correctament';
   
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
                'n_data_instal: ' || n_data_instal || ', ' ||
                'n_consum_absolut: ' || n_consum_absolut || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);

      ROLLBACK;
   
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
    
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
                'n_data_instal: ' || n_data_instal || ', ' ||
                'n_consum_absolut: ' || n_consum_absolut || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
      ROLLBACK;

END p_altaComptador;
/

