CREATE OR REPLACE PROCEDURE         p_altaConnectaComptador (

   n_cod_comptador         IN INTEGER,
   n_cod_c_distribucio     IN INTEGER,
   n_estat       IN VARCHAR2

)
IS
--variables locals-- 
   rst                            VARCHAR2(2000);
   cod_comptador_test             INTEGER;
   cod_c_distribucio_test         INTEGER;
   err_msg                        VARCHAR2(100);
   campsBuits                     EXCEPTION;
   noExisteix_cod_comptador       EXCEPTION;
   noExisteix_cod_c_distribucio   EXCEPTION;
   errorEscriptura      EXCEPTION;


BEGIN

   --verifiquem que els camps no siguin nuls
   IF (n_cod_comptador IS NULL) OR (n_cod_c_distribucio IS NULL) THEN
     RAISE campsBuits;
   END IF; 
   
    -- Verifiquem que el codi_comptador existeix a la taula COMPTADOR
   
   SELECT COUNT (*)
     INTO cod_comptador_test
     FROM COMPTADOR
   WHERE cod_comptador = n_cod_comptador;

   IF (cod_comptador_test = 0)
   THEN
      RAISE noExisteix_cod_comptador;
   END IF;

  --verifiquem que no existeix el cod_c_distribucio a la taula CENTRAL_DISTRIBUCIO
  
   SELECT COUNT (*)
     INTO cod_c_distribucio_test
     FROM CENTRAL_DISTRIBUCIO
   WHERE cod_c_distribucio = n_cod_c_distribucio;

   IF (cod_c_distribucio_test = 0)
   THEN
      RAISE noExisteix_cod_c_distribucio;
   END IF;
   
   -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;


   -- inserim la relacio entre centrals de distribuci i comptadors
  INSERT INTO CONNECTA_COMPTADOR (cod_comptador, cod_c_distribucio, estat) 
         VALUES (n_cod_comptador, n_cod_c_distribucio, n_estat);
   
   -- Fi de les operacions
 
   rst := 'OK: S''ha inserit la relaci correctament';
   
   p_insertLog('p_altaConnectaComptador',  
               'n_cod_comptador: ' || n_cod_comptador || ', ' ||
         'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
      'n_estat: ' || n_estat, rst);

   COMMIT;
   EXCEPTION
   
      WHEN noExisteix_cod_comptador    
   THEN
      rst := 'ERROR: No existeix el codi del comptador';
   
   p_insertLog('p_altaConnectaComptador',  
               'n_cod_comptador: ' || n_cod_comptador || ', ' ||
         'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
      'n_estat: ' || n_estat, rst);


      ROLLBACK; 
   
     WHEN noExisteix_cod_c_distribucio
     THEN
     rst := 'ERROR: No existeix el codi de la central de distribuci';
   
   p_insertLog('p_altaConnectaComptador',  
               'n_cod_comptador: ' || n_cod_comptador || ', ' ||
         'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
      'n_estat: ' || n_estat, rst);


      ROLLBACK;
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
   p_insertLog('p_altaConnectaComptador',  
               'n_cod_comptador: ' || n_cod_comptador || ', ' ||
         'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
      'n_estat: ' || n_estat, rst);


      ROLLBACK;  
   
 
   WHEN errorEscriptura
   THEN
        rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
   p_insertLog('p_altaConnectaComptador',  
               'n_cod_comptador: ' || n_cod_comptador || ', ' ||
         'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
      'n_estat: ' || n_estat, rst);


      ROLLBACK;    
   
 
   

   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   p_insertLog('p_altaConnectaComptador',  
               'n_cod_comptador: ' || n_cod_comptador || ', ' ||
         'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
      'n_estat: ' || n_estat, rst);

   ROLLBACK;


END p_altaConnectaComptador;
/

