CREATE OR REPLACE PROCEDURE p_altaContracte (
   n_cod_comptador   IN INTEGER,
   n_cod_client      IN INTEGER,
   n_data_alta       IN DATE,
   n_data_fi         IN DATE,
   n_potencia        IN NUMBER,
   n_estat    IN VARCHAR2
)
IS
--variables locals--

   rst                            VARCHAR2(2000);
   codi_comptador_test            INTEGER;
   codi_client_test               INTEGER;
   cod_comptador_test			  INTEGER;
   err_msg                        VARCHAR2(100);
   campsBuits                     EXCEPTION;
   noExisteix_cod_comptador       EXCEPTION;
   noExisteix_cod_client          EXCEPTION;
   noExisteixDataAlta             EXCEPTION;
   errorEscriptura                EXCEPTION;
   noDonarAlta					  EXCEPTION;


BEGIN

   --verifiquem que els camps no siguin nuls
   IF (n_cod_comptador IS NULL) OR 
   (n_cod_client IS NULL) OR
   (n_data_alta IS NULL) OR
   (n_potencia IS NULL) OR
   (n_estat IS NULL)
   THEN
       RAISE campsBuits;
   END IF; 

   -- Verifiquem que el codi_comptador existeix a la taula COMPTADOR   
   SELECT COUNT (*)
     INTO codi_comptador_test
     FROM COMPTADOR
   WHERE cod_comptador = n_cod_comptador;

   IF (codi_comptador_test = 0)
   THEN
      RAISE noExisteix_cod_comptador;
   END IF;
   
     -- Verifiquem que el codi_client existeix a la taula CLIENT  
   SELECT COUNT (*)
     INTO codi_client_test
     FROM CLIENT
   WHERE cod_client = n_cod_client;

   IF (codi_client_test = 0)
   THEN
      RAISE noExisteix_cod_client;
   END IF;
   
 
   -- Verifiquem que la data d'alta existeix (no pot ser superior  a la data del sistema(sysdate))
   IF (n_data_alta > sysdate) THEN 
     RAISE noExisteixDataAlta;
   END IF; 
   
   -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;
  
   
   -- No es pot assignar un comptador a ms d'un contracte actiu
    SELECT COUNT (*)
     INTO cod_comptador_test
     FROM CONTRACTE
     WHERE cod_comptador = n_cod_comptador AND
	       n_estat = 'actiu';

   IF (cod_comptador_test > 0) 
   THEN
      RAISE noDonarAlta;
   END IF;
   
  
   -- inserim el nou contracte
   INSERT INTO CONTRACTE (cod_comptador, cod_client, data_alta, data_fi, potencia, estat) 
         VALUES (n_cod_comptador, n_cod_client, n_data_alta, n_data_fi, n_potencia, lower(n_estat));
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit contracte correctament';
   
    p_insertLog('p_altaContracte',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
       'n_cod_client: ' || n_cod_client || ', ' ||
                'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
       'n_potencia: ' || n_potencia || ', ' ||
    'n_estat: ' || n_estat, rst);
   COMMIT;
   
  --- La data_alta d'un contracte ser la mateixa que la data_instal d'un comptador. 
  -- Actualitzem aquesta dada en la taula comptador.
  UPDATE COMPTADOR
  SET data_instal = n_data_alta
  where cod_comptador = n_cod_comptador;
    
   

   EXCEPTION
     WHEN noExisteix_cod_comptador
     THEN
     rst := 'ERROR: No exiteix el codi del comptador';
   
    p_insertLog('p_altaContracte',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
       'n_cod_client: ' || n_cod_client || ', ' ||
                'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
       'n_potencia: ' || n_potencia || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
  
  
       WHEN noDonarAlta
     THEN
     rst := 'ERROR: Un comptador no pot tenir ms d''un contracte';
   
    p_insertLog('p_altaContracte',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
       'n_cod_client: ' || n_cod_client || ', ' ||
                'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
       'n_potencia: ' || n_potencia || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
  
  
  
   WHEN noExisteix_cod_client
     THEN
     rst := 'ERROR: No exiteix el codi del client';
   
    p_insertLog('p_altaContracte',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
       'n_cod_client: ' || n_cod_client || ', ' ||
                'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
       'n_potencia: ' || n_potencia || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
    p_insertLog('p_altaContracte',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
       'n_cod_client: ' || n_cod_client || ', ' ||
                'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
       'n_potencia: ' || n_potencia || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;  
   
   WHEN noExisteixDataAlta
   THEN
      rst := 'ERROR: La data_alta introduda no pot ser superior a la data actual';
   
    p_insertLog('p_altaContracte',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
       'n_cod_client: ' || n_cod_client || ', ' ||
                'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
       'n_potencia: ' || n_potencia || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
      WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
    p_insertLog('p_altaContracte',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
       'n_cod_client: ' || n_cod_client || ', ' ||
                'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
       'n_potencia: ' || n_potencia || ', ' ||
    'n_estat: ' || n_estat, rst);

   ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:' || err_msg;
    
    p_insertLog('p_altaContracte',  
                'err_msg: ' || err_msg, rst);
      ROLLBACK;




END p_altaContracte;
/

