CREATE OR REPLACE PROCEDURE         p_altaEnergiaAbsoluta(
    n_cod_c_produccio IN INTEGER,
    n_data_prod       IN DATE,
    n_energia_prod    IN NUMBER,
 n_valid     IN VARCHAR

)
IS

--Variables locals

   rst                            VARCHAR2(2000);
   cod_c_produccio_test           INTEGER;
   err_msg                        VARCHAR2(100);
   existe                         INTEGER;
   n_data             DATE;
   eprod        NUMBER;
   n_anyo        NUMBER;
   campsBuits                     EXCEPTION;
   noExisteix_cod_c_produccio     EXCEPTION;
   noExisteixDataProd             EXCEPTION;
   datesDuplicades        EXCEPTION;
   errorValid       EXCEPTION;


BEGIN

 
   --verifiquem que els camps no siguin nuls
   IF (n_cod_c_produccio IS NULL) OR 
   (n_data_prod IS NULL) OR
   (n_energia_prod IS NULL) OR
   (n_valid IS NULL)
   THEN
       RAISE campsBuits;
   END IF; 

   -- Verifiquem que el cod_c_produccio existeix a la taula CENTRAL_PRODUCCIO  
   SELECT COUNT (*)
     INTO cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
   WHERE cod_c_produccio = n_cod_c_produccio;

   IF (cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
     
   -- Verifiquem que el camp 'valid' est escrit correctament: si o no

   IF LOWER (n_valid) NOT IN ('si', 'no') THEN
      RAISE errorValid;
   END IF;
   
   
   -- Verifiquem que la data_consum existeix (no pot ser superior  a la data del sistema(sysdate))
   IF (n_data_prod > sysdate) THEN 
     RAISE noExisteixDataProd;
   END IF; 
   
   -- No es pot introdur la mateixa data_prod pel mateix codi_c_produccio

   SELECT COUNT (*)
     INTO existe
     FROM ENERGIA_ABSOLUTA
   WHERE cod_c_produccio = n_cod_c_produccio AND data_prod = n_data_prod;

   IF (existe = 1)
   THEN
      RAISE datesDuplicades;
   END IF;
   
   

   -- inserim el registre
   INSERT INTO ENERGIA_ABSOLUTA (cod_c_produccio, data_prod, energia_prod, valid) 
         VALUES (n_cod_c_produccio, n_data_prod, n_energia_prod, lower (n_valid));
   
   
  ---------------------------------------------------------------------------------------
   --EST_NOMBRE_CENRAL_PRODUCCIO 
  
  --Obtenim el l'any de la prod de la taula energia_absoluta per EST_NOMBRE_CENRAL_PRODUCCIO 
  SELECT EXTRACT (YEAR FROM EM.DATA_PROD) into  n_anyo
  FROM ENERGIA_ABSOLUTA EM 
  WHERE EM.DATA_PROD =  n_data_prod  
  GROUP BY EM.DATA_PROD;

 
  --Cridem al procediment qu far tots els calculs de l'estadistica EST_PERCENTATGE_LINIES
 
       p_estNomCentPro (n_anyo);
 ----------------------------------------------------------------------------------------
   
   
   
   
   
   
   
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit el registre correctament';
   
    p_insertLog('p_altaEnergiaAbsoluta',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
       'n_data_prod: ' || n_data_prod || ', ' ||
                'n_energia_prod: ' || n_energia_prod || ', ' ||
    'n_valid: ' || n_valid, rst);
   COMMIT;
   
    
 
 -- Actualitzem el camp energia_maxima de la taula central_produccio
 --aquest camp ser la suma de les energies prduides al llarg de la vida de la central
 
 SELECT SUM (ENERGIA_PROD) INTO EPROD
 FROM ENERGIA_ABSOLUTA
 WHERE cod_c_produccio = n_cod_c_produccio and
       valid = 'si';
 
 UPDATE CENTRAL_PRODUCCIO
 SET ENERGIA_MAXIMA = EPROD
 WHERE cod_c_produccio = n_cod_c_produccio;
 
 
 
 
   
   EXCEPTION
     WHEN noExisteix_cod_c_produccio
     THEN
     rst := 'ERROR: No exiteix el codi de la central de produccio';
   
    p_insertLog('p_altaEnergiaAbsoluta',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
       'n_data_prod: ' || n_data_prod || ', ' ||
                'n_energia_prod: ' || n_energia_prod || ', ' ||
    'n_valid: ' || n_valid, rst);
      ROLLBACK;
  
  WHEN errorValid
  THEN
  rst := 'ERROR: El camp ''valid'' no s''ha informat correctament';
   
    p_insertLog('p_altaEnergiaAbsoluta',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
       'n_data_prod: ' || n_data_prod || ', ' ||
                'n_energia_prod: ' || n_energia_prod || ', ' ||
    'n_valid: ' || n_valid, rst);
      ROLLBACK;
  
  
  
  
     
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
    p_insertLog('p_altaEnergiaAbsoluta',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
       'n_data_prod: ' || n_data_prod || ', ' ||
                'n_energia_prod: ' || n_energia_prod || ', ' ||
    'n_valid: ' || n_valid, rst);
      ROLLBACK;  
   
   WHEN noExisteixDataProd
   THEN
      rst := 'ERROR: La data_prod introduda no pot ser superior a la data actual';
   
    p_insertLog('p_altaEnergiaAbsoluta',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
       'n_data_prod: ' || n_data_prod || ', ' ||
                'n_energia_prod: ' || n_energia_prod || ', ' ||
    'n_valid: ' || n_valid, rst);
      ROLLBACK;
   
  WHEN datesDuplicades
 THEN
     rst := 'ERROR: No es poden donar d''alta dos data_prod pel matix cod_c_produccio';
   
    p_insertLog('p_altaEnergiaAbsoluta',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
       'n_data_prod: ' || n_data_prod || ', ' ||
                'n_energia_prod: ' || n_energia_prod || ', ' ||
    'n_valid: ' || n_valid, rst);
    ROLLBACK;
 
 
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:' || err_msg;
    
    p_insertLog('p_altaEnergiaAbsoluta',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
       'n_data_prod: ' || n_data_prod || ', ' ||
                'n_energia_prod: ' || n_energia_prod || ', ' ||
    'n_valid: ' || n_valid, rst);
      ROLLBACK;


END p_altaEnergiaAbsoluta;
/

