CREATE OR REPLACE PROCEDURE         p_altaEolica (
   n_cod_c_produccio   IN INTEGER,
   n_num_molins        IN NUMBER
 
)
IS
--variables locals--

   rst                            VARCHAR2(2000);
   err_msg                        VARCHAR2(100);
   cod_c_produccio_test           INTEGER;
   campsBuits                     EXCEPTION;
   noExisteix_cod_c_produccio     EXCEPTION;


BEGIN


   --verifiquem que els camps no siguin nuls
   IF (n_cod_c_produccio IS NULL) OR 
   (n_num_molins IS NULL) 
   THEN
       RAISE campsBuits;
   END IF; 

  -- Verifiquem que el codi_c_produccio existeix a la taula CENTRAL_PRODUCCIO   
   SELECT COUNT (*)
     INTO cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
   WHERE cod_c_produccio = n_cod_c_produccio;

   IF (cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
  
   -- inserim  la central eolica
   INSERT INTO EOLICA (cod_c_produccio, num_molins) 
          VALUES (n_cod_c_produccio, n_num_molins);
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit la central eolica correctament';
   
    p_insertLog('p_altaEolica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_num_molins: ' || n_num_molins, rst);
   COMMIT;
   
   EXCEPTION
    
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
   
    p_insertLog('p_altaEolica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_num_molins: ' || n_num_molins, rst);
      ROLLBACK;  
   
   WHEN noExisteix_cod_c_produccio
      THEN
      rst := 'ERROR: El cod_c_produccio no existeix';
   
   
    p_insertLog('p_altaEolica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_num_molins: ' || n_num_molins, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:' || err_msg;
    
    p_insertLog('p_altaEolica',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_num_molins: ' || n_num_molins, rst);
      ROLLBACK;
   


END p_altaEolica;
/

