CREATE OR REPLACE PROCEDURE         p_altaLecturaConsum (
    n_cod_comptador  IN INTEGER,
    n_data_lectura   IN DATE,
    n_consum_mes     IN NUMBER,
    n_lectura        IN NUMBER,
    n_modalitat      IN VARCHAR2,
  n_estat          IN VARCHAR2
    
)
IS

--variables locals--

   rst                      VARCHAR2(2000);
   cod_comptador_test          INTEGER;
   err_msg                     VARCHAR2(100);
   lect                NUMBER;
   existe             INTEGER;
   n_data             DATE;
   n_data_duplicada            DATE;
   n_data_alta               DATE;
   compt_consum_abs      NUMBER;
   estat_comptador      VARCHAR2(10);
   n_anyo                      number; 
   campsBuits                  EXCEPTION;
   noExisteix_cod_comptador    EXCEPTION;
   noExisteixDataAlta          EXCEPTION;  
   errorEscriptura             EXCEPTION;
   datesDuplicades             EXCEPTION;
   dataLectura                 EXCEPTION;
   errorEscriptura2            EXCEPTION;
   noLectura       EXCEPTION;
   
   

BEGIN

--Verifiquem que els camps no siguin nuls

   IF (n_cod_comptador IS NULL) OR (n_data_lectura IS NULL) OR (n_lectura IS NULL)
      OR (n_modalitat IS NULL) OR (n_estat IS NULL) THEN 
     RAISE campsBuits;
   END IF; 

   
-- Verifiquem que el codi_comptador existeix a la taula COMPTADOR
   
   SELECT COUNT (*)
     INTO cod_comptador_test
     FROM COMPTADOR
   WHERE cod_comptador = n_cod_comptador;

   IF (cod_comptador_test = 0)
   THEN
      RAISE noExisteix_cod_comptador;
   END IF;
   
 
-- Verifiquem que el camp 'modalitat' est escrit correctament: telematica o presencial

   IF LOWER (n_modalitat) NOT IN ('telematica', 'presencial') THEN
      RAISE errorEscriptura;
   END IF;
   
 -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura2;
   END IF;
   
   
-- No es pot introdur una lectura que sigui anterior a al data d'alta de contracte

SELECT C.DATA_ALTA into n_data_alta
FROM CONTRACTE C   
WHERE cod_comptador=n_cod_comptador;

IF (n_data_lectura < n_data_alta) THEN
RAISE dataLectura;
END IF;


-- No es pot fer una lectura d'un comptador que estigui com 'inactiu'

SELECT estat into estat_comptador
FROM comptador
WHERE cod_comptador = n_cod_comptador;

IF (estat_comptador = 'inactiu') THEN
RAISE noLectura;
END IF;


   
--No es poden fer dos lectures en el mateix dia

 SELECT count(*)
 INTO existe
 FROM LECTURA_CONSUM 
 WHERE cod_comptador=n_cod_comptador AND DATA_LECTURA=n_data_lectura AND n_estat = 'actiu';

 IF (existe = 1) THEN
  RAISE datesDuplicades; 
 END IF;

 IF (existe = 0) THEN
 
 -- Calculem el consum_mes
 -- Guardem dintre la variable 'lect' la ultima data del mes
 
 
 SELECT lectura INTO lect FROM LECTURA_CONSUM  l
 WHERE l.data_lectura IN (SELECT MAX(data_lectura ) FROM LECTURA_CONSUM WHERE cod_comptador = n_cod_comptador) AND 
       cod_comptador=n_cod_comptador AND n_estat = 'actiu'; 
   
    
  
    
  
 -- Inserim la nova lectura
 
    INSERT INTO LECTURA_CONSUM (cod_comptador, data_lectura, consum_mes, lectura, modalitat, estat) 
           VALUES (n_cod_comptador, n_data_lectura, (n_lectura - lect), n_lectura, lower(n_modalitat), lower (n_estat));
  
  
  -----------------------------------------------------------------------------------------
  --EST_CONSUM_COMPTADOR: Cridem al procediment de l'estadisctica p_estConsumComptador--  
     p_estConsumComptador (n_cod_comptador, n_lectura, lect);
  ----------------------------------------------------------------------------------------
  
 
  ----------------------------------------------------------------------------------------
  --EST_VALOR_MITJA 
  --Cridem al procediment que far tots els clculs per l'estadistica EST_VALOR_MITJA

      p_altaEstValorMitja();  
  ----------------------------------------------------------------------------------------
 
   ---------------------------------------------------------------------------------------
   --EST_LINIA_CARREGADA
   --Cridem al procediment qu far tots els calculs de l'estadistica EST_LINIA CARREGADA
   
     p_estLiniaCarregada();
   ----------------------------------------------------------------------------------------
 
    ---------------------------------------------------------------------------------------
   --EST_CONSUM_MIG
   
   --Cridem al procediment qu far tots els calculs de l'estadistica EST_CONSUM_MIG
      p_estConsumMig();
   ----------------------------------------------------------------------------------------
 
  ---------------------------------------------------------------------------------------
   --EST_PERCENTATGE_LINIES   
  
  --Obtenim el l'any de la data_lectura de la taula lectura_consum per EST_PERCENTATGE_LINIES
  SELECT EXTRACT (YEAR FROM LC.DATA_LECTURA) INTO n_anyo
  FROM LECTURA_CONSUM LC
  WHERE LC.DATA_LECTURA = n_data_lectura
  GROUP BY LC.DATA_LECTURA;
 
  --Cridem al procediment qu far tots els calculs de l'estadistica EST_PERCENTATGE_LINIES
 
       p_estPercentatgeLinies (n_anyo);
 ----------------------------------------------------------------------------------------
 
 
  
 -- Fi de les operacions
  
    rst := 'OK: S''ha donat d''alta la lectura';
    
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);
    COMMIT;
 
END IF;

-- Actualitzem el camp consum_absolut de la taula comptador

   select sum (consum_mes) into compt_consum_abs
   from lectura_consum
   where cod_comptador = n_cod_comptador and estat = 'actiu';
   
   UPDATE COMPTADOR
   SET consum_absolut = compt_consum_abs
   where cod_comptador = n_cod_comptador;
   
 
 
 ---------------------------------------------------------------------------------------
   --EST_TOP_TEN
   --Cridem al procediment qu far tots els calculs de l'estadistica EST_TOP_TEN 
   
    p_estTopTen();  
 ---------------------------------------------------------------------------------------
 

EXCEPTION
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps nuls que no haurien de ser nuls';
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);
      ROLLBACK;  
   
   WHEN errorEscriptura  
   THEN
      rst := 'ERROR: El camp ''modalitat'' no s''ha escrit correctament';
   
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);

   ROLLBACK;
   
   WHEN noLectura 
   THEN
      rst := 'ERROR: no es pot fer una lectura d''un comptador inactiu';
   
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);

   ROLLBACK;
   
   
   
   
   WHEN errorEscriptura2  
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);

   ROLLBACK;
   
   
   
   WHEN noExisteix_cod_comptador
   THEN
       rst := 'ERROR: no existeix codi_comtpador';
    
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);

   ROLLBACK;
   
   WHEN dataLectura
   THEN
      rst := 'ERROR: La data_lectura no pot ser anterior a la data_alta del contracte';
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);
      ROLLBACK; 
   
   
   
   
   
 WHEN datesDuplicades
 THEN
     rst := 'ERROR: No es poden fer dues lectures en el mateix dia d''un mateix comptador';
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);
    ROLLBACK;  
 
 WHEN NO_DATA_FOUND 
  THEN
   
  INSERT INTO LECTURA_CONSUM (cod_comptador, data_lectura, consum_mes, lectura, modalitat, estat) 
        VALUES (n_cod_comptador, n_data_lectura, (n_lectura), n_lectura, lower(n_modalitat), lower(n_estat));
   
   -- Actualitzem el camp consum_absolut de la taula comptador
   select sum (consum_mes) into compt_consum_abs
   from lectura_consum
   where cod_comptador = n_cod_comptador;
   
   UPDATE COMPTADOR
   SET consum_absolut = compt_consum_abs
   where cod_comptador = n_cod_comptador;
   
    
 rst := 'OK: S''ha donat d''alta la lectura';
   
   
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);
  COMMIT;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
    
    p_insertLog('p_altaLecturaConsum', 
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
            'n_data_lectura:' || n_data_lectura || ', ' ||
          'n_consum_mes:' || n_consum_mes || ', ' ||
          'n_lectura:' || n_lectura || ', ' ||
          'n_modalitat:' || n_modalitat || ', ' ||
       'n_estat: ' || n_estat, rst);

   ROLLBACK;  

END p_altaLecturaConsum;
/

