CREATE OR REPLACE PROCEDURE         p_altaLinia (
  n_capacitat_maxima     IN NUMBER,
  n_estat                IN VARCHAR2

)
IS
--variables locals-- 
   rst                            VARCHAR2(2000);
   err_msg                        VARCHAR2(100);
   campsBuits                     EXCEPTION;
   errorEscriptura                EXCEPTION;
  

BEGIN

   -- Verifiquem que els camps no siguin nuls
   IF (n_capacitat_maxima IS NULL) OR (n_estat IS NULL) THEN
     RAISE campsBuits;
   END IF; 
   
  -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;
   -- Inserim la linia
  INSERT INTO LINIA (capacitat_maxima, estat) 
         VALUES (n_capacitat_maxima, lower(n_estat));
   
   -- Fi de les operacions
 
   rst := 'OK: S''ha inserit la linia correctament';
   
   p_insertLog('p_altaLinia', 'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||
               'n_estat: ' || n_estat, rst);

   COMMIT;
   
   EXCEPTION
     
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
        p_insertLog('p_altaLinia', 'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||
                    'n_estat: ' || n_estat, rst);
 
      ROLLBACK;  
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
   
       p_insertLog('p_altaLinia', 'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||
                   'n_estat: ' || n_estat, rst);

   ROLLBACK;
   

   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
    
   p_insertLog('p_altaLinia', 'n_capacitat_maxima: ' || n_capacitat_maxima, rst);
    
   ROLLBACK;

END p_altaLinia;
/

