CREATE OR REPLACE PROCEDURE         p_altaModelComptador (
   n_data_fab      IN DATE,
   n_referencia    IN VARCHAR2,
   n_empresa_fab   IN VARCHAR2,
   n_descripcio    IN VARCHAR2,
   n_adrea        IN VARCHAR2,
   n_localitat     IN VARCHAR2,
   n_cp            IN VARCHAR2,
   n_pais          IN VARCHAR2,
   n_telefon       IN VARCHAR2
)
IS
--variables locals--

     rst                         VARCHAR2(2000);
     referencia_test             VARCHAR2(20);
     err_msg                     VARCHAR2(100);
     existeix_referencia         EXCEPTION;
     campsBuits                  EXCEPTION;
     noExisteixData              EXCEPTION;

BEGIN
   --verifiquem que els camps no siguin nuls
   IF (n_data_fab IS NULL) OR (n_referencia IS NULL) OR (n_empresa_fab IS NULL) OR (n_adrea IS NULL) OR (n_localitat IS NULL) OR 
      (n_cp IS NULL) OR (n_pais IS NULL) OR (n_telefon IS NULL) THEN 
     RAISE campsBuits;
   END IF; 

  --verifiquem que no existeix  la referencia del model de comptador
    SELECT COUNT (*)
     INTO referencia_test 
     FROM MODEL_COMPTADOR
    WHERE referencia = n_referencia;

   IF (referencia_test > 0)
   THEN
      RAISE existeix_referencia;
   END IF;

   --verifiquem que la data existeix (no pot ser superior  a la data del sistema(sysdate))
  IF (n_data_fab > sysdate) THEN 
    RAISE noExisteixData;
  END IF; 
  
  
   -- inserim el nou model de comptador
  INSERT INTO MODEL_COMPTADOR (data_fab, referencia, empresa_fab, descripcio, adrea, localitat, cp, pais, telefon) 
          VALUES (n_data_fab, n_referencia, n_empresa_fab, n_descripcio, n_adrea, n_localitat, n_cp, n_pais, n_telefon);
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit model de comptador';
   
    p_insertLog('p_altaModelComptador',  
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_empresa_fab: ' || n_empresa_fab || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
    'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
   COMMIT;
   
EXCEPTION
   WHEN existeix_referencia
   THEN
      rst := 'ERROR: El codi de model de comptador ja existeix';
     p_insertLog('p_altaModelComptador',  
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_empresa_fab: ' || n_empresa_fab || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
    'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
      ROLLBACK;
  
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
     p_insertLog('p_altaModelComptador',  
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_empresa_fab: ' || n_empresa_fab || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
    'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
      ROLLBACK;  
   
   WHEN noExisteixData
   THEN
      rst := 'ERROR: La data introduda no pot ser superior a la data actual';
   
    p_insertLog('p_altaModelComptador',  
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_empresa_fab: ' || n_empresa_fab || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
    'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);

      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   p_insertLog('p_altaModelComptador',  
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_empresa_fab: ' || n_empresa_fab || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
    'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
      ROLLBACK;

END p_altaModelComptador;
/

