CREATE OR REPLACE PROCEDURE         p_altaNuclear (
   n_cod_c_produccio   IN INTEGER,
   n_energia_minima    IN NUMBER,
   n_rebuig            IN NUMBER
)
IS
--variables locals--

   rst                            VARCHAR2(2000);
   err_msg                        VARCHAR2(100);
   cod_c_produccio_test           INTEGER;
   campsBuits                     EXCEPTION;
   noExisteix_cod_c_produccio     EXCEPTION;


BEGIN


   --verifiquem que els camps no siguin nuls
   IF (n_cod_c_produccio IS NULL) OR 
   (n_energia_minima IS NULL) OR
   (n_rebuig IS NULL) 
   THEN
       RAISE campsBuits;
   END IF; 

  -- Verifiquem que el codi_c_produccio existeix a la taula CENTRAL_PRODUCCIO   
   SELECT COUNT (*)
     INTO cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
   WHERE cod_c_produccio = n_cod_c_produccio;

   IF (cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
  
   -- inserim  la central nuclear
   INSERT INTO NUCLEAR  (cod_c_produccio, energia_minima, rebuig) 
          VALUES (n_cod_c_produccio, n_energia_minima, n_rebuig);
   
   -- Fi de les operacions
 
   rst := 'OK: Inserit la central nuclear correctament';
   
    p_insertLog('p_altaNuclear',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_energia_minima: ' || n_energia_minima || ', ' ||
                'n_rebuig): ' || n_rebuig, rst);
   COMMIT;
   
   EXCEPTION
    
   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
   
    p_insertLog('p_altaNuclear',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_energia_minima: ' || n_energia_minima || ', ' ||
                'n_rebuig): ' || n_rebuig, rst);
      ROLLBACK;  
   
   WHEN noExisteix_cod_c_produccio
      THEN
      rst := 'ERROR: El codi_c_produccio no existeix';
   
    p_insertLog('p_altaNuclear',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_energia_minima: ' || n_energia_minima || ', ' ||
                'n_rebuig): ' || n_rebuig, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:' || err_msg;
    
    p_insertLog('p_altaNuclear',  
                'n_cod_c_produccio: ' || n_cod_c_produccio || ', ' ||
    'n_energia_minima: ' || n_energia_minima || ', ' ||
                'n_rebuig): ' || n_rebuig, rst);
      ROLLBACK;

END p_altaNuclear;
/

