CREATE OR REPLACE PROCEDURE         p_baixaAlimentaCentral (
       n_cod_c_distribucio  IN INTEGER,
       n_cod_linia          IN INTEGER,
       n_cod_c_produccio    IN INTEGER,
    n_estat    IN VARCHAR2
)
IS  

--Variables locals
   cod_c_distribucio_test               INTEGER;
   n_cod_linia_test                     INTEGER;   
   n_cod_c_produccio_test               INTEGER;
   err_msg                              VARCHAR2(100);
   rst                                  VARCHAR2(100);
   noExisteix_cod_c_distribucio         EXCEPTION;
   noExisteix_cod_linia                 EXCEPTION;
   noExisteix_cod_c_produccio           EXCEPTION;
   campsBuits                           EXCEPTION;
   
BEGIN

  --El codis de central distribucio, linia i central 
  --de producci no poden estar buits
  
  IF ( n_cod_c_distribucio IS NULL) OR (n_cod_linia IS NULL) OR (n_cod_c_produccio IS NULL)
  THEN 
    RAISE campsBuits;
  END IF; 
  
    --El codi de la central de distribuci ha d'existir
  SELECT COUNT (*)
     INTO cod_c_distribucio_test
     FROM CENTRAL_DISTRIBUCIO
     WHERE cod_c_distribucio = n_cod_c_distribucio;

   IF (cod_c_distribucio_test = 0)
   THEN
      RAISE noExisteix_cod_c_distribucio;
   END IF;
   
     --El codi de la linia ha d'existir
  SELECT COUNT (*)
     INTO n_cod_linia_test
     FROM LINIA
     WHERE cod_linia = n_cod_linia;

   IF (n_cod_linia_test = 0)
   THEN
      RAISE noExisteix_cod_linia;
   END IF;
   
     --El codi de la central de producci ha d'existir
  SELECT COUNT (*)
     INTO n_cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
     WHERE cod_c_produccio = n_cod_c_produccio;

   IF (n_cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
  
   --Esborrem la relaci
   DELETE FROM ALIMENTA_CENTRAL
   WHERE cod_c_distribucio = n_cod_c_distribucio AND
         cod_linia = n_cod_linia AND
         cod_c_produccio = n_cod_c_produccio;
   
   -- Fi de les operacions
   rst := 'OK: S''ha donat de baixa la relaci';

    p_insertLog('p_baixaAlimentaCentral', 
               'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
               'n_cod_linia: ' || n_cod_linia || ', ' ||
               'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_c_distribucio
   THEN
      rst := 'ERROR: El codi de la central de distribuci no existeix';
      
    p_insertLog('p_baixaAlimentaCentral', 
             'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
    'n_cod_linia: ' || n_cod_linia || ', ' ||
             'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;
   
   WHEN noExisteix_cod_linia
   THEN
      rst := 'ERROR: El codi de la linia no existeix';
      
    p_insertLog('p_baixaAlimentaCentral', 
             'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
    'n_cod_linia: ' || n_cod_linia || ', ' ||
             'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
 ROLLBACK;
 
   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de produccio no existeix';
      
    p_insertLog('p_baixaAlimentaCentral', 
             'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
    'n_cod_linia: ' || n_cod_linia || ', ' ||
             'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
 ROLLBACK;
       
   WHEN campsBuits
   THEN
      rst := 'ERROR: Els camps buits no poden estar buits';
      
    p_insertLog('p_baixaAlimentaCentral', 
             'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
    'n_cod_linia: ' || n_cod_linia || ', ' ||
             'n_cod_c_produccio: ' || n_cod_c_produccio, rst);
      ROLLBACK;
 

END p_baixaAlimentaCentral;
/

