CREATE OR REPLACE PROCEDURE p_baixaCentralDistribucio (
    n_cod_c_distribucio  IN INTEGER,
 n_estat              IN VARCHAR2  
)
IS  

--Variables locals
   n_cod_c_distribucio_test        INTEGER;
   err_msg                         VARCHAR2(100);
   rst                             VARCHAR2(100);
   noExisteix_cod_c_distribucio    EXCEPTION;
   esNul_cod_c_distribucio         EXCEPTION;
   errorEscriptura                 EXCEPTION;
   no_actiu						   EXCEPTION;
   
BEGIN

  --Verifiquem que el cod_c_distribucio no sigui nul
  IF ( n_cod_c_distribucio  IS NULL) 
  THEN 
    RAISE esNul_cod_c_distribucio;
  END IF; 
  
  --El codi de la central de distribuci ha d'existir
  SELECT COUNT (*)
     INTO n_cod_c_distribucio_test
     FROM CENTRAL_DISTRIBUCIO
     WHERE cod_c_distribucio = n_cod_c_distribucio;

   IF (n_cod_c_distribucio_test = 0)
   THEN
      RAISE noExisteix_cod_c_distribucio;
   END IF;
   
   --No es pot activar un central de distirbucio inactiva
   --en aquest cas s'hauria de fer una alta nova amb un nou codi de central
   
   IF (n_estat = 'actiu') THEN
   RAISE no_actiu;
   END IF;
  
   -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('actiu', 'inactiu') THEN
      RAISE errorEscriptura;
   END IF;
  
  
   --Fem una baixa lgica de central de distribuci actualitzant
   --el camp estat d'actiu a inactiu
   UPDATE CENTRAL_DISTRIBUCIO
   SET estat = n_estat
   WHERE cod_c_distribucio = n_cod_c_distribucio;
   
   -- Fi de les operacions
   rst := 'OK: La central de distribuc est inactiva';

   p_insertLog('p_baixaCentralDistribucio', 'n_cod_c_distribucio: ' || n_cod_c_distribucio ||','||
                                            'n_estat: ' || n_estat, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_c_distribucio
   THEN
      rst := 'ERROR: El codi de la central de distribuci no existeix';
      
   p_insertLog('p_baixaCentralDistribucio', 'n_cod_c_distribucio: ' || n_cod_c_distribucio ||','||
                                            'n_estat: ' || n_estat, rst);
      ROLLBACK;
	  
   WHEN no_actiu
   THEN
      rst := 'ERROR: No es pot activar una central de distribuci inactiva';
      
   p_insertLog('p_baixaCentralDistribucio', 'n_cod_c_distribucio: ' || n_cod_c_distribucio ||','||
                                            'n_estat: ' || n_estat, rst);
      ROLLBACK;	  
   
   WHEN esNul_cod_c_distribucio
   THEN
      rst := 'ERROR: El codi de la central de distribuci no pot ser nul';
      
   p_insertLog('p_baixaCentralDistribucio', 'n_cod_c_distribucio: ' || n_cod_c_distribucio ||','||
                                            'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
      
   p_insertLog('p_baixaCentralDistribucio', 'n_cod_c_distribucio: ' || n_cod_c_distribucio ||','||
                                            'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_baixaCentralDistribucio', 'n_cod_c_distribucio: ' || n_cod_c_distribucio ||','||
                                            'n_estat: ' || n_estat, rst);
      ROLLBACK;




END p_baixaCentralDistribucio;
/

