CREATE OR REPLACE PROCEDURE p_baixaCentralProduccio (
    n_cod_c_produccio  IN INTEGER,
 n_estat      IN VARCHAR2  
)
IS  

--Variables locals
   n_cod_c_produccio_test          INTEGER;
   err_msg                         VARCHAR2(100);
   rst                             VARCHAR2(100);
   noExisteix_cod_c_produccio      EXCEPTION;
   esNul_cod_c_produccio           EXCEPTION;
   errorEscriptura                 EXCEPTION;
   no_actiu						   EXCEPTION;
   
BEGIN

  --Verifiquem que el ccod_c_produccio no sigui nul
  IF ( n_cod_c_produccio  IS NULL) 
  THEN 
    RAISE esNul_cod_c_produccio;
  END IF; 
  
  --El codi de la central de producci ha d'existir
  SELECT COUNT (*)
     INTO n_cod_c_produccio_test
     FROM CENTRAL_PRODUCCIO
     WHERE cod_c_produccio = n_cod_c_produccio;

   IF (n_cod_c_produccio_test = 0)
   THEN
      RAISE noExisteix_cod_c_produccio;
   END IF;
   
   	-- Una central de produccio no pot passar d'inactiva a activa
	--en aquest cas, s'haur de fer una alta nova i tenir un altre codi
	
	IF (n_estat = 'actiu') THEN
	RAISE no_actiu;
	END IF;  
	
    -- Verifiquem que el camp 'estat' est escrit correctament: actiu o inactiu
   IF LOWER (n_estat) NOT IN ('inactiu') THEN
      RAISE errorEscriptura;
   END IF;
        
   --Fem la baixa logica de la central de produccio
   UPDATE CENTRAL_PRODUCCIO
   SET estat = n_estat
   WHERE cod_c_produccio = n_cod_c_produccio;
   
   --Si s'ha fet una baixa d'una central de produccio, les seves
   --produccions d'energies hauran d'estar inactives, per tant el camp valid
   --de la taula energia_Absoluta estar com a 'no'
   
   
  IF (n_estat = 'inactiu') then  
    UPDATE ENERGIA_ABSOLUTA
  SET valid = 'no'
  WHERE cod_c_produccio = n_cod_c_produccio;
  END IF;
   
   -- Fi de les operacions
   rst := 'OK: La central de producci est inactiva';

         p_insertLog('p_baixaCentralProduccio', 'n_cod_c_produccio: ' || n_cod_c_produccio ||','||
                'n_estat: ' || n_estat, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de producci no existeix';
      
       p_insertLog('p_baixaCentralProduccio', 'n_cod_c_produccio: ' || n_cod_c_produccio ||','||
                'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN esNul_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de producci no pot ser nul';
      
              p_insertLog('p_baixaCentralProduccio', 'n_cod_c_produccio: ' || n_cod_c_produccio ||','||
                'n_estat: ' || n_estat, rst);
      ROLLBACK;
	  
   WHEN no_actiu
   THEN
      rst := 'ERROR: No es pot activar una central de producci inactiva';
      
              p_insertLog('p_baixaCentralProduccio', 'n_cod_c_produccio: ' || n_cod_c_produccio ||','||
                'n_estat: ' || n_estat, rst);
   ROLLBACK;
   
   WHEN errorEscriptura
   THEN
      rst := 'ERROR: El camp ''estat'' no s''ha escrit correctament';
      
       p_insertLog('p_baixaCentralProduccio', 'n_cod_c_produccio: ' || n_cod_c_produccio ||','||
                'n_estat: ' || n_estat, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
       p_insertLog('p_baixaCentralProduccio', 'n_cod_c_produccio: ' || n_cod_c_produccio ||','||
                'n_estat: ' || n_estat, rst);
      ROLLBACK;

END p_baixaCentralProduccio;
/

