CREATE OR REPLACE PROCEDURE         p_baixaConnectaComptador (
     n_cod_comptador       IN INTEGER,
     n_cod_c_distribucio   IN INTEGER,
  n_estat      IN VARCHAR2
  
)
IS  

--Variables locals
   n_cod_comptador_test               INTEGER;
   n_cod_c_distribucio_test           INTEGER;   
   err_msg                            VARCHAR2(100);
   rst                                VARCHAR2(100);
   noExisteix_cod_comptador           EXCEPTION;
   esNul_cod_comptador                EXCEPTION;
   noExisteix_cod_c_distribucio       EXCEPTION;
   esNul_cod_c_distribucio            EXCEPTION;
   
BEGIN

  --El cod_comptador no pot ser nuls
  IF ( n_cod_comptador IS NULL) 
  THEN 
    RAISE esNul_cod_comptador;
  END IF; 
  
  --El cod_c_distribucio  no pot ser nul
  IF ( n_cod_c_distribucio IS NULL) 
  THEN 
    RAISE esNul_cod_c_distribucio;
  END IF; 
  
  --El codi del comtpador ha d'existir
  SELECT COUNT (*)
     INTO n_cod_comptador_test
     FROM COMPTADOR
     WHERE cod_comptador = n_cod_comptador;

   IF (n_cod_comptador_test = 0)
   THEN
      RAISE noExisteix_cod_comptador;
   END IF;
   
  --El codi de la central de distribuci ha d'existir
  SELECT COUNT (*)
     INTO n_cod_c_distribucio_test
     FROM CENTRAL_DISTRIBUCIO
     WHERE cod_c_distribucio = n_cod_c_distribucio;

   IF (n_cod_c_distribucio_test = 0)
   THEN
      RAISE noExisteix_cod_c_distribucio;
   END IF;
  
   --Esborrem la relaci
   DELETE FROM CONNECTA_COMPTADOR 
   WHERE cod_comptador = n_cod_comptador AND
         cod_c_distribucio = n_cod_c_distribucio;
   
   -- Fi de les operacions
   rst := 'OK: S''ha donat de baixa la relaci';

    p_insertLog(' p_baixaConnectaComptador', 'n_cod_comptador: ' || n_cod_comptador || ', ' ||
               'n_cod_c_distribucio: ' || n_cod_c_distribucio , rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_comptador
   THEN
      rst := 'ERROR: El codi del comptador no existeix';
      
       p_insertLog('p_baixaConnectaComptador', 'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_c_distribucio: ' || n_cod_c_distribucio , rst);
      ROLLBACK;
   
   WHEN noExisteix_cod_c_distribucio
   THEN
      rst := 'ERROR: El codi de la central de distribucio no existeix';
      
       p_insertLog('p_baixaConnectaComptador', 'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_c_distribucio: ' || n_cod_c_distribucio , rst);
       
   WHEN esNul_cod_comptador
   THEN
      rst := 'ERROR: El codi del comptador s nul';
      
      p_insertLog('p_baixaConnectaComptador', 'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_c_distribucio: ' || n_cod_c_distribucio , rst);
      ROLLBACK;
   
   WHEN esNul_cod_c_distribucio
   THEN
      rst := 'ERROR: El codi de la central de distribucio s nul';
      
      p_insertLog('p_baixaConnectaComptador', 'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_c_distribucio: ' || n_cod_c_distribucio , rst);
    ROLLBACK;
    
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
      p_insertLog('p_baixaConnectaComptador', 'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_c_distribucio: ' || n_cod_c_distribucio , rst);
      ROLLBACK;

END p_baixaConnectaComptador;
/

