CREATE OR REPLACE PROCEDURE         p_baixaEnergiaAbsoluta (
     n_cod_ene_absoluta  IN INTEGER, 
       n_valid             IN VARCHAR2
)
IS

--Variables locals
   n_cod_ene_absoluta_test         INTEGER;
   err_msg                         VARCHAR2(100);
   rst                    VARCHAR2(100);
   n_data_prod        DATE;
   n_anyo         NUMBER;
   noExisteix_cod_ene_absoluta     EXCEPTION;
   esNul_cod_ene_absoluta          EXCEPTION;
   
BEGIN

  --Verifiquem que el cod_ene_absoluta no sigui nul
  IF ( n_cod_ene_absoluta  IS NULL) 
  THEN 
    RAISE esNul_cod_ene_absoluta;
  END IF; 
  
  --El codi de ENERGIA_ABSOLUTA ha d'existir
  SELECT COUNT (*)
     INTO n_cod_ene_absoluta_test
     FROM ENERGIA_ABSOLUTA
     WHERE cod_ene_absoluta = n_cod_ene_absoluta;

   IF (n_cod_ene_absoluta_test = 0)
   THEN
      RAISE noExisteix_cod_ene_absoluta;
   END IF;
  
   --Fem la baixa lgica del resigre
   UPDATE ENERGIA_ABSOLUTA
   SET valid  = n_valid            
   WHERE cod_ene_absoluta = n_cod_ene_absoluta;
   
   ----------------------------------------------------------------------------------------------
     --EST_NOMBRE_CENTRAL_PRODUCCIO
   
   --Hem de saber la data_prod de produccio que es donar de baixa per tal de poder
   --cridar al procediment p_modEstNomCentProduccio(n_anyo);
   
   SELECT EA.DATA_PROD INTO n_data_prod
   FROM ENERGIA_ABSOLUTA EA
   WHERE EA.COD_ENE_ABSOLUTA = n_cod_ene_absoluta;
   
     
    --Obtenim el l'any de la data_prod de la taula energia_absoluta per EST_NOMBRE_CENTRAL_PRODUCCIO
    SELECT EXTRACT (YEAR FROM EA.DATA_PROD) INTO n_anyo
    FROM ENERGIA_ABSOLUTA EA
    WHERE EA.DATA_PROD = n_data_prod
    GROUP BY EA.DATA_PROD;
 
    --Cridem al procediment qu far tots els calculs de l'estadistica EST_NOMBRE_CENTRAL_PRODUCCIO
 
       p_modEstNomCentProduccio(n_anyo);  
   
   ---------------------------------------------------------------------------------------------- 
   
   -- Fi de les operacions
   rst := 'OK: S''ha actualitzat el camp ''valid'' correctament';

   p_insertLog('p_baixaEnergiaAbsoluta', 'n_cod_ene_absoluta: ' || n_cod_ene_absoluta || ', ' ||
               'n_valid: ' || n_valid, rst);
   COMMIT;
   
EXCEPTION
   WHEN noExisteix_cod_ene_absoluta
   THEN
      rst := 'ERROR: El codi cod_ene_absoluta no existeix';
      
   p_insertLog('p_baixaEnergiaAbsoluta', 'n_cod_ene_absoluta: ' || n_cod_ene_absoluta || ', ' ||
               'n_valid: ' || n_valid, rst);
      ROLLBACK;
   
   WHEN esNul_cod_ene_absoluta
   THEN
      rst := 'ERROR: El codi de cod_ene_absoluta no pot ser nul';
      
   p_insertLog('p_baixaEnergiaAbsoluta', 'n_cod_ene_absoluta: ' || n_cod_ene_absoluta || ', ' ||
               'n_valid: ' || n_valid, rst);
      ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_baixaEnergiaAbsoluta', 'n_cod_ene_absoluta: ' || n_cod_ene_absoluta || ', ' ||
               'n_valid: ' || n_valid, rst);
      ROLLBACK;


   
END p_baixaEnergiaAbsoluta;
/

