CREATE OR REPLACE PROCEDURE         p_modCentralDistribucio (
    n_cod_c_distribucio   IN INTEGER,
    n_maxim_energia       IN NUMBER,
    n_adrea              IN VARCHAR2,
    n_localitat           IN VARCHAR2,
    n_cp                  IN VARCHAR2,
    n_pais                IN VARCHAR2

)
IS  

-- Variables locals
     n_maxim_energia_ins           NUMBER;
     n_adrea_ins                  VARCHAR2(100 CHAR);
     n_localitat_ins               VARCHAR2(100 CHAR); 
     n_cp_ins                      VARCHAR2(5 CHAR); 
     n_pais_ins                    VARCHAR2(50 CHAR);
     rst                           VARCHAR2(2000);
     cod_c_distribucio_test        INTEGER;
     err_msg                       VARCHAR2(100);
     noExisteix_cod_c_distribucio  EXCEPTION;
     existeixenCampsNuls           EXCEPTION;
  
  
   
BEGIN
   n_maxim_energia_ins := n_maxim_energia;
   n_adrea_ins := SUBSTR (n_adrea, 1, 100);
   n_localitat_ins  := SUBSTR (n_localitat, 1, 100);
   n_cp_ins := SUBSTR (n_cp, 1, 5);
   n_pais_ins := SUBSTR (n_pais, 1, 50);

   
  --verifiquem que els camps no nuls no siguin nuls
  IF (n_cod_c_distribucio IS NULL) 
     OR (n_maxim_energia IS NULL)
     OR (n_adrea IS NULL)
     OR (n_localitat IS NULL)
     OR (n_pais IS NULL)
  THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que la central de distribucio a modificar existeix
  SELECT COUNT (*)
     INTO cod_c_distribucio_test
     FROM CENTRAL_DISTRIBUCIO
     WHERE cod_c_distribucio = n_cod_c_distribucio;

   IF (cod_c_distribucio_test = 0)
   THEN
      RAISE noExisteix_cod_c_distribucio;
   END IF;
   

   
    -- modifiquem la central de distribucio
   UPDATE CENTRAL_DISTRIBUCIO
   SET maxim_energia = n_maxim_energia_ins, 
       adrea = n_adrea_ins, 
       localitat = n_localitat_ins, 
       cp = n_cp_ins, 
       pais = n_pais_ins
   WHERE cod_c_distribucio = n_cod_c_distribucio;

   
   -- Fi de les operacions
   rst := 'OK, central distribucio modificada correctament';

   p_insertLog('p_modCentralDistribucio',  
                'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
                'n_maxim_energia: ' || n_maxim_energia || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais, rst);
   COMMIT;
EXCEPTION
   WHEN noExisteix_cod_c_distribucio
   THEN
      rst := 'ERROR: El codi de la central de distriucio no existeix';
      
   p_insertLog('p_modCentralDistribucio',  
                'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
                'n_maxim_energia: ' || n_maxim_energia || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais, rst);
      ROLLBACK;
  
 
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
   p_insertLog('p_modCentralDistribucio',  
                'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
                'n_maxim_energia: ' || n_maxim_energia || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais, rst);
      ROLLBACK;
   
 
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modCentralDistribucio',  
                'n_cod_c_distribucio: ' || n_cod_c_distribucio || ', ' ||
                'n_maxim_energia: ' || n_maxim_energia || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais, rst);
      ROLLBACK;

END p_modCentralDistribucio;
/

