CREATE OR REPLACE PROCEDURE p_modCentralProduccio (

    n_cod_c_produccio     IN INTEGER,
	n_data_inspeccio      IN DATE,
    n_capacitat_maxima    IN NUMBER,    
    n_adrea              IN VARCHAR2,
    n_localitat           IN VARCHAR2,
    n_cp                  IN VARCHAR2,
    n_pais                IN VARCHAR2 
)
IS

-- Variables locals
   n_adrea_ins                   VARCHAR (100 CHAR);
   n_localitat_ins                VARCHAR (100 CHAR);
   n_cp_ins                       VARCHAR (5);
   n_pais_ins                     VARCHAR (50);
   rst                            VARCHAR2(2000);
   err_msg                        VARCHAR2(100);
   cod_c_produccio_test           INTEGER;
   campsBuits                     EXCEPTION;
   noExisteixDataInspeccio        EXCEPTION;
   noExisteix_cod_c_produccio     EXCEPTION;
  
   
BEGIN

   n_adrea_ins := SUBSTR (n_adrea, 1, 100); 
   n_localitat_ins := SUBSTR (n_localitat, 1, 100);
   n_cp_ins := SUBSTR (n_cp, 1, 5);
   n_pais_ins  := SUBSTR (n_pais, 1, 50);

   --verifiquem que els camps no siguin nuls
   IF (n_data_inspeccio IS NULL) OR
   (n_capacitat_maxima IS NULL) OR
   (n_adrea IS NULL) OR
   (n_localitat IS NULL) OR
   (n_pais IS NULL)

   THEN
       RAISE campsBuits;
   END IF;  
  
  --verifiquem que la central de produccio a modificar existeix
  SELECT COUNT (*)
  INTO cod_c_produccio_test
  FROM CENTRAL_PRODUCCIO
  WHERE cod_c_produccio = n_cod_c_produccio;

  IF (cod_c_produccio_test = 0)
   THEN
  RAISE noExisteix_cod_c_produccio;
  END IF;
  
  
   -- Verifiquem que la data_inspeccio existeix (no pot ser superior  a la data del sistema(sysdate))
   IF (n_data_inspeccio > sysdate) THEN 
     RAISE noExisteixDataInspeccio;
   END IF; 
     
   -- Modifiquem la central de produccio
   UPDATE CENTRAL_PRODUCCIO
   SET 
    data_inspeccio = n_data_inspeccio,
    capacitat_maxima = n_capacitat_maxima,
    adrea = n_adrea,
    localitat = n_localitat,
    cp = n_cp,
    pais = n_pais
    
   WHERE cod_c_produccio = n_cod_c_produccio;

   -- Fi de les operacions
   rst := 'OK, central de producci modificada correctament';

    p_insertLog('p_altaCentralProduccio', 
       'n_data_inspeccio: ' || n_data_inspeccio || ', ' || 
          'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais, rst);
   COMMIT;

   EXCEPTION

   WHEN noExisteix_cod_c_produccio
   THEN
      rst := 'ERROR: El codi de la central de proudccio no existeix';
      
    p_insertLog('p_altaCentralProduccio', 
       'n_data_inspeccio: ' || n_data_inspeccio || ', ' || 
          'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais, rst);
      ROLLBACK;
   

   WHEN campsBuits
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
    p_insertLog('p_altaCentralProduccio', 
       'n_data_inspeccio: ' || n_data_inspeccio || ', ' || 
          'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais, rst);
      ROLLBACK;
   
   WHEN noExisteixDataInspeccio
   THEN
      rst := 'ERROR: La data_inspeccio introduda no s correcte';
      
    p_insertLog('p_altaCentralProduccio', 
       'n_data_inspeccio: ' || n_data_inspeccio || ', ' || 
          'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais, rst);
   ROLLBACK;
   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
    p_insertLog('p_altaCentralProduccio', 
       'n_data_inspeccio: ' || n_data_inspeccio || ', ' || 
          'n_capacitat_maxima: ' || n_capacitat_maxima || ', ' ||                
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
       'n_cp: ' || n_cp || ', ' ||
       'n_pais: ' || n_pais, rst);
      ROLLBACK;

END p_modCentralProduccio;
/
