CREATE OR REPLACE PROCEDURE         p_modClient (
    n_cod_client   IN INTEGER,
    n_nif          IN VARCHAR2,
    n_nom          IN VARCHAR2,
    n_cognoms      IN VARCHAR2,
    n_adrea       IN VARCHAR2,
    n_localitat    IN VARCHAR2,
    n_cp           IN VARCHAR2,
    n_pais         IN VARCHAR2,
    n_telefon      IN VARCHAR2,
 n_estat     IN VARCHAR2
)
IS  

-- Variables locals
   n_nif_ins                     VARCHAR2(12 CHAR); 
   n_nom_ins                     VARCHAR2(15 CHAR); 
   n_cognoms_ins                 VARCHAR2(50 CHAR); 
   n_adrea_ins                  VARCHAR2(100 CHAR);
   n_localitat_ins               VARCHAR2(100 CHAR); 
   n_cp_ins                      VARCHAR2(5 CHAR); 
   n_pais_ins                    VARCHAR2(50 CHAR);
   n_telefon_ins                 VARCHAR2(20 CHAR);
   rst                           VARCHAR2(2000);
   cod_client_test               INTEGER;
   nif_test                      VARCHAR2(12 CHAR); 
   err_msg                       VARCHAR2(100);
   noExisteix_cod_client         EXCEPTION;
   existeixenCampsNuls           EXCEPTION;
  
   
BEGIN
   n_nif_ins := SUBSTR (n_nif, 1, 12);
   n_nom_ins := SUBSTR (n_nom, 1, 15);
   n_cognoms_ins := SUBSTR (n_cognoms, 1, 50);
   n_adrea_ins := SUBSTR (n_adrea, 1, 100);
   n_localitat_ins  := SUBSTR (n_localitat, 1, 100);
   n_cp_ins := SUBSTR (n_cp, 1, 5);
   n_pais_ins := SUBSTR (n_pais, 1, 50);
   n_telefon_ins := SUBSTR (n_telefon, 1, 20);
   
  --verifiquem que els camps no nuls no siguin nuls
  IF (n_nif IS NULL) 
     OR (n_nom IS NULL)
     OR (n_cognoms IS NULL)
     OR (n_adrea IS NULL)
     OR (n_localitat IS NULL)
     OR (n_cp IS NULL)
     OR (n_pais IS NULL)
     OR (n_telefon IS NULL)
  OR (n_estat IS NULL)
  THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que el client a modificar existeix
  SELECT COUNT (*)
     INTO cod_client_test
     FROM CLIENT
     WHERE cod_client = n_cod_client;

   IF (cod_client_test = 0)
   THEN
      RAISE noExisteix_cod_client;
   END IF;
   
    -- modifiquem el client
   UPDATE CLIENT
   SET nif = n_nif_ins, 
       nom = n_nom_ins,
       cognoms = n_cognoms_ins, 
       adrea = n_adrea_ins, 
       localitat = n_localitat_ins, 
       cp = n_cp_ins, 
       pais = n_pais_ins, 
       telefon = n_telefon_ins 
 WHERE cod_client = n_cod_client;


 
   -- Fin de las operaciones
   rst := 'OK, client modificat correctament';

   p_insertLog('p_modClient',  
                'n_cod_client: ' || n_cod_client || ', ' ||
                'n_nom: ' || n_nom || ', ' ||
                'n_cognoms: ' || n_cognoms || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ' ||
    'n_estat: ' || n_estat, rst);
   COMMIT;
EXCEPTION
   WHEN noExisteix_cod_client
   THEN
      rst := 'ERROR: El codi del client no existeix';
      
   p_insertLog('p_modClient',  
                'n_cod_client: ' || n_cod_client || ', ' ||
                'n_nom: ' || n_nom || ', ' ||
                'n_cognoms: ' || n_cognoms || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
  
 
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
   p_insertLog('p_modClient',  
                'n_cod_client: ' || n_cod_client || ', ' ||
                'n_nom: ' || n_nom || ', ' ||
                'n_cognoms: ' || n_cognoms || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modClient',  
                'n_cod_client: ' || n_cod_client || ', ' ||
                'n_nom: ' || n_nom || ', ' ||
                'n_cognoms: ' || n_cognoms || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ' ||
    'n_estat: ' || n_estat, rst);
      ROLLBACK;
END p_modClient;
/

