CREATE OR REPLACE PROCEDURE         p_modComptador (
 n_cod_comptador    IN INTEGER,  
 n_cod_model        IN INTEGER,
 n_referencia       IN VARCHAR2,
 n_adrea           IN VARCHAR2,
 n_localitat        IN VARCHAR2,
 n_cp               IN VARCHAR2,
 n_pais             IN VARCHAR2,
 n_estat            IN VARCHAR2
)
IS

-- Variables locals
   n_cod_model_ins               INTEGER;
   n_referencia_ins              VARCHAR2(20 CHAR);
   n_adrea_ins                  VARCHAR2(100 CHAR);
   n_localitat_ins               VARCHAR2(100 CHAR);
   n_cp_ins                      VARCHAR2 (5 CHAR);
   n_pais_ins                    VARCHAR2 (50 CHAR);
   n_estat_ins                   VARCHAR2 (7 CHAR);
   rst                           VARCHAR2(2000);
   cod_comptador_test            INTEGER;
   cod_model_test                INTEGER;
   err_msg                       VARCHAR2(100);
   noExisteix_cod_comptador      EXCEPTION;
   existeixenCampsNuls           EXCEPTION;
   noExisteix_cod_model          EXCEPTION;
  
   
BEGIN
   n_cod_model_ins := n_cod_model; 
   n_referencia_ins := n_referencia;
   n_referencia_ins  := SUBSTR (n_referencia, 1, 20);
   n_adrea_ins := SUBSTR (n_adrea, 1, 100);
   n_localitat_ins := SUBSTR (n_localitaT, 1, 100);
   n_cp_ins := SUBSTR (n_cp, 1, 5);
   n_pais_ins  := SUBSTR (n_pais, 1, 50);
   n_estat_ins := SUBSTR (n_estat, 1, 7);
   
  --verifiquem que els camps no nuls no siguin nuls
  IF (n_cod_model IS NULL) OR (n_estat IS NULL) OR (n_referencia IS NULL)THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que el comptador a modificar existeix
  SELECT COUNT (*)
     INTO cod_comptador_test
     FROM COMPTADOR
     WHERE cod_comptador = n_cod_comptador;

   IF (cod_comptador_test = 0)
   THEN
      RAISE noExisteix_cod_comptador;
   END IF;
   
    -- Verifiquem que el codi_model existeix a la taula MODEL_COMPTADOR
   
   SELECT COUNT (*)
     INTO cod_model_test
     FROM MODEL_COMPTADOR
   WHERE cod_model = n_cod_model;

   IF (cod_model_test = 0)
   THEN
      RAISE noExisteix_cod_model;
   END IF;
   
    -- modifiquem el comptador
   UPDATE COMPTADOR
   SET cod_model = n_cod_model_ins, 
       referencia = n_referencia_ins,
   -- consum_mes = n_consum_mes,
   -- consum_absolut = n_consum_absolut,
   -- darrera_lectura = n_darrera_lectura,
    adrea  = n_adrea_ins,
    localitat = n_localitat_ins, 
       cp = n_cp_ins, 
       pais = n_pais_ins, 
       estat = n_estat_ins    
    WHERE cod_comptador = n_cod_comptador;

   -- Fi de les operacions
   rst := 'OK, comptador modificat correctament';

   p_insertLog('p_modComptador',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
    --'consum_mes: ' || n_consum_mes || ', ' ||
    --'consum_absolut: ' || n_consum_absolut || ', ' ||
    --'darrera_lectura: ' || n_darrera_lectura || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
   COMMIT;
EXCEPTION
   WHEN noExisteix_cod_comptador
   THEN
      rst := 'ERROR: El codi del comptador no existeix';
      
   p_insertLog('p_modComptador',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
    --'consum_mes: ' || n_consum_mes || ', ' ||
    --'consum_absolut: ' || n_consum_absolut || ', ' ||
    --'darrera_lectura: ' || n_darrera_lectura || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
      ROLLBACK;
  
        WHEN noExisteix_cod_model    
   THEN
      rst := 'ERROR: No existeix el codi del model';
   
    p_insertLog('p_altaComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
               -- 'n_data: ' || n_data || ', ' ||
              --  'n_consum_mes: ' || n_consum_mes || ', ' ||
    --'n_consum_absolut: ' || n_consum_absolut || ', ' ||
    --'n_darrera_lectura: ' || n_darrera_lectura || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
    'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);

      ROLLBACK; 
 
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
   p_insertLog('p_modComptador',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
    --'consum_mes: ' || n_consum_mes || ', ' ||
    --'consum_absolut: ' || n_consum_absolut || ', ' ||
    --'darrera_lectura: ' || n_darrera_lectura || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
      ROLLBACK;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modComptador',  
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_referencia: ' || n_referencia || ', ' ||
    --'consum_mes: ' || n_consum_mes || ', ' ||
    --'consum_absolut: ' || n_consum_absolut || ', ' ||
    --'darrera_lectura: ' || n_darrera_lectura || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_estat: ' || n_estat || ', ', rst);
      ROLLBACK;

END p_modComptador;
/

