CREATE OR REPLACE PROCEDURE         p_modContracte (
    n_cod_contracte    IN INTEGER,
    n_cod_comptador    IN INTEGER,  
    n_cod_client       IN INTEGER,
    n_data_alta        IN DATE,
    n_data_fi          IN DATE,
    n_potencia         IN NUMBER

)
IS

-- Variables locals
     rst                             VARCHAR2(2000);
     cod_contracte_test              INTEGER;
     cod_comptador_test              INTEGER;
     cod_client_test                 INTEGER;
     err_msg                         VARCHAR2(100);
     existeixenCampsNuls             EXCEPTION;
     noExisteix_cod_contracte        EXCEPTION;
     noExisteix_cod_comptador        EXCEPTION;
     noExisteix_cod_client           EXCEPTION; 
     noExisteixDataFi                EXCEPTION;
     noExisteixDataAlta              EXCEPTION;
     datesIncorrectes                EXCEPTION;
  
   
BEGIN


  --verifiquem que els camps no nuls no siguin nuls
  IF (n_cod_comptador IS NULL) OR 
     (n_cod_client IS NULL) OR 
  (n_data_alta IS NULL) OR
  (n_potencia IS NULL) 
  THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que el contracte a modificar existeix
  SELECT COUNT (*)
  INTO cod_contracte_test
  FROM CONTRACTE
  WHERE cod_contracte = n_cod_contracte;

  IF (cod_contracte_test = 0)
   THEN
  RAISE noExisteix_cod_contracte;
  END IF;
   
  -- Verifiquem que el codi_comptador existeix en la taula COMPTADOR   
  SELECT COUNT (*)
  INTO cod_comptador_test
  FROM COMPTADOR
  WHERE cod_comptador = n_cod_comptador;

   IF (cod_comptador_test = 0)
   THEN
      RAISE noExisteix_cod_comptador;
   END IF;
   
    -- Verifiquem que el codi_client existeix en la taula CLIENT   
   SELECT COUNT (*)
     INTO cod_client_test
     FROM CLIENT
   WHERE cod_client = n_cod_client;

   IF (cod_client_test = 0)
   THEN
      RAISE noExisteix_cod_client;
   END IF;
   
     -- Verifiquem que la data_alta existeix (no pot ser superior  a la data del sistema(sysdate))
   IF (n_data_alta > sysdate) THEN 
     RAISE noExisteixDataAlta;
   END IF; 
     
   -- Verifiquem que la data_fi existeix (no pot ser superior  a la data del sistema(sysdate))
   IF (n_data_fi > sysdate) THEN 
     RAISE noExisteixDataFi;
   END IF; 
   
   -- Verifiquem que la data_fi del contracte s superior a la data_alta
   IF (n_data_fi <= n_data_alta) THEN
      RAISE datesIncorrectes;
   END IF;
   
    -- Modifiquem el contracte
   UPDATE CONTRACTE
   SET cod_comptador = n_cod_comptador,
       cod_client = n_cod_client,
    data_alta = n_data_alta,
    data_fi = n_data_fi,
    potencia = n_potencia        
   WHERE cod_contracte = n_cod_contracte;

   -- Fi de les operacions
   rst := 'OK, contracte modificat correctament';

   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
   COMMIT;

   EXCEPTION

   WHEN noExisteix_cod_contracte
   THEN
      rst := 'ERROR: El codi del contracte no existeix';
      
   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
      ROLLBACK;

   WHEN noExisteix_cod_comptador
   THEN
      rst := 'ERROR: El codi del comptador no existeix';
      
   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
      ROLLBACK;
  
        WHEN noExisteix_cod_client    
   THEN
      rst := 'ERROR: No existeix el codi del client';
   
   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
      ROLLBACK; 
 
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
      ROLLBACK;
   
   WHEN noExisteixDataAlta
   THEN
      rst := 'ERROR: La data_alta introduda no s correcte';
      
   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
   ROLLBACK;
   
   WHEN noExisteixDataFi
   THEN
      rst := 'ERROR: La data fi introduda no s correcte';
      
   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
   ROLLBACK;
   
   WHEN datesIncorrectes
   THEN
      rst := 'ERROR: La data_fi no pot ser inferior a la data_fi del contracte';
      
    p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
   ROLLBACK;
    
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
   p_insertLog('p_modContracte',  
                'n_cod_contracte: ' || n_cod_contracte || ', ' ||
                'n_cod_comptador: ' || n_cod_comptador || ', ' ||
                'n_cod_client: ' || n_cod_client || ', ' ||
    'n_data_alta: ' || n_data_alta || ', ' ||
                'n_data_fi: ' || n_data_fi || ', ' ||
                'n_potencia: ' || n_potencia, rst);
      ROLLBACK;


END p_modContracte;
/

