CREATE OR REPLACE PROCEDURE         p_modModelComptador (
   n_cod_model     IN INTEGER,
   n_data_fab      IN DATE,
   n_referencia    IN VARCHAR2,
   n_empresa_fab   IN VARCHAR2,
   n_descripcio    IN VARCHAR2,
   n_adrea        IN VARCHAR2,
   n_localitat     IN VARCHAR2,
   n_cp            IN VARCHAR2,
   n_pais          IN VARCHAR2,
   n_telefon       IN VARCHAR2
   
)
IS   


-- Variables locals

   n_data_fab_ins                DATE; 
   n_referencia_ins              VARCHAR2(20 CHAR); 
   n_empresa_fab_ins             VARCHAR2(100 CHAR); 
   n_descripcio_ins              VARCHAR (100 CHAR);
   n_adrea_ins                  VARCHAR2(100 CHAR);
   n_localitat_ins               VARCHAR2(100 CHAR); 
   n_cp_ins                      VARCHAR2(5 CHAR); 
   n_pais_ins                    VARCHAR2(50 CHAR);
   n_telefon_ins                 VARCHAR2(20 CHAR);
   rst                           VARCHAR2(2000);
   cod_model_test                INTEGER;
   referencia_test               VARCHAR2(20 CHAR); 
   err_msg                       VARCHAR2(100);
   noExisteix_cod_model          EXCEPTION;
   existeixenCampsNuls           EXCEPTION;
   
  
  BEGIN
   n_referencia_ins := SUBSTR (n_referencia, 1, 20);
   n_empresa_fab_ins := SUBSTR (n_empresa_fab, 1, 100);
   n_descripcio_ins := SUBSTR (n_descripcio, 1, 100);
   n_adrea_ins := SUBSTR (n_adrea, 1, 100);
   n_localitat_ins  := SUBSTR (n_localitat, 1, 100);
   n_cp_ins := SUBSTR (n_cp, 1, 96);
   n_pais_ins := SUBSTR (n_pais, 1, 50);
   n_telefon_ins := SUBSTR (n_telefon, 1, 20);
   
  --verifiquem que els camps no nuls no siguin nuls
  IF (n_data_fab IS NULL) 
     OR (n_referencia IS NULL)
     OR (n_empresa_fab IS NULL)
  OR (n_descripcio IS NULL)
     OR (n_adrea IS NULL)
     OR (n_localitat IS NULL)
     OR (n_cp IS NULL)
  OR (n_pais IS NULL)
  OR (n_telefon IS NULL)
  THEN 
    RAISE existeixenCampsNuls;
  END IF; 
  
  --verifiquem que el model a modificar existeix
  SELECT COUNT (*)
     INTO cod_model_test
     FROM MODEL_COMPTADOR
     WHERE cod_model = n_cod_model;

   IF (cod_model_test = 0)
   THEN
      RAISE noExisteix_cod_model;
   END IF;
   
  
    -- modifiquem el model
   UPDATE MODEL_COMPTADOR
   SET data_fab = n_data_fab, 
       referencia = n_referencia,
       empresa_fab = n_empresa_fab,
    descripcio = n_descripcio, 
       adrea = n_adrea_ins, 
       localitat = n_localitat_ins, 
       cp = n_cp_ins, 
       pais = n_pais_ins, 
       telefon = n_telefon_ins 
    WHERE cod_model = n_cod_model;

   -- Fi de les operacions
   rst := 'OK, model modificat correctament';

   p_insertLog('p_modModelComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
   COMMIT;
EXCEPTION
   WHEN noExisteix_cod_model
   THEN
      rst := 'ERROR: El codi del model no existeix';
      
        p_insertLog('p_modModelComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
      ROLLBACK;
  
  
   WHEN existeixenCampsNuls
   THEN
      rst := 'ERROR: Existeixen camps buits que no poden ser nuls';
      
          p_insertLog('p_modModelComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
      ROLLBACK;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
      
       p_insertLog('p_modModelComptador',  
                'n_cod_model: ' || n_cod_model || ', ' ||
                'n_data_fab: ' || n_data_fab || ', ' ||
    'n_referencia: ' || n_referencia || ', ' ||
                'n_descripcio: ' || n_descripcio || ', ' ||
                'n_adrea: ' || n_adrea || ', ' ||
                'n_localitat: ' || n_localitat || ', ' ||
                'n_cp: ' || n_cp || ', ' ||
                'n_pais: ' || n_pais || ', ' ||
                'n_telefon: ' || n_telefon || ', ', rst);
      ROLLBACK;

END p_modModelComptador;
/

